@@ -1,34 +1,8 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the qmake application of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "unixmake.h"
 #include "option.h"
-#include <qregexp.h>
 #include <qfile.h>
 #include <qhash.h>
 #include <qdir.h>
@@ -40,7 +14,7 @@ QT_BEGIN_NAMESPACE
 ProStringList UnixMakefileGenerator::libdirToFlags(const ProKey &key)
 {
     ProStringList results;
-    for (const auto &libdir : qAsConst(project->values(key))) {
+    for (const auto &libdir : std::as_const(project->values(key))) {
         if (!project->isEmpty("QMAKE_LFLAGS_RPATH") && project->isActiveConfig("rpath_libdirs"))
             project->values("QMAKE_LFLAGS") += var("QMAKE_LFLAGS_RPATH") + libdir;
         results.append("-L" + escapeFilePath(libdir));
@@ -74,7 +48,7 @@ UnixMakefileGenerator::init()
     for (const ProString &iif : project->values("QMAKE_INTERNAL_INCLUDED_FILES")) {
         if (iif == project->cacheFile())
             continue;
-        if (iif.startsWith(sroot) && iif.at(sroot.length()) == QLatin1Char('/'))
+        if (iif.startsWith(sroot) && iif.at(sroot.size()) == QLatin1Char('/'))
             project->values("DISTFILES") += fileFixify(iif.toQString(), FileFixifyRelative);
     }
 
@@ -127,7 +101,7 @@ UnixMakefileGenerator::init()
         const ProStringList &rpathdirs = project->values("QMAKE_RPATHDIR");
         for (int i = 0; i < rpathdirs.size(); ++i) {
             QString rpathdir = rpathdirs[i].toQString();
-            if (rpathdir.length() > 1 && rpathdir.at(0) == '$' && rpathdir.at(1) != '(') {
+            if (rpathdir.size() > 1 && rpathdir.at(0) == '$' && rpathdir.at(1) != '(') {
                 rpathdir.replace(0, 1, "\\$$");  // Escape from make and the shell
             } else if (!rpathdir.startsWith('@') && fileInfo(rpathdir).isRelative()) {
                 QString rpathbase = project->first("QMAKE_REL_RPATH_BASE").toQString();
@@ -199,9 +173,8 @@ UnixMakefileGenerator::init()
                 QString headerSuffix;
                 if (project->isActiveConfig("clang_pch_style"))
                     headerSuffix = project->first("QMAKE_PCH_OUTPUT_EXT").toQString();
-                else
-                    pchBaseName += project->first("QMAKE_PCH_OUTPUT_EXT").toQString();
 
+                pchBaseName += project->first("QMAKE_PCH_OUTPUT_EXT").toQString();
                 pchBaseName += Option::dir_sep;
 
                 ProString language = project->first(ProKey("QMAKE_LANGUAGE_" + compiler));
@@ -210,15 +183,15 @@ UnixMakefileGenerator::init()
                                      escapeFilePath(pchBaseName + language + headerSuffix));
                     const ProStringList pchArchs = project->values("QMAKE_PCH_ARCHS");
                     for (const ProString &arch : pchArchs) {
-                        QString suffix = headerSuffix;
-                        suffix.replace(QLatin1String("${QMAKE_PCH_ARCH}"), arch.toQString());
+                        QString file = pchBaseName + language + headerSuffix;
+                        file.replace(QLatin1String("${QMAKE_PCH_ARCH}"), arch.toQString());
                         if (project->isActiveConfig("clang_pch_style")
-                            && (suffix.endsWith(QLatin1String(".pch"))
-                                || suffix.endsWith(QLatin1String(".gch")))) {
-                            suffix.chop(4); // must omit header suffix for -include to recognize the PCH
+                            && (file.endsWith(QLatin1String(".pch"))
+                                || file.endsWith(QLatin1String(".gch")))) {
+                            file.chop(4); // must omit header suffix for -include to recognize the PCH
                         }
                         pchFlags.replace(QLatin1String("${QMAKE_PCH_OUTPUT_") + arch + QLatin1Char('}'),
-                                         escapeFilePath(pchBaseName + language + suffix));
+                                         escapeFilePath(file));
                     }
                 }
             }
@@ -268,7 +241,6 @@ UnixMakefileGenerator::init()
             if(project->isEmpty("QMAKE_BUNDLE_LOCATION"))
                 project->values("QMAKE_BUNDLE_LOCATION").append("Contents/MacOS");
             project->values("QMAKE_PKGINFO").append(project->first("DESTDIR") + bundle + "/Contents/PkgInfo");
-            project->values("QMAKE_BUNDLE_RESOURCE_FILE").append(project->first("DESTDIR") + bundle + "/Contents/Resources/empty.lproj");
         } else if(project->first("TEMPLATE") == "lib" && !project->isActiveConfig("staticlib") &&
                   ((!project->isActiveConfig("plugin") && project->isActiveConfig("lib_bundle")) ||
                    (project->isActiveConfig("plugin") && project->isActiveConfig("plugin_bundle")))) {
@@ -320,14 +292,13 @@ QStringList
         if(!project->isEmpty("PRECOMPILED_DIR"))
             header_prefix = project->first("PRECOMPILED_DIR").toQString();
         header_prefix += project->first("QMAKE_ORIG_TARGET").toQString();
-        if (!project->isActiveConfig("clang_pch_style"))
-            header_prefix += project->first("QMAKE_PCH_OUTPUT_EXT").toQString();
+        header_prefix += project->first("QMAKE_PCH_OUTPUT_EXT").toQString();
         if (project->isActiveConfig("icc_pch_style")) {
             // icc style
             ProStringList pchArchs = project->values("QMAKE_PCH_ARCHS");
             if (pchArchs.isEmpty())
                 pchArchs << ProString(); // normal single-arch PCH
-            for (const ProString &arch : qAsConst(pchArchs)) {
+            for (const ProString &arch : std::as_const(pchArchs)) {
                 auto pfx = header_prefix;
                 if (!arch.isEmpty())
                     pfx.replace(QLatin1String("${QMAKE_PCH_ARCH}"), arch.toQString());
@@ -365,11 +336,12 @@ QStringList
                     ProStringList pchArchs = project->values("QMAKE_PCH_ARCHS");
                     if (pchArchs.isEmpty())
                         pchArchs << ProString(); // normal single-arch PCH
-                    for (const ProString &arch : qAsConst(pchArchs)) {
-                        QString suffix = header_suffix;
-                        if (!arch.isEmpty())
-                            suffix.replace(QLatin1String("${QMAKE_PCH_ARCH}"), arch.toQString());
-                        QString precompiledHeader = header_prefix + language + suffix;
+                    for (const ProString &arch : std::as_const(pchArchs)) {
+                        QString precompiledHeader = header_prefix + language + header_suffix;
+                        if (!arch.isEmpty()) {
+                            precompiledHeader.replace(QLatin1String("${QMAKE_PCH_ARCH}"),
+                                                      arch.toQString());
+                        }
                         if (!ret.contains(precompiledHeader))
                             ret += precompiledHeader;
                     }
@@ -399,6 +371,8 @@ UnixMakefileGenerator::findLibraries(bool linkPrl, bool mergeLflags)
         libdirs.append(QMakeLocalFileName(dlib.toQString()));
     frameworkdirs.append(QMakeLocalFileName("/System/Library/Frameworks"));
     frameworkdirs.append(QMakeLocalFileName("/Library/Frameworks"));
+    ProStringList extens;
+    extens << project->first("QMAKE_EXTENSION_SHLIB") << "a";
     static const char * const lflags[] = { "LIBS", "LIBS_PRIVATE",
                                            "QMAKE_LIBS", "QMAKE_LIBS_PRIVATE", nullptr };
     for (int i = 0; lflags[i]; i++) {
@@ -417,11 +391,8 @@ UnixMakefileGenerator::findLibraries(bool linkPrl, bool mergeLflags)
                     libdirs.insert(libidx++, f);
                 } else if(opt.startsWith("-l")) {
                     QString lib = opt.mid(2);
-                    ProStringList extens;
-                    extens << project->first("QMAKE_EXTENSION_SHLIB") << "a";
-                    for (QList<QMakeLocalFileName>::Iterator dep_it = libdirs.begin();
-                         dep_it != libdirs.end(); ++dep_it) {
-                        QString libBase = (*dep_it).local() + '/'
+                    for (const QMakeLocalFileName &libdir : std::as_const(libdirs)) {
+                        QString libBase = libdir.local() + '/'
                                 + project->first("QMAKE_PREFIX_SHLIB") + lib;
                         if (linkPrl && processPrlFile(libBase, true))
                             goto found;
@@ -435,12 +406,9 @@ UnixMakefileGenerator::findLibraries(bool linkPrl, bool mergeLflags)
                     QMakeLocalFileName f(opt.mid(2));
                     if (!frameworkdirs.contains(f))
                         frameworkdirs.insert(fwidx++, f);
-                } else if (target_mode == TARG_MAC_MODE && opt.startsWith("-framework")) {
+                } else if (target_mode == TARG_MAC_MODE && opt == "-framework") {
                     if (linkPrl) {
-                        if (opt.length() == 10)
-                            opt = (*++it).toQString();
-                        else
-                            opt = opt.mid(10).trimmed();
+                        opt = (*++it).toQString();
                         static const QChar suffixMarker = ',';
                         const int suffixPosition = opt.indexOf(suffixMarker);
                         const bool hasSuffix = suffixPosition >= 0;
@@ -449,19 +417,25 @@ UnixMakefileGenerator::findLibraries(bool linkPrl, bool mergeLflags)
                             frameworkName.truncate(suffixPosition);
                             opt.remove(suffixMarker); // Apply suffix by removing marker
                         }
-                        for (const QMakeLocalFileName &dir : qAsConst(frameworkdirs)) {
+                        for (const QMakeLocalFileName &dir : std::as_const(frameworkdirs)) {
+                            auto processPrlIfFound = [&](QString directory) {
+                                QString suffixedPrl = directory + opt;
+                                if (processPrlFile(suffixedPrl, true))
+                                    return true;
+                                if (hasSuffix) {
+                                    QString unsuffixedPrl = directory + frameworkName;
+                                    if (processPrlFile(unsuffixedPrl, true))
+                                        return true;
+                                }
+                                return false;
+                            };
                             QString frameworkDirectory = dir.local() + "/" + frameworkName + + ".framework/";
-                            QString suffixedPrl = frameworkDirectory + opt;
-                            if (processPrlFile(suffixedPrl, true))
+                            if (processPrlIfFound(frameworkDirectory + "Resources/")
+                             || processPrlIfFound(frameworkDirectory))
                                 break;
-                            if (hasSuffix) {
-                                QString unsuffixedPrl = frameworkDirectory + frameworkName;
-                                if (processPrlFile(unsuffixedPrl, true))
-                                    break;
-                            }
                         }
                     } else {
-                        if (opt.length() == 10)
+                        if (opt.size() == 10)
                             ++it;
                         // Skip
                     }
@@ -498,31 +472,37 @@ UnixMakefileGenerator::findLibraries(bool linkPrl, bool mergeLflags)
                         // Make sure we keep the dependency order of libraries
                         lflags[arch].removeAll(opt);
                         lflags[arch].append(opt);
-                    } else if (target_mode == TARG_MAC_MODE && opt.startsWith("-framework")) {
-                        if (opt.length() > 10) {
-                            opt = opt.mid(10).trimmed();
-                        } else {
-                            opt = l.at(++lit);
-                            if (opt.startsWith("-Xarch"))
-                                opt = l.at(++lit); // The user has done the right thing and prefixed each part
-                        }
-                        bool found = false;
+                    } else if (target_mode == TARG_MAC_MODE
+                        && (opt == "-framework" || opt == "-force_load")) {
+                        // Handle space separated options
+                        ProString dashOpt = opt;
+                        opt = l.at(++lit);
+                        if (opt.startsWith("-Xarch"))
+                            opt = l.at(++lit); // The user has done the right thing and prefixed each part
                         for(int x = 0; x < lflags[arch].size(); ++x) {
-                            if (lflags[arch].at(x) == "-framework" && lflags[arch].at(++x) == opt) {
-                                found = true;
+                            if (lflags[arch].at(x) == dashOpt && lflags[arch].at(++x) == opt) {
+                                lflags[arch].remove(x - 1, 2);
                                 break;
                             }
                         }
-                        if(!found) {
-                            lflags[arch].append("-framework");
-                            lflags[arch].append(opt);
-                        }
+                        lflags[arch].append(dashOpt);
+                        lflags[arch].append(opt);
                     } else {
                         lflags[arch].append(opt);
                     }
                 } else if(!opt.isNull()) {
+                    for (const ProString &ext : extens) {
+                        if (opt.size() > ext.size() && opt.endsWith(ext)
+                                && opt.at(opt.size() - ext.size() - 1) == '.') {
+                            // Make sure we keep the dependency order of libraries
+                            lflags[arch].removeAll(opt);
+                            lflags[arch].append(opt);
+                            goto found2;
+                        }
+                    }
                     if(!lflags[arch].contains(opt))
                         lflags[arch].append(opt);
+                  found2: ;
                 }
             }
 
@@ -598,7 +578,7 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
                 dst = escapeFilePath(filePrefixRoot(root, targetdir + src.section('/', -1)));
         if(!ret.isEmpty())
             ret += "\n\t";
-        ret += "-$(QINSTALL) " + escapeFilePath(Option::fixPathToTargetOS(src, false)) + ' ' + dst;
+        ret += "$(QINSTALL) " + escapeFilePath(Option::fixPathToTargetOS(src, false)) + ' ' + dst;
         if(!uninst.isEmpty())
             uninst.append("\n\t");
         uninst.append("-$(DEL_FILE) " + dst);
@@ -634,16 +614,16 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
 
         QString copy_cmd;
         if (bundle == SolidBundle) {
-            copy_cmd += "-$(QINSTALL) " + src_targ + ' ' + plain_targ;
+            copy_cmd += "$(QINSTALL) " + src_targ + ' ' + plain_targ;
         } else if (project->first("TEMPLATE") == "lib" && project->isActiveConfig("staticlib")) {
-            copy_cmd += "-$(QINSTALL) " + src_targ + ' ' + dst_targ;
+            copy_cmd += "$(QINSTALL) " + src_targ + ' ' + dst_targ;
         } else if (!isAux) {
             if (bundle == SlicedBundle) {
                 if (!ret.isEmpty())
                     ret += "\n\t";
                 ret += mkdir_p_asstring("\"`dirname " + dst_targ + "`\"", false);
             }
-            copy_cmd += "-$(QINSTALL_PROGRAM) " + src_targ + ' ' + dst_targ;
+            copy_cmd += "$(QINSTALL_PROGRAM) " + src_targ + ' ' + dst_targ;
         }
         if(project->first("TEMPLATE") == "lib" && !project->isActiveConfig("staticlib")
            && project->values(ProKey(t + ".CONFIG")).indexOf("fix_rpath") != -1) {
@@ -696,7 +676,7 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
                     ret += "\n\t";
                 ret += mkdir_p_asstring("\"`dirname " + dst + "`\"", false) + "\n\t";
                 ret += "-$(DEL_FILE) " + dst + "\n\t"; // Can't overwrite symlinks to directories
-                ret += "-$(QINSTALL) " + escapeFilePath(src) + " " + dst;
+                ret += "$(QINSTALL) " + escapeFilePath(src) + " " + dst;
                 if (!uninst.isEmpty())
                     uninst.append("\n\t");
                 uninst.append("-$(DEL_FILE) " + dst);
@@ -708,7 +688,7 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
                     QString link = Option::fixPathToTargetOS(destdir + links[i], false);
                     int lslash = link.lastIndexOf(Option::dir_sep);
                     if(lslash != -1)
-                        link = link.right(link.length() - (lslash + 1));
+                        link = link.right(link.size() - (lslash + 1));
                     QString dst_link = escapeFilePath(
                                 filePrefixRoot(root, fileFixify(targetdir + link, FileFixifyAbsolute)));
                     ret += "\n\t-$(SYMLINK) $(TARGET) " + dst_link;
@@ -719,7 +699,7 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
             }
         }
     }
-    if(project->first("TEMPLATE") == "lib") {
+    if (isAux || project->first("TEMPLATE") == "lib") {
         QStringList types;
         types << "prl" << "libtool" << "pkgconfig";
         for(int i = 0; i < types.size(); ++i) {
