@@ -6,6 +6,7 @@ package org.qtproject.qt.android;
 import android.app.Activity;
 import android.content.Context;
 import android.graphics.Rect;
+import android.os.Build;
 import android.os.Bundle;
 import android.os.Handler;
 import android.os.ResultReceiver;
@@ -16,12 +17,15 @@ import android.view.InputDevice;
 import android.view.KeyCharacterMap;
 import android.view.KeyEvent;
 import android.view.MotionEvent;
+import android.view.WindowInsets;
+import android.view.WindowInsets.Type;
+import android.view.WindowInsetsAnimationController;
+import android.view.WindowInsetsAnimationControlListener;
 import android.view.WindowManager;
+import android.view.View;
+import android.view.ViewTreeObserver;
 import android.view.inputmethod.InputMethodManager;
 
-import org.qtproject.qt.android.QtInputConnection.QtInputConnectionListener;
-
-/** @noinspection FieldCanBeLocal*/
 class QtInputDelegate implements QtInputConnection.QtInputConnectionListener, QtInputInterface
 {
 
@@ -45,6 +49,11 @@ class QtInputDelegate implements QtInputConnection.QtInputConnectionListener, Qt
     private QtEditText m_currentEditText = null;
     private InputMethodManager m_imm;
 
+    // We can't rely on a hardcoded value, because screens have different resolutions.
+    // That is why we assume that the keyboard should be higher than 0.15 of the screen.
+    private static final float KEYBOARD_TO_SCREEN_RATIO = 0.15f;
+
+    private boolean m_keyboardTransitionInProgress = false;
     private boolean m_keyboardIsVisible = false;
     private boolean m_isKeyboardHidingAnimationOngoing = false;
     private long m_showHideTimeStamp = System.nanoTime();
@@ -80,6 +89,40 @@ class QtInputDelegate implements QtInputConnection.QtInputConnectionListener, Qt
     void initInputMethodManager(Activity activity)
     {
         m_imm = (InputMethodManager) activity.getSystemService(Context.INPUT_METHOD_SERVICE);
+        if (m_imm == null)
+            Log.w(TAG, "getSystemService() returned a null InputMethodManager instance");
+
+        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
+            View rootView = activity.getWindow().getDecorView();
+            rootView.setOnApplyWindowInsetsListener((view, insets) -> {
+                    WindowInsets windowInsets = view.onApplyWindowInsets(insets);
+                    if (m_keyboardIsVisible != windowInsets.isVisible(WindowInsets.Type.ime()))
+                        setKeyboardVisibility_internal(!m_keyboardIsVisible, System.nanoTime());
+                    return windowInsets;
+            });
+        }
+    }
+
+    private final ViewTreeObserver.OnGlobalLayoutListener keyboardListener =
+                                                new ViewTreeObserver.OnGlobalLayoutListener() {
+        @Override
+        public void onGlobalLayout() {
+            if (!isKeyboardHidden())
+                setKeyboardTransitionInProgress(false);
+        }
+    };
+
+    private void setKeyboardTransitionInProgress(boolean state)
+    {
+        if (m_keyboardTransitionInProgress == state || m_currentEditText == null)
+            return;
+
+        m_keyboardTransitionInProgress = state;
+        ViewTreeObserver observer = m_currentEditText.getViewTreeObserver();
+        if (state)
+            observer.addOnGlobalLayoutListener(keyboardListener);
+        else
+            observer.removeOnGlobalLayoutListener(keyboardListener);
     }
 
     // QtInputInterface implementation begin
@@ -87,12 +130,65 @@ class QtInputDelegate implements QtInputConnection.QtInputConnectionListener, Qt
     public void updateSelection(final int selStart, final int selEnd,
                                 final int candidatesStart, final int candidatesEnd)
     {
-        QtNative.runAction(() -> {
-            if (m_imm != null) {
-                m_imm.updateSelection(m_currentEditText, selStart, selEnd,
-                        candidatesStart, candidatesEnd);
-            }
-        });
+        if (m_imm != null) {
+            QtNative.runAction(() -> {
+                if (m_imm != null) {
+                    m_imm.updateSelection(m_currentEditText, selStart, selEnd,
+                            candidatesStart, candidatesEnd);
+                }
+            });
+        }
+    }
+
+    private void showKeyboard(Activity activity,
+                              final int x, final int y, final int width, final int height,
+                              final int inputHints, final int enterKeyType)
+    {
+        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
+            activity.getWindow().getInsetsController().controlWindowInsetsAnimation(
+                WindowInsets.Type.ime(), -1, null, null,
+                    new WindowInsetsAnimationControlListener() {
+                        @Override
+                        public void onCancelled(WindowInsetsAnimationController controller) { }
+
+                        @Override
+                        public void onReady(WindowInsetsAnimationController controller, int types) { }
+
+                        @Override
+                        public void onFinished(WindowInsetsAnimationController controller) {
+                            QtNativeInputConnection.updateCursorPosition();
+                            if (m_softInputMode == 0)
+                                probeForKeyboardHeight(activity, x, y, width, height,
+                                                       inputHints, enterKeyType);
+                        }
+                    });
+            activity.getWindow().getInsetsController().show(Type.ime());
+        } else {
+            if (m_imm == null)
+                return;
+            m_imm.showSoftInput(m_currentEditText, 0, new ResultReceiver(new Handler()) {
+                @Override
+                @SuppressWarnings("fallthrough")
+                protected void onReceiveResult(int resultCode, Bundle resultData) {
+                    switch (resultCode) {
+                        case InputMethodManager.RESULT_SHOWN:
+                            QtNativeInputConnection.updateCursorPosition();
+                            //FALLTHROUGH
+                        case InputMethodManager.RESULT_UNCHANGED_SHOWN:
+                            setKeyboardVisibility(true, System.nanoTime());
+                            if (m_softInputMode == 0) {
+                                probeForKeyboardHeight(activity,
+                                        x, y, width, height, inputHints, enterKeyType);
+                            }
+                            break;
+                        case InputMethodManager.RESULT_HIDDEN:
+                        case InputMethodManager.RESULT_UNCHANGED_HIDDEN:
+                            setKeyboardVisibility(false, System.nanoTime());
+                            break;
+                    }
+                }
+            });
+        }
     }
 
     @Override
@@ -100,6 +196,9 @@ class QtInputDelegate implements QtInputConnection.QtInputConnectionListener, Qt
                                      final int x, final int y, final int width, final int height,
                                      final int inputHints, final int enterKeyType)
     {
+        if (m_imm == null)
+            return;
+
         QtNative.runAction(() -> {
             if (m_imm == null || m_currentEditText == null)
                 return;
@@ -111,29 +210,7 @@ class QtInputDelegate implements QtInputConnection.QtInputConnectionListener, Qt
             m_currentEditText.setLayoutParams(new QtLayout.LayoutParams(width, height, x, y));
             m_currentEditText.requestFocus();
             m_currentEditText.postDelayed(() -> {
-                if (m_imm == null)
-                    return;
-                m_imm.showSoftInput(m_currentEditText, 0, new ResultReceiver(new Handler()) {
-                    @Override
-                    protected void onReceiveResult(int resultCode, Bundle resultData) {
-                        switch (resultCode) {
-                            case InputMethodManager.RESULT_SHOWN:
-                                QtNativeInputConnection.updateCursorPosition();
-                                //FALLTHROUGH
-                            case InputMethodManager.RESULT_UNCHANGED_SHOWN:
-                                setKeyboardVisibility(true, System.nanoTime());
-                                if (m_softInputMode == 0) {
-                                    probeForKeyboardHeight(activity,
-                                            x, y, width, height, inputHints, enterKeyType);
-                                }
-                                break;
-                            case InputMethodManager.RESULT_HIDDEN:
-                            case InputMethodManager.RESULT_UNCHANGED_HIDDEN:
-                                setKeyboardVisibility(false, System.nanoTime());
-                                break;
-                        }
-                    }
-                });
+                showKeyboard(activity, x, y, width, height, inputHints, enterKeyType);
                 if (m_currentEditText.m_optionsChanged) {
                     m_imm.restartInput(m_currentEditText);
                     m_currentEditText.m_optionsChanged = false;
@@ -174,6 +251,8 @@ class QtInputDelegate implements QtInputConnection.QtInputConnectionListener, Qt
         if (m_imm == null || m_currentEditText == null)
             return;
         m_currentEditText.postDelayed(() -> {
+            if (m_imm == null || m_currentEditText == null)
+                return;
             m_imm.restartInput(m_currentEditText);
             m_currentEditText.m_optionsChanged = false;
         }, 5);
@@ -182,27 +261,39 @@ class QtInputDelegate implements QtInputConnection.QtInputConnectionListener, Qt
     @Override
     public void hideSoftwareKeyboard()
     {
+        if (m_imm == null || m_currentEditText == null)
+            return;
+
         m_isKeyboardHidingAnimationOngoing = true;
         QtNative.runAction(() -> {
             if (m_imm == null || m_currentEditText == null)
                 return;
 
-            m_imm.hideSoftInputFromWindow(m_currentEditText.getWindowToken(), 0,
-                    new ResultReceiver(new Handler()) {
-                        @Override
-                        protected void onReceiveResult(int resultCode, Bundle resultData) {
-                            switch (resultCode) {
-                                case InputMethodManager.RESULT_SHOWN:
-                                case InputMethodManager.RESULT_UNCHANGED_SHOWN:
-                                    setKeyboardVisibility(true, System.nanoTime());
-                                    break;
-                                case InputMethodManager.RESULT_HIDDEN:
-                                case InputMethodManager.RESULT_UNCHANGED_HIDDEN:
-                                    setKeyboardVisibility(false, System.nanoTime());
-                                    break;
+            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
+                Activity activity = QtNative.activity();
+                if (activity == null) {
+                    Log.w(TAG, "hideSoftwareKeyboard: The activity reference is null");
+                    return;
+                }
+                activity.getWindow().getInsetsController().hide(Type.ime());
+            } else {
+                m_imm.hideSoftInputFromWindow(m_currentEditText.getWindowToken(), 0,
+                        new ResultReceiver(new Handler()) {
+                            @Override
+                            protected void onReceiveResult(int resultCode, Bundle resultData) {
+                                switch (resultCode) {
+                                    case InputMethodManager.RESULT_SHOWN:
+                                    case InputMethodManager.RESULT_UNCHANGED_SHOWN:
+                                        setKeyboardVisibility(true, System.nanoTime());
+                                        break;
+                                    case InputMethodManager.RESULT_HIDDEN:
+                                    case InputMethodManager.RESULT_UNCHANGED_HIDDEN:
+                                        setKeyboardVisibility(false, System.nanoTime());
+                                        break;
+                                }
                             }
-                        }
-                    });
+                        });
+            }
         });
     }
 
@@ -215,6 +306,32 @@ class QtInputDelegate implements QtInputConnection.QtInputConnectionListener, Qt
     // QtInputInterface implementation end
 
     // QtInputConnectionListener methods
+    @Override
+    public boolean keyboardTransitionInProgress() {
+       return m_keyboardTransitionInProgress;
+    }
+
+    @Override
+    public boolean isKeyboardHidden() {
+        Activity activity = QtNative.activity();
+        if (activity == null) {
+            Log.w(TAG, "isKeyboardHidden: The activity reference is null");
+            return true;
+        }
+
+        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.R) {
+            Rect r = new Rect();
+            activity.getWindow().getDecorView().getWindowVisibleDisplayFrame(r);
+            DisplayMetrics metrics = new DisplayMetrics();
+            activity.getWindowManager().getDefaultDisplay().getMetrics(metrics);
+            int screenHeight = metrics.heightPixels;
+            final int kbHeight = screenHeight - r.bottom;
+            return kbHeight < screenHeight * KEYBOARD_TO_SCREEN_RATIO;
+        }
+
+        return !m_keyboardIsVisible;
+    }
+
     @Override
     public void onSetClosing(boolean closing) {
         if (!closing)
@@ -259,6 +376,14 @@ class QtInputDelegate implements QtInputConnection.QtInputConnectionListener, Qt
     }
 
     void setKeyboardVisibility(boolean visibility, long timeStamp)
+    {
+        // Since API 30 keyboard visibility changes are tracked by OnApplyWindowInsetsListener.
+        // There are no manual changes anymore
+        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.R)
+            setKeyboardVisibility_internal(visibility, timeStamp);
+    }
+
+    private void setKeyboardVisibility_internal(boolean visibility, long timeStamp)
     {
         if (m_showHideTimeStamp > timeStamp)
             return;
@@ -268,6 +393,7 @@ class QtInputDelegate implements QtInputConnection.QtInputConnectionListener, Qt
             return;
         m_keyboardIsVisible = visibility;
         keyboardVisibilityUpdated(m_keyboardIsVisible);
+        setKeyboardTransitionInProgress(visibility);
 
         if (!visibility) {
             // Hiding the keyboard clears the immersive mode, so we need to set it again.
@@ -279,6 +405,7 @@ class QtInputDelegate implements QtInputConnection.QtInputConnectionListener, Qt
 
     void setFocusedView(QtEditText currentEditText)
     {
+        setKeyboardTransitionInProgress(false);
         m_currentEditText = currentEditText;
     }
 
@@ -441,7 +568,7 @@ class QtInputDelegate implements QtInputConnection.QtInputConnectionListener, Qt
     // pointer methods
     static native void mouseDown(int winId, int x, int y, int mouseButtonState);
     static native void mouseUp(int winId, int x, int y, int mouseButtonState);
-    static native void mouseMove(int winId, int x, int y);
+    static native void mouseMove(int winId, int x, int y, int mouseButtonState);
     static native void mouseWheel(int winId, int x, int y, float hDelta, float vDelta);
     static native void touchBegin(int winId);
     static native void touchAdd(int winId, int pointerId, int action, boolean primary,
@@ -567,12 +694,12 @@ class QtInputDelegate implements QtInputConnection.QtInputConnectionListener, Qt
             case MotionEvent.ACTION_HOVER_MOVE:
             case MotionEvent.ACTION_MOVE:
                 if (event.getToolType(0) == MotionEvent.TOOL_TYPE_MOUSE) {
-                    mouseMove(id, (int) event.getX(), (int) event.getY());
+                    mouseMove(id, (int) event.getX(), (int) event.getY(), event.getButtonState());
                 } else {
                     int dx = (int) (event.getX() - m_oldX);
                     int dy = (int) (event.getY() - m_oldY);
                     if (Math.abs(dx) > 5 || Math.abs(dy) > 5) {
-                        mouseMove(id, (int) event.getX(), (int) event.getY());
+                        mouseMove(id, (int) event.getX(), (int) event.getY(), event.getButtonState());
                         m_oldX = (int) event.getX();
                         m_oldY = (int) event.getY();
                     }
