@@ -4,9 +4,10 @@
 package org.qtproject.qt.android;
 
 import android.annotation.SuppressLint;
+import android.app.ActionBar;
 import android.app.Activity;
 import android.content.Context;
-
+import android.content.res.TypedArray;
 import android.graphics.Insets;
 
 import android.view.DisplayCutout;
@@ -15,8 +16,8 @@ import android.view.MotionEvent;
 import android.view.Surface;
 import android.view.View;
 import android.view.ViewGroup;
+import android.view.ViewTreeObserver;
 import android.view.WindowInsets;
-
 import android.os.Build;
 
 import java.util.HashMap;
@@ -30,6 +31,8 @@ class QtWindow extends QtLayout implements QtSurfaceInterface {
     private GestureDetector m_gestureDetector;
     private final QtEditText m_editText;
     private final QtInputConnection.QtInputConnectionListener m_inputConnectionListener;
+    private boolean m_firstSafeMarginsDelivered = false;
+    private int m_actionBarHeight = -1;
 
     private static native void setSurface(int windowId, Surface surface);
     private static native void safeAreaMarginsChanged(Insets insets, int id);
@@ -76,46 +79,151 @@ class QtWindow extends QtLayout implements QtSurfaceInterface {
             m_gestureDetector.setIsLongpressEnabled(true);
         });
 
-        if (getContext() instanceof QtActivityBase) {
-            setOnApplyWindowInsetsListener((view, insets) -> {
-                Insets safeInsets;
-                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
-                    int types = WindowInsets.Type.displayCutout() | WindowInsets.Type.systemBars();
-                    safeInsets = insets.getInsets(types);
-                } else {
-                    int left = 0;
-                    int top = 0;
-                    int right = 0;
-                    int bottom = 0;
+        registerSafeAreaMarginsListener();
+    }
 
-                    int visibility = view.getSystemUiVisibility();
-                    if ((visibility & View.SYSTEM_UI_FLAG_FULLSCREEN) == 0) {
-                        left = insets.getSystemWindowInsetLeft();
-                        top = insets.getSystemWindowInsetTop();
-                        right = insets.getSystemWindowInsetRight();
-                        bottom = insets.getSystemWindowInsetBottom();
-                    }
+    void registerSafeAreaMarginsListener()
+    {
+        if (!(getContext() instanceof QtActivityBase))
+            return;
 
-                    // Android 9 and 10 emulators don't seem to be able
-                    // to handle this, but let's have the logic here anyway
-                    DisplayCutout cutout = insets.getDisplayCutout();
-                    if (cutout != null) {
-                        left = Math.max(left, cutout.getSafeInsetLeft());
-                        top = Math.max(top, cutout.getSafeInsetTop());
-                        right = Math.max(right, cutout.getSafeInsetRight());
-                        bottom = Math.max(bottom, cutout.getSafeInsetBottom());
-                    }
+        setOnApplyWindowInsetsListener((view, insets) -> {
+            WindowInsets windowInsets = view.onApplyWindowInsets(insets);
+            reportSafeAreaMargins(windowInsets, getId());
 
-                    safeInsets = Insets.of(left, top, right, bottom);
+            return windowInsets;
+        });
+
+        // If the window is attached, try to directly deliver root insets
+        if (isAttachedToWindow()) {
+            WindowInsets insets = getRootWindowInsets();
+            if (insets != null) {
+                getRootView().post(() -> reportSafeAreaMargins(insets, getId()));
+                m_firstSafeMarginsDelivered = true;
+            }
+        } else { // Otherwise request it upon attachement
+            addOnAttachStateChangeListener(new View.OnAttachStateChangeListener() {
+                @Override
+                public void onViewAttachedToWindow(View view) {
+                    view.removeOnAttachStateChangeListener(this);
+                    view.requestApplyInsets();
                 }
 
-                QtNative.runAction(() -> safeAreaMarginsChanged(safeInsets, getId()));
-
-                return insets;
+                @Override
+                public void onViewDetachedFromWindow(View view) {}
             });
-
-            QtNative.runAction(() -> requestApplyInsets());
         }
+
+        // Further, tag into pre draw to deliver safe area margins early on
+        if (!m_firstSafeMarginsDelivered) {
+            ViewTreeObserver.OnPreDrawListener listener = new ViewTreeObserver.OnPreDrawListener() {
+                @Override
+                public boolean onPreDraw() {
+                    if (isAttachedToWindow()) {
+                        WindowInsets insets = getRootWindowInsets();
+                        if (insets != null) {
+                            getViewTreeObserver().removeOnPreDrawListener(this);
+                            getRootView().post(() -> reportSafeAreaMargins(insets, getId()));
+                            m_firstSafeMarginsDelivered = true;
+                            return true;
+                        }
+                    }
+
+                    requestApplyInsets();
+
+                    return true;
+                }
+            };
+            getViewTreeObserver().addOnPreDrawListener(listener);
+        }
+
+        addOnLayoutChangeListener((view, l, t, r, b, oldl, oldt, oldr, oldb) -> {
+            WindowInsets insets = getRootWindowInsets();
+            if (insets != null)
+                getRootView().post(() -> reportSafeAreaMargins(insets, getId()));
+        });
+    }
+
+    @SuppressWarnings("deprecation")
+    Insets getSafeInsets(View view, WindowInsets insets)
+    {
+        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
+            int types = WindowInsets.Type.displayCutout() | WindowInsets.Type.systemBars();
+            return insets.getInsets(types);
+        }
+
+        // Android R and older
+        int left = insets.getSystemWindowInsetLeft();
+        int top = insets.getSystemWindowInsetTop();
+        int right = insets.getSystemWindowInsetRight();
+        int bottom = insets.getSystemWindowInsetBottom();
+
+        // Android 9 and 10 emulators don't seem to be able
+        // to handle this, but let's have the logic here anyway
+        DisplayCutout cutout = insets.getDisplayCutout();
+        if (cutout != null) {
+            left = Math.max(left, cutout.getSafeInsetLeft());
+            top = Math.max(top, cutout.getSafeInsetTop());
+            right = Math.max(right, cutout.getSafeInsetRight());
+            bottom = Math.max(bottom, cutout.getSafeInsetBottom());
+        }
+
+        // If a theme supports an action bar, sometimes it even if it's hidden
+        // the insets might report values including the bar's height, not entirely
+        // sure whether it's due to a delay or a bug, either way ensure the top
+        // margin doesn't include the action bar's height.
+        ActionBar actionBar = ((Activity) getContext()).getActionBar();
+        if (actionBar == null || !actionBar.isShowing()) {
+            int topWithoutActionBar = top - actionBarHeight();
+            if (topWithoutActionBar > 0)
+                top = topWithoutActionBar;
+        }
+
+        return Insets.of(left, top, right, bottom);
+    }
+
+    private void reportSafeAreaMargins(WindowInsets insets, int id)
+    {
+        View rootView = getRootView();
+
+        int[] rootLocation = new int[2];
+        rootView.getLocationOnScreen(rootLocation);
+        int rootX = rootLocation[0];
+        int rootY = rootLocation[1];
+
+        int[] windowLocation = new int[2];
+        getLocationOnScreen(windowLocation);
+        int windowX = windowLocation[0];
+        int windowY = windowLocation[1];
+
+        // Offset values of window from root
+        int leftOffset = windowX - rootX;
+        int topOffset = windowY - rootY;
+        int rightOffset = (rootX + rootView.getWidth()) - (windowX + getWidth());
+        int bottomOffset = (rootY + rootView.getHeight()) - (windowY + getHeight());
+
+        // Find the remaining minimum safe margins
+        Insets safeInsets = getSafeInsets(rootView, insets);
+        int left = Math.max(0, Math.min(safeInsets.left, safeInsets.left - leftOffset));
+        int top = Math.max(0, Math.min(safeInsets.top, safeInsets.top - topOffset));
+        int right = Math.max(0, Math.min(safeInsets.right, safeInsets.right - rightOffset));
+        int bottom = Math.max(0, Math.min(safeInsets.bottom, safeInsets.bottom - bottomOffset));
+
+        safeAreaMarginsChanged(Insets.of(left, top, right, bottom), id);
+    }
+
+    private int actionBarHeight()
+    {
+        if (m_actionBarHeight == -1) {
+            TypedArray ta = getContext().getTheme().obtainStyledAttributes(
+                new int[] { android.R.attr.actionBarSize });
+            try {
+                m_actionBarHeight = ta.getDimensionPixelSize(0, 0);
+            } finally {
+                ta.recycle();
+            }
+        }
+        return m_actionBarHeight;
     }
 
     @UsedFromNativeCode
