@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 /*!
     \page qtconcurrentfilter.html
@@ -25,6 +26,20 @@
     Note that the result types above are not QFuture objects, but real result
     types (in this case, QStringList and QSet<QString>).
 
+    \section1 Optimize includes
+
+    If you include the \c <QtConcurrent> header, the entire Qt Concurrent
+    module with the entire Qt Core module will be included, which may increase
+    compilation times and binary sizes. To use the
+    \l {QtConcurrent::filter}{QtConcurrent::filter()},
+    \l {QtConcurrent::filtered}{QtConcurrent::filtered()}, and
+    \l {QtConcurrent::filteredReduced}{QtConcurrent::filteredReduced()}
+    functions, you can include a more specific header:
+
+    \code
+    #include <QtConcurrentFilter>
+    \endcode
+
     \section1 Concurrent Filter
 
     QtConcurrent::filtered() takes an input sequence and a filter function.
