@@ -114,15 +114,17 @@ enum {
 #if defined(Q_OS_DARWIN)
 static inline bool hasResourcePropertyFlag(const QFileSystemMetaData &data,
                                            const QFileSystemEntry &entry,
-                                           CFStringRef key)
+                                           CFStringRef key, QCFType<CFURLRef> &url)
 {
-    QCFString path = CFStringCreateWithFileSystemRepresentation(0,
-        entry.nativeFilePath().constData());
-    if (!path)
-        return false;
+    if (!url) {
+        QCFString path = CFStringCreateWithFileSystemRepresentation(0,
+            entry.nativeFilePath().constData());
+        if (!path)
+           return false;
 
-    QCFType<CFURLRef> url = CFURLCreateWithFileSystemPath(0, path, kCFURLPOSIXPathStyle,
-        data.hasFlags(QFileSystemMetaData::DirectoryType));
+        url = CFURLCreateWithFileSystemPath(0, path, kCFURLPOSIXPathStyle,
+            data.hasFlags(QFileSystemMetaData::DirectoryType));
+    }
     if (!url)
         return false;
 
@@ -135,7 +137,8 @@ static inline bool hasResourcePropertyFlag(const QFileSystemMetaData &data,
     return false;
 }
 
-static bool isPackage(const QFileSystemMetaData &data, const QFileSystemEntry &entry)
+static bool isPackage(const QFileSystemMetaData &data, const QFileSystemEntry &entry,
+                      QCFType<CFURLRef> &cachedUrl)
 {
     if (!data.isDirectory())
         return false;
@@ -174,7 +177,7 @@ static bool isPackage(const QFileSystemMetaData &data, const QFileSystemEntry &e
     }
 
     // Third step: check if the directory has the package bit set
-    return hasResourcePropertyFlag(data, entry, kCFURLIsPackageKey);
+    return hasResourcePropertyFlag(data, entry, kCFURLIsPackageKey, cachedUrl);
 }
 #endif
 
@@ -336,10 +339,8 @@ flagsFromStMode(mode_t mode, [[maybe_unused]] quint64 attributes)
     // UF_COMPRESSED and STATX_ATTR_COMPRESSED
     // UF_IMMUTABLE and STATX_ATTR_IMMUTABLE
     // UF_NODUMP and STATX_ATTR_NODUMP
-#ifdef UF_HIDDEN
-    if (attributes & UF_HIDDEN)
-        entryFlags |= QFileSystemMetaData::HiddenAttribute;
-#elif defined(Q_OS_VXWORKS) && __has_include(<dosFsLib.h>)
+
+#if defined(Q_OS_VXWORKS) && __has_include(<dosFsLib.h>)
     if (attributes & DOS_ATTR_RDONLY) {
         // on a DOS FS, stat() always returns 0777 bits set in st_mode
         // when DOS FS is read only the write permissions are removed
@@ -907,11 +908,17 @@ bool QFileSystemEngine::fillMetaData(const QFileSystemEntry &entry, QFileSystemM
     }
     if (what & QFileSystemMetaData::AliasType)
         what |= QFileSystemMetaData::LinkType;
-#endif
+#endif // defined(Q_OS_DARWIN)
+
+    bool needLstat = what.testAnyFlag(QFileSystemMetaData::LinkType);
+
 #ifdef UF_HIDDEN
     if (what & QFileSystemMetaData::HiddenAttribute) {
-        // OS X >= 10.5: st_flags & UF_HIDDEN
-        what |= QFileSystemMetaData::PosixStatFlags;
+        // Some OSes (BSDs) have the ability to mark directory entries as
+        // hidden besides the usual Unix way of naming them with a leading dot.
+        // For those OSes, we must lstat() the entry itself so we can find
+        // out if a symlink is hidden or not.
+        needLstat = true;
     }
 #endif // defined(Q_OS_DARWIN)
 
@@ -942,7 +949,7 @@ bool QFileSystemEngine::fillMetaData(const QFileSystemEntry &entry, QFileSystemM
         struct statx statxBuffer;
     };
     int statResult = -1;
-    if (what & QFileSystemMetaData::LinkType) {
+    if (needLstat) {
         mode_t mode = 0;
         statResult = qt_lstatx(nativeFilePath, &statxBuffer);
         if (statResult == -ENOSYS) {
@@ -956,6 +963,14 @@ bool QFileSystemEngine::fillMetaData(const QFileSystemEntry &entry, QFileSystemM
         }
 
         if (statResult >= 0) {
+#ifdef UF_HIDDEN
+            // currently only supported on systems with no statx() call
+            Q_ASSERT(statResult == 0);
+            if (statBuffer.st_flags & UF_HIDDEN)
+                data.entryFlags |= QFileSystemMetaData::HiddenAttribute;
+            data.knownFlagsMask |= QFileSystemMetaData::HiddenAttribute;
+#endif
+
             if (S_ISLNK(mode)) {
                // it's a symlink, we don't know if the file "exists"
                 data.entryFlags |= QFileSystemMetaData::LinkType;
@@ -1014,28 +1029,36 @@ bool QFileSystemEngine::fillMetaData(const QFileSystemEntry &entry, QFileSystemM
     // third, we try access(2)
     if (what & (QFileSystemMetaData::UserPermissions | QFileSystemMetaData::ExistsAttribute)) {
 #if defined(Q_OS_VXWORKS)
-        // on VxWorks if the filesystem is not POSIX, access() always returns false, despite the
-        // file is readable
         struct statfs statBuf;
-        if (statfs(nativeFilePath, &statBuf) != 0) {
-            what &= ~QFileSystemMetaData::LinkType; // don't clear link: could be broken symlink
-            data.clearFlags(what);
-            return false;
-        }
-        if (statBuf.f_type != NFSV2_MAGIC && statBuf.f_type != NFSV3_MAGIC &&
-            statBuf.f_type != HRFS_MAGIC) {
+        if (statfs(nativeFilePath, &statBuf) == 0) {
+            if (statBuf.f_type != NFSV2_MAGIC && statBuf.f_type != NFSV3_MAGIC &&
+                statBuf.f_type != HRFS_MAGIC) {
 #if __has_include(<dosFsLib.h>)
-            if (data.entryFlags & QFileSystemMetaData::OwnerWritePermission) {
-                data.entryFlags |= QFileSystemMetaData::UserWritePermission;
-            }
-            if (data.entryFlags & QFileSystemMetaData::OwnerExecutePermission) {
-                data.entryFlags |= QFileSystemMetaData::UserExecutePermission;
-            }
+                if (data.entryFlags & QFileSystemMetaData::OwnerWritePermission) {
+                    data.entryFlags |= QFileSystemMetaData::UserWritePermission;
+                }
+                if (data.entryFlags & QFileSystemMetaData::OwnerExecutePermission) {
+                    data.entryFlags |= QFileSystemMetaData::UserExecutePermission;
+                }
 #endif
-            data.entryFlags |= QFileSystemMetaData::UserReadPermission |
-                    QFileSystemMetaData::ExistsAttribute;
-            return true;
+                data.entryFlags |= QFileSystemMetaData::UserReadPermission |
+                        QFileSystemMetaData::ExistsAttribute;
+                return true;
+            }
         }
+#if defined(QT_DEBUG)
+        else {
+              //on VxWorks hostfs, used for debugging, failes on statfs and falsely reports
+              //WasDeleted
+              statResult = QT_STAT(nativeFilePath, &statBuffer);
+              if (statResult == 0) {
+                  data.entryFlags |= QFileSystemMetaData::UserReadPermission |
+                                     QFileSystemMetaData::ExistsAttribute;
+                  data.entryFlags &= ~QFileSystemMetaData::WasDeletedAttribute;
+                  return true;
+              }
+        }
+#endif
 #endif
         // calculate user permissions
         auto checkAccess = [&](QFileSystemMetaData::MetaDataFlag flag, int mode) {
@@ -1065,8 +1088,9 @@ bool QFileSystemEngine::fillMetaData(const QFileSystemEntry &entry, QFileSystemM
     }
 
 #if defined(Q_OS_DARWIN)
+    QCFType<CFURLRef> cachedUrl;
     if (what & QFileSystemMetaData::AliasType) {
-        if (entryErrno == 0 && hasResourcePropertyFlag(data, entry, kCFURLIsAliasFileKey)) {
+        if (entryErrno == 0 && hasResourcePropertyFlag(data, entry, kCFURLIsAliasFileKey, cachedUrl)) {
             // kCFURLIsAliasFileKey includes symbolic links, so filter those out
             if (!(data.entryFlags & QFileSystemMetaData::LinkType))
                 data.entryFlags |= QFileSystemMetaData::AliasType;
@@ -1075,15 +1099,15 @@ bool QFileSystemEngine::fillMetaData(const QFileSystemEntry &entry, QFileSystemM
     }
 
     if (what & QFileSystemMetaData::BundleType) {
-        if (entryErrno == 0 && isPackage(data, entry))
+        if (entryErrno == 0 && isPackage(data, entry, cachedUrl))
             data.entryFlags |= QFileSystemMetaData::BundleType;
 
         data.knownFlagsMask |= QFileSystemMetaData::BundleType;
     }
 
     if (what & QFileSystemMetaData::CaseSensitive) {
-        if (entryErrno == 0 && hasResourcePropertyFlag(
-            data, entry, kCFURLVolumeSupportsCaseSensitiveNamesKey))
+        if (entryErrno == 0 && hasResourcePropertyFlag(data, entry,
+            kCFURLVolumeSupportsCaseSensitiveNamesKey, cachedUrl))
             data.entryFlags |= QFileSystemMetaData::CaseSensitive;
         data.knownFlagsMask |= QFileSystemMetaData::CaseSensitive;
     }
@@ -1101,11 +1125,7 @@ bool QFileSystemEngine::fillMetaData(const QFileSystemEntry &entry, QFileSystemM
             --lastSlash;        // skip non-slashes
         --lastSlash;            // point to the slash or -1 if no slash
 
-        if (nativeFilePath.at(lastSlash + 1) == '.'
-#if defined(Q_OS_DARWIN)
-                || (entryErrno == 0 && hasResourcePropertyFlag(data, entry, kCFURLIsHiddenKey))
-#endif
-                )
+        if (nativeFilePath.at(lastSlash + 1) == '.')
             data.entryFlags |= QFileSystemMetaData::HiddenAttribute;
         data.knownFlagsMask |= QFileSystemMetaData::HiddenAttribute;
     }
@@ -1390,17 +1410,17 @@ struct FreeDesktopTrashOperation
     }
 
     // opens a directory and returns the file descriptor
-    static int openDirFd(int dfd, const char *path, int mode = 0)
+    static int openDirFd(int dfd, const char *path, int mode)
     {
-        mode |= QT_OPEN_RDONLY | O_NOFOLLOW | O_DIRECTORY;
+        mode |= QT_OPEN_RDONLY | O_DIRECTORY;
         return qt_safe_openat(dfd, path, mode);
     }
 
     // opens an XDG Trash directory that is a subdirectory of dfd, creating if necessary
-    static int openOrCreateDir(int dfd, const char *path)
+    static int openOrCreateDir(int dfd, const char *path, int openmode = 0)
     {
         // try to open it as a dir, first
-        int fd = openDirFd(dfd, path);
+        int fd = openDirFd(dfd, path, openmode);
         if (fd >= 0 || errno != ENOENT)
             return fd;
 
@@ -1409,19 +1429,19 @@ struct FreeDesktopTrashOperation
             return -1;
 
         // try to open it again
-        return openDirFd(dfd, path);
+        return openDirFd(dfd, path, openmode);
     }
 
     // opens or makes the XDG Trash hierarchy on parentfd (may be -1) called targetDir
     bool getTrashDir(int parentfd, QString targetDir, const QFileSystemEntry &source,
-                     QSystemError &error)
+                     int openmode, QSystemError &error)
     {
         if (parentfd == AT_FDCWD)
             trashPath = targetDir;
         QByteArray nativePath = QFile::encodeName(targetDir);
 
         // open the directory
-        int trashfd = openOrCreateDir(parentfd, nativePath);
+        int trashfd = openOrCreateDir(parentfd, nativePath, openmode);
         if (trashfd < 0 && errno != ENOENT) {
             error = QSystemError(errno, QSystemError::StandardLibraryError);
             return false;
@@ -1486,7 +1506,7 @@ struct FreeDesktopTrashOperation
         QFileSystemEntry dotTrashDir(sourceStorage.rootPath() + dotTrash);
 
         // we MUST check that the sticky bit is set, and that it is not a symlink
-        int genericTrashFd = openDirFd(AT_FDCWD, dotTrashDir.nativeFilePath());
+        int genericTrashFd = openDirFd(AT_FDCWD, dotTrashDir.nativeFilePath(), O_NOFOLLOW);
         QT_STATBUF st = {};
         if (genericTrashFd < 0 && errno != ENOENT && errno != EACCES) {
             // O_DIRECTORY + O_NOFOLLOW produces ENOTDIR on Linux
@@ -1514,7 +1534,7 @@ struct FreeDesktopTrashOperation
                      the implementation MUST immediately create it, without any warnings or
                      delays for the user."
                 */
-                if (getTrashDir(genericTrashFd, userID, source, error)) {
+                if (getTrashDir(genericTrashFd, userID, source, O_NOFOLLOW, error)) {
                     // recreate the resulting path
                     trashPath = dotTrashDir.filePath() + u'/' + userID;
                 }
@@ -1530,7 +1550,8 @@ struct FreeDesktopTrashOperation
              immediately create it, without any warnings or delays for the user."
         */
         if (!isTrashDirOpen())
-            getTrashDir(AT_FDCWD, sourceStorage.rootPath() + dotTrash + u'-' + userID, source, error);
+            getTrashDir(AT_FDCWD, sourceStorage.rootPath() + dotTrash + u'-' + userID, source,
+                        O_NOFOLLOW, error);
 
         if (isTrashDirOpen()) {
             volumePrefixLength = sourceStorage.rootPath().size();
@@ -1545,7 +1566,8 @@ struct FreeDesktopTrashOperation
     bool openHomeTrashLocation(const QFileSystemEntry &source, QSystemError &error)
     {
         QString topDir = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation);
-        return getTrashDir(AT_FDCWD, topDir + "/Trash"_L1, source, error);
+        int openmode = 0;   // do allow following symlinks
+        return getTrashDir(AT_FDCWD, topDir + "/Trash"_L1, source, openmode, error);
     }
 
     bool findTrashFor(const QFileSystemEntry &source, QSystemError &error)
