@@ -1,55 +1,45 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 Intel Corporation.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 Intel Corporation.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qdeadlinetimer.h"
-#include "qdeadlinetimer_p.h"
-#include <qpair.h>
+#include "private/qnumeric_p.h"
 
 QT_BEGIN_NAMESPACE
 
-Q_DECL_CONST_FUNCTION static inline QPair<qint64, qint64> toSecsAndNSecs(qint64 nsecs)
+QT_IMPL_METATYPE_EXTERN(QDeadlineTimer)
+
+using namespace std::chrono;
+
+namespace {
+struct TimeReference : std::numeric_limits<qint64>
 {
-    qint64 secs = nsecs / (1000*1000*1000);
-    if (nsecs < 0)
-        --secs;
-    nsecs -= secs * 1000*1000*1000;
-    return qMakePair(secs, nsecs);
+    static constexpr qint64 Min = min();
+    static constexpr qint64 Max = max();
+};
+}
+
+template <typename Duration1, typename... Durations>
+static qint64 add_saturate(qint64 t1, Duration1 dur, Durations... extra)
+{
+    qint64 v = dur.count();
+    qint64 saturated = std::numeric_limits<qint64>::max();
+    if (v < 0)
+        saturated = std::numeric_limits<qint64>::min();
+
+    // convert to nanoseconds with saturation
+    using Ratio = std::ratio_divide<typename Duration1::period, nanoseconds::period>;
+    static_assert(Ratio::den == 1, "sub-multiples of nanosecond are not supported");
+    if (qMulOverflow<Ratio::num>(v, &v))
+        return saturated;
+
+    qint64 r;
+    if (qAddOverflow(t1, v, &r))
+        return saturated;
+    if constexpr (sizeof...(Durations)) {
+        // chain more additions
+        return add_saturate(r, extra...);
+    }
+    return r;
 }
 
 /*!
@@ -61,6 +51,8 @@ Q_DECL_CONST_FUNCTION static inline QPair<qint64, qint64> toSecsAndNSecs(qint64
     \reentrant
     \ingroup tools
 
+    \compares strong
+
     The QDeadlineTimer class is usually used to calculate future deadlines and
     verify whether the deadline has expired. QDeadlineTimer can also be used
     for deadlines without expiration ("forever"). It forms a counterpart to
@@ -76,17 +68,7 @@ Q_DECL_CONST_FUNCTION static inline QPair<qint64, qint64> toSecsAndNSecs(qint64
     QDeadlineTimer objects can be passed to functions being called to execute
     this operation so they know how long to still operate.
 
-    \code
-    void executeOperation(int msecs)
-    {
-        QDeadlineTimer deadline(msecs);
-        do {
-            if (readFromDevice(deadline.remainingTime())
-                break;
-            waitForReadyRead(deadline);
-        } while (!deadline.hasExpired());
-    }
-    \endcode
+    \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 0
 
     Many QDeadlineTimer functions deal with time out values, which all are
     measured in milliseconds. There are two special values, the same as many
@@ -104,11 +86,12 @@ Q_DECL_CONST_FUNCTION static inline QPair<qint64, qint64> toSecsAndNSecs(qint64
 
     \section1 Timer types
 
-    Like QTimer, QDeadlineTimer can select among different levels of coarseness
-    on the timers. You can select precise timing by passing Qt::PreciseTimer to
-    the functions that set of change the timer, or you can select coarse timing
-    by passing Qt::CoarseTimer. Qt::VeryCoarseTimer is currently interpreted
-    the same way as Qt::CoarseTimer.
+    Like QTimer and QChronoTimer, QDeadlineTimer can select among
+    different levels of coarseness on the timers. You can select
+    precise timing by passing Qt::PreciseTimer to the functions that
+    set of change the timer, or you can select coarse timing by passing
+    Qt::CoarseTimer. Qt::VeryCoarseTimer is currently interpreted the same
+    way as Qt::CoarseTimer.
 
     This feature is dependent on support from the operating system: if the OS
     does not support a coarse timer functionality, then QDeadlineTimer will
@@ -124,17 +107,10 @@ Q_DECL_CONST_FUNCTION static inline QPair<qint64, qint64> toSecsAndNSecs(qint64
     QDeadlineTimer is compatible with the \c{std::chrono} API from C++11 and
     can be constructed from or compared to both \c{std::chrono::duration} and
     \c{std::chrono::time_point} objects. In addition, it is fully compatible
-    with the time literals from C++14, which allow one to write code as:
+    with the \l{chrono_literals Symbol Index}{time literals
+    from C++14}, which allow one to write code such as:
 
-    \code
-        using namespace std::chrono;
-        using namespace std::chrono_literals;
-
-        QDeadlineTimer deadline(30s);
-        device->waitForReadyRead(deadline);
-        if (deadline.remainingTime<nanoseconds>() > 300ms)
-            cleanup();
-    \endcode
+    \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 1
 
     As can be seen in the example above, QDeadlineTimer offers a templated
     version of remainingTime() and deadline() that can be used to return
@@ -146,15 +122,9 @@ Q_DECL_CONST_FUNCTION static inline QPair<qint64, qint64> toSecsAndNSecs(qint64
     Also note that, due to this conversion, the deadlines will not be precise,
     so the following code is not expected to compare equally:
 
-    \code
-        using namespace std::chrono;
-        using namespace std::chrono_literals;
-        auto now = steady_clock::now();
-        QDeadlineTimer deadline(now + 1s);
-        Q_ASSERT(deadline == now + 1s);
-    \endcode
+    \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 2
 
-    \sa QTime, QTimer, QDeadlineTimer, Qt::TimerType
+    \sa QTime, QChronoTimer, QElapsedTimer, Qt::TimerType
 */
 
 /*!
@@ -165,10 +135,12 @@ Q_DECL_CONST_FUNCTION static inline QPair<qint64, qint64> toSecsAndNSecs(qint64
 */
 
 /*!
+    \fn QDeadlineTimer::QDeadlineTimer()
     \fn QDeadlineTimer::QDeadlineTimer(Qt::TimerType timerType)
 
     Constructs an expired QDeadlineTimer object. For this object,
-    remainingTime() will return 0.
+    remainingTime() will return 0. If \a timerType is not set, then the object
+    will use the \l{Qt::CoarseTimer}{coarse} \l{QDeadlineTimer#Timer types}{timer type}.
 
     The timer type \a timerType may be ignored, since the timer is already
     expired. Similarly, for optimization purposes, this function will not
@@ -187,8 +159,8 @@ Q_DECL_CONST_FUNCTION static inline QPair<qint64, qint64> toSecsAndNSecs(qint64
     For such objects, remainingTime() will return -1, deadline() will return the
     maximum value, and isForever() will return true.
 
-    The timer type \a timerType may be ignored, since the timer is already
-    expired.
+    The timer type \a timerType may be ignored, since the timer will never
+    expire.
 
     \sa ForeverConstant, hasExpired(), isForever(), remainingTime(), timerType()
 */
@@ -198,9 +170,9 @@ Q_DECL_CONST_FUNCTION static inline QPair<qint64, qint64> toSecsAndNSecs(qint64
     from the moment of the creation of this object, if msecs is positive. If \a
     msecs is zero, this QDeadlineTimer will be marked as expired, causing
     remainingTime() to return zero and deadline() to return an indeterminate
-    time point in the past. If \a msecs is -1, the timer will be set it to
-    never expire, causing remainingTime() to return -1 and deadline() to return
-    the maximum value.
+    time point in the past. If \a msecs is negative, the timer will be set to never
+    expire, causing remainingTime() to return -1 and deadline() to return the
+    maximum value.
 
     The QDeadlineTimer object will be constructed with the specified timer \a type.
 
@@ -211,10 +183,12 @@ Q_DECL_CONST_FUNCTION static inline QPair<qint64, qint64> toSecsAndNSecs(qint64
     functionality is required, use QDeadlineTimer::current() and add time to
     it.
 
+    \note Prior to Qt 6.6, the only value that caused the timer to never expire
+    was -1.
+
     \sa hasExpired(), isForever(), remainingTime(), setRemainingTime()
 */
-QDeadlineTimer::QDeadlineTimer(qint64 msecs, Qt::TimerType type) Q_DECL_NOTHROW
-    : t2(0)
+QDeadlineTimer::QDeadlineTimer(qint64 msecs, Qt::TimerType type) noexcept
 {
     setRemainingTime(msecs, type);
 }
@@ -224,7 +198,7 @@ QDeadlineTimer::QDeadlineTimer(qint64 msecs, Qt::TimerType type) Q_DECL_NOTHROW
 
     Constructs a QDeadlineTimer object with a deadline at \a deadline time
     point, converting from the clock source \c{Clock} to Qt's internal clock
-    source (see QElapsedTimer::clcokType()).
+    source (see QElapsedTimer::clockType()).
 
     If \a deadline is in the past, this QDeadlineTimer object is set to
     expired, whereas if \a deadline is equal to \c{Duration::max()}, then this
@@ -245,12 +219,10 @@ QDeadlineTimer::QDeadlineTimer(qint64 msecs, Qt::TimerType type) Q_DECL_NOTHROW
 
     The QDeadlineTimer object will be constructed with the specified timer \a type.
 
-    This constructor can be used with C++14's user-defined literals for time, such as in:
+    This constructor can be used with \l{chrono_literals Symbol Index}
+    {C++14's user-defined literals for time}, such as in:
 
-    \code
-        using namespace std::chrono_literals;
-        QDeadlineTimer deadline(250ms);
-    \endcode
+    \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 3
 
     For optimization purposes, if \a remaining is zero or negative, this
     function may skip obtaining the current time and may instead use a value
@@ -267,7 +239,7 @@ QDeadlineTimer::QDeadlineTimer(qint64 msecs, Qt::TimerType type) Q_DECL_NOTHROW
 
     Sets this QDeadlineTimer to the deadline marked by \a deadline time
     point, converting from the clock source \c{Clock} to Qt's internal clock
-    source (see QElapsedTimer::clcokType()).
+    source (see QElapsedTimer::clockType()).
 
     If \a deadline is in the past, this QDeadlineTimer object is set to
     expired, whereas if \a deadline is equal to \c{Duration::max()}, then this
@@ -281,49 +253,69 @@ QDeadlineTimer::QDeadlineTimer(qint64 msecs, Qt::TimerType type) Q_DECL_NOTHROW
 /*!
     Sets the remaining time for this QDeadlineTimer object to \a msecs
     milliseconds from now, if \a msecs has a positive value. If \a msecs is
-    zero, this QDeadlineTimer object will be marked as expired, whereas a value
-    of -1 will set it to never expire.
+    zero, this QDeadlineTimer object will be marked as expired, whereas a
+    negative value will set it to never expire.
+
+    For optimization purposes, if \a msecs is zero, this function may skip
+    obtaining the current time and may instead use a value known to be in the
+    past. If that happens, deadline() may return an unexpected value and this
+    object cannot be used in calculation of how long it is overdue. If that
+    functionality is required, use QDeadlineTimer::current() and add time to
+    it.
 
     The timer type for this QDeadlineTimer object will be set to the specified \a timerType.
 
+    \note Prior to Qt 6.6, the only value that caused the timer to never expire
+    was -1.
+
     \sa setPreciseRemainingTime(), hasExpired(), isForever(), remainingTime()
 */
-void QDeadlineTimer::setRemainingTime(qint64 msecs, Qt::TimerType timerType) Q_DECL_NOTHROW
+void QDeadlineTimer::setRemainingTime(qint64 msecs, Qt::TimerType timerType) noexcept
 {
-    if (msecs == -1)
+    if (msecs < 0) {
         *this = QDeadlineTimer(Forever, timerType);
-    else
-        setPreciseRemainingTime(0, msecs * 1000 * 1000, timerType);
+    } else if (msecs == 0) {
+        *this = QDeadlineTimer(timerType);
+        t1 = std::numeric_limits<qint64>::min();
+    } else {
+        *this = current(timerType);
+        milliseconds ms(msecs);
+        t1 = add_saturate(t1, ms);
+    }
 }
 
 /*!
     Sets the remaining time for this QDeadlineTimer object to \a secs seconds
     plus \a nsecs nanoseconds from now, if \a secs has a positive value. If \a
-    secs is -1, this QDeadlineTimer will be set it to never expire. If both
-    parameters are zero, this QDeadlineTimer will be marked as expired.
+    secs is negative, this QDeadlineTimer will be set it to never expire (this
+    behavior does not apply to \a nsecs). If both parameters are zero, this
+    QDeadlineTimer will be marked as expired.
+
+    For optimization purposes, if both \a secs and \a nsecs are zero, this
+    function may skip obtaining the current time and may instead use a value
+    known to be in the past. If that happens, deadline() may return an
+    unexpected value and this object cannot be used in calculation of how long
+    it is overdue. If that functionality is required, use
+    QDeadlineTimer::current() and add time to it.
 
     The timer type for this QDeadlineTimer object will be set to the specified
     \a timerType.
 
+    \note Prior to Qt 6.6, the only condition that caused the timer to never
+    expire was when \a secs was -1.
+
     \sa setRemainingTime(), hasExpired(), isForever(), remainingTime()
 */
-void QDeadlineTimer::setPreciseRemainingTime(qint64 secs, qint64 nsecs, Qt::TimerType timerType) Q_DECL_NOTHROW
+void QDeadlineTimer::setPreciseRemainingTime(qint64 secs, qint64 nsecs, Qt::TimerType timerType) noexcept
 {
-    if (secs == -1) {
+    if (secs < 0) {
         *this = QDeadlineTimer(Forever, timerType);
-        return;
-    }
-
-    *this = current(timerType);
-    if (QDeadlineTimerNanosecondsInT2) {
-        t1 += secs + toSecsAndNSecs(nsecs).first;
-        t2 += toSecsAndNSecs(nsecs).second;
-        if (t2 > 1000*1000*1000) {
-            t2 -= 1000*1000*1000;
-            ++t1;
-        }
+    } else if (secs == 0 && nsecs == 0) {
+        *this = QDeadlineTimer(timerType);
+        t1 = std::numeric_limits<qint64>::min();
     } else {
-        t1 += secs * 1000 * 1000 * 1000 + nsecs;
+        *this = current(timerType);
+        t1 = add_saturate(t1, seconds{secs}, nanoseconds{nsecs});
     }
 }
 
@@ -338,12 +330,10 @@ void QDeadlineTimer::setPreciseRemainingTime(qint64 secs, qint64 nsecs, Qt::Time
 
     The timer type for this QDeadlineTimer object will be set to the specified \a type.
 
-    This function can be used with C++14's user-defined literals for time, such as in:
+    This function can be used with \l{chrono_literals Symbol Index}
+    {C++14's user-defined literals for time}, such as in:
 
-    \code
-        using namespace std::chrono_literals;
-        deadline.setRemainingTime(250ms);
-    \endcode
+    \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 4
 
     \sa setDeadline(), remainingTime(), hasExpired(), isForever()
 */
@@ -368,10 +358,12 @@ void QDeadlineTimer::setPreciseRemainingTime(qint64 secs, qint64 nsecs, Qt::Time
 
     \sa isForever(), remainingTime()
 */
-bool QDeadlineTimer::hasExpired() const Q_DECL_NOTHROW
+bool QDeadlineTimer::hasExpired() const noexcept
 {
     if (isForever())
         return false;
+    if (t1 == std::numeric_limits<qint64>::min())
+        return true;
     return *this <= current(timerType());
 }
 
@@ -411,16 +403,17 @@ void QDeadlineTimer::setTimerType(Qt::TimerType timerType)
     lock functions in \l QMutex, \l QWaitCondition, \l QSemaphore, or
     \l QReadWriteLock. For example:
 
-    \code
-        mutex.tryLock(deadline.remainingTime());
-    \endcode
+    \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 5
 
     \sa remainingTimeNSecs(), isForever(), hasExpired()
 */
-qint64 QDeadlineTimer::remainingTime() const Q_DECL_NOTHROW
+qint64 QDeadlineTimer::remainingTime() const noexcept
 {
-    qint64 ns = remainingTimeNSecs();
-    return ns <= 0 ? ns : (ns + 999999) / (1000 * 1000);
+    if (isForever())
+        return -1;
+
+    nanoseconds nsecs(remainingTimeNSecs());
+    return ceil<milliseconds>(nsecs).count();
 }
 
 /*!
@@ -431,7 +424,7 @@ qint64 QDeadlineTimer::remainingTime() const Q_DECL_NOTHROW
 
     \sa remainingTime(), isForever(), hasExpired()
 */
-qint64 QDeadlineTimer::remainingTimeNSecs() const Q_DECL_NOTHROW
+qint64 QDeadlineTimer::remainingTimeNSecs() const noexcept
 {
     if (isForever())
         return -1;
@@ -442,14 +435,19 @@ qint64 QDeadlineTimer::remainingTimeNSecs() const Q_DECL_NOTHROW
 /*!
     \internal
     Same as remainingTimeNSecs, but may return negative remaining times. Does
-    not deal with Forever.
+    not deal with Forever. In case of underflow, which is only possible if the
+    timer has expired, an arbitrary negative value is returned.
 */
-qint64 QDeadlineTimer::rawRemainingTimeNSecs() const Q_DECL_NOTHROW
+qint64 QDeadlineTimer::rawRemainingTimeNSecs() const noexcept
 {
+    if (t1 == std::numeric_limits<qint64>::min())
+        return t1;          // we'd saturate to this anyway
+
     QDeadlineTimer now = current(timerType());
-    if (QDeadlineTimerNanosecondsInT2)
-        return (t1 - now.t1) * (1000*1000*1000) + t2 - now.t2;
-    return t1 - now.t1;
+    qint64 r;
+    if (qSubOverflow(t1, now.t1, &r))
+        return -1;      // any negative number is fine
+    return r;
 }
 
 /*!
@@ -465,27 +463,22 @@ qint64 QDeadlineTimer::rawRemainingTimeNSecs() const Q_DECL_NOTHROW
     overdue, by subtracting QDeadlineTimer::current() or
     QElapsedTimer::msecsSinceReference(), as in the following example:
 
-    \code
-        qint64 realTimeLeft = deadline.deadline();
-        if (realTimeLeft != (std::numeric_limits<qint64>::max)()) {
-            realTimeLeft -= QDeadlineTimer::current().deadline();
-            // or:
-            //QElapsedTimer timer;
-            //timer.start();
-            //realTimeLeft -= timer.msecsSinceReference();
-        }
-    \endcode
+    \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 6
 
     \note Timers that were created as expired have an indetermine time point in
     the past as their deadline, so the above calculation may not work.
 
     \sa remainingTime(), deadlineNSecs(), setDeadline()
 */
-qint64 QDeadlineTimer::deadline() const Q_DECL_NOTHROW
+qint64 QDeadlineTimer::deadline() const noexcept
 {
     if (isForever())
+        return TimeReference::Max;
+    if (t1 == TimeReference::Min)
         return t1;
-    return deadlineNSecs() / (1000 * 1000);
+
+    nanoseconds ns(t1);
+    return duration_cast<milliseconds>(ns).count();
 }
 
 /*!
@@ -494,30 +487,26 @@ qint64 QDeadlineTimer::deadline() const Q_DECL_NOTHROW
     same as QElapsedTimer::msecsSinceReference(). The value will be in the past
     if this QDeadlineTimer has expired.
 
-    If this QDeadlineTimer never expires, this function returns
+    If this QDeadlineTimer never expires or the number of nanoseconds until the
+    deadline can't be accommodated in the return type, this function returns
     \c{std::numeric_limits<qint64>::max()}.
 
     This function can be used to calculate the amount of time a timer is
     overdue, by subtracting QDeadlineTimer::current(), as in the following
     example:
 
-    \code
-        qint64 realTimeLeft = deadline.deadlineNSecs();
-        if (realTimeLeft != std::numeric_limits<qint64>::max())
-            realTimeLeft -= QDeadlineTimer::current().deadlineNSecs();
-    \endcode
+    \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 7
 
     \note Timers that were created as expired have an indetermine time point in
     the past as their deadline, so the above calculation may not work.
 
-    \sa remainingTime(), deadlineNSecs()
+    \sa remainingTime(), deadline(), setDeadline()
 */
-qint64 QDeadlineTimer::deadlineNSecs() const Q_DECL_NOTHROW
+qint64 QDeadlineTimer::deadlineNSecs() const noexcept
 {
     if (isForever())
-        return t1;
-    if (QDeadlineTimerNanosecondsInT2)
-        return t1 * 1000 * 1000 * 1000 + t2;
+        return TimeReference::Max;
+
     return t1;
 }
 
@@ -528,18 +517,21 @@ qint64 QDeadlineTimer::deadlineNSecs() const Q_DECL_NOTHROW
     timerType. If the value is in the past, this QDeadlineTimer will be marked
     as expired.
 
-    If \a msecs is \c{std::numeric_limits<qint64>::max()}, this QDeadlineTimer
-    will be set to never expire.
+    If \a msecs is \c{std::numeric_limits<qint64>::max()} or the deadline is
+    beyond a representable point in the future, this QDeadlineTimer will be set
+    to never expire.
 
     \sa setPreciseDeadline(), deadline(), deadlineNSecs(), setRemainingTime()
 */
-void QDeadlineTimer::setDeadline(qint64 msecs, Qt::TimerType timerType) Q_DECL_NOTHROW
+void QDeadlineTimer::setDeadline(qint64 msecs, Qt::TimerType timerType) noexcept
 {
-    if (msecs == (std::numeric_limits<qint64>::max)()) {
-        setPreciseDeadline(msecs, 0, timerType);    // msecs == MAX implies Forever
-    } else {
-        setPreciseDeadline(msecs / 1000, msecs % 1000 * 1000 * 1000, timerType);
+    if (msecs == TimeReference::Max) {
+        *this = QDeadlineTimer(Forever, timerType);
+        return;
     }
+
+    type = timerType;
+    t1 = add_saturate(0, milliseconds{msecs});
 }
 
 /*!
@@ -554,17 +546,10 @@ void QDeadlineTimer::setDeadline(qint64 msecs, Qt::TimerType timerType) Q_DECL_N
 
     \sa setDeadline(), deadline(), deadlineNSecs(), setRemainingTime()
 */
-void QDeadlineTimer::setPreciseDeadline(qint64 secs, qint64 nsecs, Qt::TimerType timerType) Q_DECL_NOTHROW
+void QDeadlineTimer::setPreciseDeadline(qint64 secs, qint64 nsecs, Qt::TimerType timerType) noexcept
 {
     type = timerType;
-    if (secs == (std::numeric_limits<qint64>::max)() || nsecs == (std::numeric_limits<qint64>::max)()) {
-        *this = QDeadlineTimer(Forever, timerType);
-    } else if (QDeadlineTimerNanosecondsInT2) {
-        t1 = secs + toSecsAndNSecs(nsecs).first;
-        t2 = toSecsAndNSecs(nsecs).second;
-    } else {
-        t1 = secs * (1000*1000*1000) + nsecs;
-    }
+    t1 = add_saturate(0, seconds{secs}, nanoseconds{nsecs});
 }
 
 /*!
@@ -575,20 +560,12 @@ void QDeadlineTimer::setPreciseDeadline(qint64 secs, qint64 nsecs, Qt::TimerType
     \note if \a dt was created as expired, its deadline is indeterminate and
     adding an amount of time may or may not cause it to become unexpired.
 */
-QDeadlineTimer QDeadlineTimer::addNSecs(QDeadlineTimer dt, qint64 nsecs) Q_DECL_NOTHROW
+QDeadlineTimer QDeadlineTimer::addNSecs(QDeadlineTimer dt, qint64 nsecs) noexcept
 {
-    if (dt.isForever() || nsecs == (std::numeric_limits<qint64>::max)()) {
-        dt = QDeadlineTimer(Forever, dt.timerType());
-    } else if (QDeadlineTimerNanosecondsInT2) {
-        dt.t1 += toSecsAndNSecs(nsecs).first;
-        dt.t2 += toSecsAndNSecs(nsecs).second;
-        if (dt.t2 > 1000*1000*1000) {
-            dt.t2 -= 1000*1000*1000;
-            ++dt.t1;
-        }
-    } else {
-        dt.t1 += nsecs;
-    }
+    if (dt.isForever())
+        return dt;
+
+    dt.t1 = add_saturate(dt.t1, nanoseconds{nsecs});
     return dt;
 }
 
@@ -601,106 +578,117 @@ QDeadlineTimer QDeadlineTimer::addNSecs(QDeadlineTimer dt, qint64 nsecs) Q_DECL_
 
     The QDeadlineTimer object will be constructed with the specified \a timerType.
 */
+QDeadlineTimer QDeadlineTimer::current(Qt::TimerType timerType) noexcept
+{
+    // ensure we get nanoseconds; this will work so long as steady_clock's
+    // time_point isn't of finer resolution (picoseconds)
+    std::chrono::nanoseconds ns = std::chrono::steady_clock::now().time_since_epoch();
+
+    QDeadlineTimer result;
+    result.t1 = ns.count();
+    result.type = timerType;
+    return result;
+}
 
 /*!
-    \fn bool operator==(QDeadlineTimer d1, QDeadlineTimer d2)
-    \relates QDeadlineTimer
+    \fn bool QDeadlineTimer::operator==(const QDeadlineTimer &lhs, const QDeadlineTimer &rhs)
 
-    Returns true if the deadline on \a d1 and the deadline in \a d2 are the
+    Returns true if the deadline on \a lhs and the deadline in \a rhs are the
     same, false otherwise. The timer type used to create the two deadlines is
     ignored. This function is equivalent to:
 
-    \code
-        return d1.deadlineNSecs() == d2.deadlineNSecs();
-    \endcode
+    \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 8
 
     \note comparing QDeadlineTimer objects with different timer types is
     not supported and may result in unpredictable behavior.
 */
 
 /*!
-    \fn bool operator!=(QDeadlineTimer d1, QDeadlineTimer d2)
-    \relates QDeadlineTimer
+    \fn bool QDeadlineTimer::operator!=(const QDeadlineTimer &lhs, const QDeadlineTimer &rhs)
 
-    Returns true if the deadline on \a d1 and the deadline in \a d2 are
-    diferent, false otherwise. The timer type used to create the two deadlines
+    Returns true if the deadline on \a lhs and the deadline in \a rhs are
+    different, false otherwise. The timer type used to create the two deadlines
     is ignored. This function is equivalent to:
 
-    \code
-        return d1.deadlineNSecs() != d2.deadlineNSecs();
-    \endcode
+    \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 9
 
     \note comparing QDeadlineTimer objects with different timer types is
     not supported and may result in unpredictable behavior.
 */
 
 /*!
-    \fn bool operator<(QDeadlineTimer d1, QDeadlineTimer d2)
-    \relates QDeadlineTimer
+    \fn bool QDeadlineTimer::operator<(const QDeadlineTimer &lhs, const QDeadlineTimer &rhs)
 
-    Returns true if the deadline on \a d1 is earlier than the deadline in \a
-    d2, false otherwise. The timer type used to create the two deadlines is
+    Returns true if the deadline on \a lhs is earlier than the deadline in \a
+    rhs, false otherwise. The timer type used to create the two deadlines is
     ignored. This function is equivalent to:
 
-    \code
-        return d1.deadlineNSecs() < d2.deadlineNSecs();
-    \endcode
+    \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 10
 
     \note comparing QDeadlineTimer objects with different timer types is
     not supported and may result in unpredictable behavior.
 */
 
 /*!
-    \fn bool operator<=(QDeadlineTimer d1, QDeadlineTimer d2)
-    \relates QDeadlineTimer
+    \fn bool QDeadlineTimer::operator<=(const QDeadlineTimer &lhs, const QDeadlineTimer &rhs)
 
-    Returns true if the deadline on \a d1 is earlier than or the same as the
-    deadline in \a d2, false otherwise. The timer type used to create the two
+    Returns true if the deadline on \a lhs is earlier than or the same as the
+    deadline in \a rhs, false otherwise. The timer type used to create the two
     deadlines is ignored. This function is equivalent to:
 
-    \code
-        return d1.deadlineNSecs() <= d2.deadlineNSecs();
-    \endcode
+    \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 11
 
     \note comparing QDeadlineTimer objects with different timer types is
     not supported and may result in unpredictable behavior.
 */
 
 /*!
-    \fn bool operator>(QDeadlineTimer d1, QDeadlineTimer d2)
-    \relates QDeadlineTimer
+    \fn bool QDeadlineTimer::operator>(const QDeadlineTimer &lhs, const QDeadlineTimer &rhs)
 
-    Returns true if the deadline on \a d1 is later than the deadline in \a
-    d2, false otherwise. The timer type used to create the two deadlines is
+    Returns true if the deadline on \a lhs is later than the deadline in \a
+    rhs, false otherwise. The timer type used to create the two deadlines is
     ignored. This function is equivalent to:
 
-    \code
-        return d1.deadlineNSecs() > d2.deadlineNSecs();
-    \endcode
+    \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 12
 
     \note comparing QDeadlineTimer objects with different timer types is
     not supported and may result in unpredictable behavior.
 */
 
 /*!
-    \fn bool operator>=(QDeadlineTimer d1, QDeadlineTimer d2)
-    \relates QDeadlineTimer
+    \fn bool QDeadlineTimer::operator>=(const QDeadlineTimer &lhs, const QDeadlineTimer &rhs)
 
-    Returns true if the deadline on \a d1 is later than or the same as the
-    deadline in \a d2, false otherwise. The timer type used to create the two
+    Returns true if the deadline on \a lhs is later than or the same as the
+    deadline in \a rhs, false otherwise. The timer type used to create the two
     deadlines is ignored. This function is equivalent to:
 
-    \code
-        return d1.deadlineNSecs() >= d2.deadlineNSecs();
-    \endcode
+    \snippet code/src_corelib_kernel_qdeadlinetimer.cpp 13
 
     \note comparing QDeadlineTimer objects with different timer types is
     not supported and may result in unpredictable behavior.
 */
 
 /*!
-    \fn QDeadlineTimer operator+(QDeadlineTimer dt, qint64 msecs)
-    \relates QDeadlineTimer
+    \fn QDeadlineTimer QDeadlineTimer::operator+(QDeadlineTimer dt, qint64 msecs)
+
+    Returns a QDeadlineTimer object whose deadline is \a msecs later than the
+    deadline stored in \a dt. If \a dt is set to never expire, this function
+    returns a QDeadlineTimer that does not expire either.
+
+    To add times of precision greater than 1 millisecond, use addNSecs().
+*/
+
+QDeadlineTimer operator+(QDeadlineTimer dt, qint64 msecs)
+{
+    if (dt.isForever())
+        return dt;
+
+    dt.t1 = add_saturate(dt.t1, milliseconds{msecs});
+    return dt;
+}
+
+/*!
+    \fn QDeadlineTimer QDeadlineTimer::operator+(qint64 msecs, QDeadlineTimer dt)
 
     Returns a QDeadlineTimer object whose deadline is \a msecs later than the
     deadline stored in \a dt. If \a dt is set to never expire, this function
@@ -710,19 +698,7 @@ QDeadlineTimer QDeadlineTimer::addNSecs(QDeadlineTimer dt, qint64 nsecs) Q_DECL_
 */
 
 /*!
-    \fn QDeadlineTimer operator+(qint64 msecs, QDeadlineTimer dt)
-    \relates QDeadlineTimer
-
-    Returns a QDeadlineTimer object whose deadline is \a msecs later than the
-    deadline stored in \a dt. If \a dt is set to never expire, this function
-    returns a QDeadlineTimer that does not expire either.
-
-    To add times of precision greater than 1 millisecond, use addNSecs().
-*/
-
-/*!
-    \fn QDeadlineTimer operator-(QDeadlineTimer dt, qint64 msecs)
-    \relates QDeadlineTimer
+    \fn QDeadlineTimer QDeadlineTimer::operator-(QDeadlineTimer dt, qint64 msecs)
 
     Returns a QDeadlineTimer object whose deadline is \a msecs before the
     deadline stored in \a dt. If \a dt is set to never expire, this function
@@ -751,8 +727,7 @@ QDeadlineTimer QDeadlineTimer::addNSecs(QDeadlineTimer dt, qint64 nsecs) Q_DECL_
 
 /*!
   \fn void QDeadlineTimer::swap(QDeadlineTimer &other)
-
-  Swaps this deadline timer with the \a other deadline timer.
+    \memberswap{deadline timer}
  */
 
 /*!
@@ -773,6 +748,4 @@ QDeadlineTimer QDeadlineTimer::addNSecs(QDeadlineTimer dt, qint64 nsecs) Q_DECL_
   Returns the time remaining before the deadline.
  */
 
-// the rest of the functions are in qelapsedtimer_xxx.cpp
-
 QT_END_NAMESPACE
