@@ -28,7 +28,7 @@ QT_REQUIRE_CONFIG(permissions);
 
 QT_BEGIN_NAMESPACE
 
-Q_DECLARE_EXPORTED_LOGGING_CATEGORY(lcPermissions, Q_CORE_EXPORT)
+QT_DECLARE_EXPORTED_QT_LOGGING_CATEGORY(lcPermissions, Q_CORE_EXPORT)
 
 namespace QPermissions::Private
 {
