@@ -14,7 +14,7 @@
 
 QT_BEGIN_NAMESPACE
 
-Q_LOGGING_CATEGORY(lcQPropertyBinding, "qt.qproperty.binding");
+Q_STATIC_LOGGING_CATEGORY(lcQPropertyBinding, "qt.qproperty.binding");
 
 using namespace QtPrivate;
 
@@ -1079,7 +1079,7 @@ QString QPropertyBindingError::description() const
   long as the returned \c QPropertyChangeHandler and the property are kept alive.
   On each value change, the handler is either called immediately, or deferred, depending on the context.
 
-  \sa onValueChanged(), subscribe()
+  \sa QProperty::onValueChanged(), subscribe()
 */
 
 /*!
@@ -1176,6 +1176,7 @@ QString QPropertyBindingError::description() const
 
 /*!
    \fn template<typename T> QBindable<T>::QBindable(QObject *obj, const char *property)
+   \since 6.5
 
    Constructs a QBindable for the \l Q_PROPERTY \a property on \a obj. The property must
    have a notify signal but does not need to have \c BINDABLE in its \c Q_PROPERTY
@@ -1202,6 +1203,7 @@ QString QPropertyBindingError::description() const
 
 /*!
    \fn template<typename T> QBindable<T>::QBindable(QObject *obj, const QMetaProperty &property)
+   \since 6.5
 
    See \l QBindable::QBindable(QObject *obj, const char *property)
 */
@@ -1379,6 +1381,7 @@ QString QPropertyBindingError::description() const
 /*!
   \fn template <typename T> template <typename Functor> QPropertyBinding<T> QProperty<T>::setBinding(Functor f)
   \overload
+  \since 6.0
 
   Associates the value of this property with the provided functor \a f and
   returns the previously associated binding. The property's value is set to the
@@ -1436,6 +1439,7 @@ QString QPropertyBindingError::description() const
 
 /*!
   \fn template <typename T> template <typename Functor> QPropertyChangeHandler<T, Functor> QProperty<T>::subscribe(Functor f)
+  \since 6.0
 
   Subscribes the given functor \a f as a callback that is called immediately and
   whenever the value of the property changes in the future. On each value
@@ -1453,6 +1457,7 @@ QString QPropertyBindingError::description() const
 
 /*!
   \fn template <typename T> template <typename Functor> QPropertyNotifier QProperty<T>::addNotifier(Functor f)
+  \since 6.2
 
   Subscribes the given functor \a f as a callback that is called whenever
   the value of the property changes.
@@ -1939,6 +1944,7 @@ QString QPropertyBindingError::description() const
   \brief The QPropertyNotifier class controls the lifecycle of change callback installed on a QProperty.
 
   \ingroup tools
+  \since 6.2
 
   QPropertyNotifier is created when registering a callback on a QProperty to
   listen to changes to the property's value, using QProperty::addNotifier. As
