@@ -1,6 +1,7 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // Copyright (C) 2016 Intel Corporation.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 #include "qlocale_p.h"
 #include "qlocale_tools_p.h"
@@ -37,7 +38,7 @@ static auto scanLangEnv()
     {
         QByteArray name; // empty means unknown; lookup from id may work
         LCID id = 0; // 0 means unknown; lookup from name may work
-    } result;
+    };
     const QByteArray lang = qgetenv("LANG");
     if (lang.size() && (lang == "C" || qt_splitLocaleName(QString::fromLocal8Bit(lang)))) {
         // See if we have a Windows locale code instead of a locale name:
@@ -990,50 +991,51 @@ struct ByWindowsCode {
 
 static constexpr WindowsToISOListElt windows_to_iso_list[] = {
     { 0x0401, "ar_SA" },
-    { 0x0402, "bg\0  " },
-    { 0x0403, "ca\0  " },
+    { 0x0402, "bg"    },
+    { 0x0403, "ca"    },
     { 0x0404, "zh_TW" },
-    { 0x0405, "cs\0  " },
-    { 0x0406, "da\0  " },
-    { 0x0407, "de\0  " },
-    { 0x0408, "el\0  " },
+    { 0x0405, "cs"    },
+    { 0x0406, "da"    },
+    { 0x0407, "de"    },
+    { 0x0408, "el"    },
     { 0x0409, "en_US" },
-    { 0x040a, "es\0  " },
-    { 0x040b, "fi\0  " },
-    { 0x040c, "fr\0  " },
-    { 0x040d, "he\0  " },
-    { 0x040e, "hu\0  " },
-    { 0x040f, "is\0  " },
-    { 0x0410, "it\0  " },
-    { 0x0411, "ja\0  " },
-    { 0x0412, "ko\0  " },
-    { 0x0413, "nl\0  " },
-    { 0x0414, "no\0  " },
-    { 0x0415, "pl\0  " },
+    { 0x040a, "es"    },
+    { 0x040b, "fi"    },
+    { 0x040c, "fr"    },
+    { 0x040d, "he"    },
+    { 0x040e, "hu"    },
+    { 0x040f, "is"    },
+    { 0x0410, "it"    },
+    { 0x0411, "ja"    },
+    { 0x0412, "ko"    },
+    { 0x0413, "nl"    },
+    { 0x0414, "no"    },
+    { 0x0414, "nb"    }, // alternative spelling; lower_bound will find the first one
+    { 0x0415, "pl"    },
     { 0x0416, "pt_BR" },
-    { 0x0418, "ro\0  " },
-    { 0x0419, "ru\0  " },
-    { 0x041a, "hr\0  " },
-    { 0x041c, "sq\0  " },
-    { 0x041d, "sv\0  " },
-    { 0x041e, "th\0  " },
-    { 0x041f, "tr\0  " },
-    { 0x0420, "ur\0  " },
-    { 0x0421, "in\0  " },
-    { 0x0422, "uk\0  " },
-    { 0x0423, "be\0  " },
-    { 0x0425, "et\0  " },
-    { 0x0426, "lv\0  " },
-    { 0x0427, "lt\0  " },
-    { 0x0429, "fa\0  " },
-    { 0x042a, "vi\0  " },
-    { 0x042d, "eu\0  " },
-    { 0x042f, "mk\0  " },
-    { 0x0436, "af\0  " },
-    { 0x0438, "fo\0  " },
-    { 0x0439, "hi\0  " },
-    { 0x043e, "ms\0  " },
-    { 0x0458, "mt\0  " },
+    { 0x0418, "ro"    },
+    { 0x0419, "ru"    },
+    { 0x041a, "hr"    },
+    { 0x041c, "sq"    },
+    { 0x041d, "sv"    },
+    { 0x041e, "th"    },
+    { 0x041f, "tr"    },
+    { 0x0420, "ur"    },
+    { 0x0421, "in"    },
+    { 0x0422, "uk"    },
+    { 0x0423, "be"    },
+    { 0x0425, "et"    },
+    { 0x0426, "lv"    },
+    { 0x0427, "lt"    },
+    { 0x0429, "fa"    },
+    { 0x042a, "vi"    },
+    { 0x042d, "eu"    },
+    { 0x042f, "mk"    },
+    { 0x0436, "af"    },
+    { 0x0438, "fo"    },
+    { 0x0439, "hi"    },
+    { 0x043e, "ms"    },
+    { 0x0458, "mt"    },
     { 0x0801, "ar_IQ" },
     { 0x0804, "zh_CN" },
     { 0x0807, "de_CH" },
@@ -1041,19 +1043,20 @@ static constexpr WindowsToISOListElt windows_to_iso_list[] = {
     { 0x080a, "es_MX" },
     { 0x080c, "fr_BE" },
     { 0x0810, "it_CH" },
-    { 0x0812, "ko\0  " },
+    { 0x0812, "ko"    },
     { 0x0813, "nl_BE" },
-    { 0x0814, "no\0  " },
-    { 0x0816, "pt\0  " },
-    { 0x081a, "sr\0  " },
+    { 0x0814, "no"    },
+    { 0x0814, "nn"    }, // alternative spelling; lower_bound will find the first one
+    { 0x0816, "pt"    },
+    { 0x081a, "sr"    },
     { 0x081d, "sv_FI" },
     { 0x0c01, "ar_EG" },
     { 0x0c04, "zh_HK" },
     { 0x0c07, "de_AT" },
     { 0x0c09, "en_AU" },
-    { 0x0c0a, "es\0  " },
+    { 0x0c0a, "es"    },
     { 0x0c0c, "fr_CA" },
-    { 0x0c1a, "sr\0  " },
+    { 0x0c1a, "sr"    },
     { 0x1001, "ar_LY" },
     { 0x1004, "zh_SG" },
     { 0x1007, "de_LU" },
@@ -1075,7 +1078,7 @@ static constexpr WindowsToISOListElt windows_to_iso_list[] = {
     { 0x2009, "en_JM" },
     { 0x200a, "es_VE" },
     { 0x2401, "ar_YE" },
-    { 0x2409, "en\0  " },
+    { 0x2409, "en"    },
     { 0x240a, "es_CO" },
     { 0x2801, "ar_SY" },
     { 0x2809, "en_BZ" },
@@ -1126,15 +1129,13 @@ LCID qt_inIsoNametoLCID(const char *name)
 {
     if (!name)
         return LOCALE_USER_DEFAULT;
-    // handle norwegian manually, the list above will fail
-    if (!strncmp(name, "nb", 2))
-        return 0x0414;
-    if (!strncmp(name, "nn", 2))
-        return 0x0814;
+    if (std::strlen(name) >= sizeof(WindowsToISOListElt::iso_name))
+        return LOCALE_USER_DEFAULT; // cannot possibly match (too long)
 
-    char n[64];
+    // normalize separators:
+    char n[sizeof(WindowsToISOListElt::iso_name)];
+    // we know it will fit (we checked at the top of the function)
     strncpy(n, name, sizeof(n));
-    n[sizeof(n)-1] = 0;
     char *c = n;
     while (*c) {
         if (*c == '-')
@@ -1143,7 +1144,7 @@ LCID qt_inIsoNametoLCID(const char *name)
     }
 
     for (const WindowsToISOListElt &i : windows_to_iso_list) {
-        if (!strcmp(n, i.iso_name))
+        if (!memcmp(n, i.iso_name, sizeof(WindowsToISOListElt::iso_name)))
             return i.windows_code;
     }
     return LOCALE_USER_DEFAULT;
@@ -1198,7 +1199,6 @@ static QString winIso3116CtryName(LCID id)
 
 static QByteArray getWinLocaleName(LCID id)
 {
-    QByteArray result;
     if (id == LOCALE_USER_DEFAULT) {
         const auto [name, lcid] = scanLangEnv();
         if (!name.isEmpty())
