@@ -162,6 +162,12 @@ QT_BEGIN_NAMESPACE
     \sa reverse_iterator, const_iterator
 */
 
+/*!
+    \fn size_t qHash(QStringView key, size_t seed)
+    \since 5.10
+    \qhashold{QStringView}
+*/
+
 /*!
     \fn QStringView::QStringView()
 
@@ -179,7 +185,7 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \fn  template <typename Char> QStringView::QStringView(const Char *str, qsizetype len)
+    \fn template <typename Char, QStringView::if_compatible_char<Char> = true> QStringView::QStringView(const Char *str, qsizetype len)
 
     Constructs a string view on \a str with length \a len.
 
@@ -189,13 +195,15 @@ QT_BEGIN_NAMESPACE
 
     The behavior is undefined if \a len is negative or, when positive, if \a str is \nullptr.
 
-    This constructor only participates in overload resolution if \c Char is a compatible
+//! [compatible-char-types]
+    \constraints \c Char is a compatible
     character type. The compatible character types are: \c QChar, \c ushort, \c char16_t and
     (on platforms, such as Windows, where it is a 16-bit type) \c wchar_t.
+//! [compatible-char-types]
 */
 
 /*!
-    \fn template <typename Char> QStringView::QStringView(const Char *first, const Char *last)
+    \fn template <typename Char, QStringView::if_compatible_char<Char> = true> QStringView::QStringView(const Char *first, const Char *last)
 
     Constructs a string view on \a first with length (\a last - \a first).
 
@@ -208,10 +216,7 @@ QT_BEGIN_NAMESPACE
     The behavior is undefined if \a last precedes \a first, or \a first
     is \nullptr and \a last is not.
 
-    This constructor only participates in overload resolution if \c Char
-    is a compatible character type. The compatible character types
-    are: \c QChar, \c ushort, \c char16_t and (on platforms, such as
-    Windows, where it is a 16-bit type) \c wchar_t.
+    \include qstringview.cpp compatible-char-types
 */
 
 /*!
@@ -224,11 +229,7 @@ QT_BEGIN_NAMESPACE
 
     Passing \nullptr as \a str is safe and results in a null string view.
 
-    This constructor only participates in overload resolution if \a
-    str is not an array and if \c Char is a compatible character
-    type. The compatible character types are: \c QChar, \c ushort, \c
-    char16_t and (on platforms, such as Windows, where it is a 16-bit
-    type) \c wchar_t.
+    \include qstringview.cpp compatible-char-types
 */
 
 /*!
@@ -242,11 +243,7 @@ QT_BEGIN_NAMESPACE
     \a string must remain valid for the lifetime of this string view
     object.
 
-    This constructor only participates in overload resolution if \a
-    string is an actual array and \c Char is a compatible character
-    type. The compatible character types are: \c QChar, \c ushort, \c
-    char16_t and (on platforms, such as Windows, where it is a 16-bit
-    type) \c wchar_t.
+    \include qstringview.cpp compatible-char-types
 
     \sa fromArray
 */
@@ -262,26 +259,26 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \fn template <typename Container, if_compatible_container<Container>> QStringView::QStringView(const Container &str)
+    \fn template <typename Container, QStringView::if_compatible_container<Container>> QStringView::QStringView(const Container &str)
 
-    Constructs a string view on \a str. The length is taken from \c{str.size()}.
+    Constructs a string view on \a str. The length is taken from \c{std::size(str)}.
 
-    \c{str.data()} must remain valid for the lifetime of this string view object.
+    \c{std::data(str)} must remain valid for the lifetime of this string view object.
 
-    This constructor only participates in overload resolution if \c StdBasicString is an
-    instantiation of \c std::basic_string with a compatible character type. The
+    The string view will be empty if and only if \c{std::size(str) == 0}. It is unspecified
+    whether this constructor can result in a null string view (\c{std::data(str)} would
+    have to return \nullptr for this).
+
+    \constraints \c Container is a
+    container with a compatible character type as \c{value_type}. The
     compatible character types are: \c QChar, \c ushort, \c char16_t and
     (on platforms, such as Windows, where it is a 16-bit type) \c wchar_t.
 
-    The string view will be empty if and only if \c{str.empty()}. It is unspecified
-    whether this constructor can result in a null string view (\c{str.data()} would
-    have to return \nullptr for this).
-
     \sa isNull(), isEmpty()
 */
 
 /*!
-    \fn template <typename Char, size_t Size> static QStringView QStringView::fromArray(const Char (&string)[Size]) noexcept
+    \fn template <typename Char, size_t Size, QStringView::if_compatible_char<Char> = true> static QStringView QStringView::fromArray(const Char (&string)[Size]) noexcept
 
     Constructs a string view on the full character string literal \a string,
     including any trailing \c{Char(0)}. If you don't want the
@@ -310,9 +307,11 @@ QT_BEGIN_NAMESPACE
 /*!
     \fn const QChar *QStringView::data() const
 
+//! [const-pointer-to-first-ch]
     Returns a const pointer to the first character in the string view.
 
     \note The character array represented by the return value is \e not null-terminated.
+//! [const-pointer-to-first-ch]
 
     \sa begin(), end(), utf16()
 */
@@ -321,9 +320,7 @@ QT_BEGIN_NAMESPACE
     \fn const QChar *QStringView::constData() const
     \since 6.0
 
-    Returns a const pointer to the first character in the string view.
-
-    \note The character array represented by the return value is \e not null-terminated.
+    \include qstringview.cpp const-pointer-to-first-ch
 
     \sa data(), begin(), end(), utf16()
 */
@@ -331,12 +328,10 @@ QT_BEGIN_NAMESPACE
 /*!
     \fn const storage_type *QStringView::utf16() const
 
-    Returns a const pointer to the first character in the string view.
+    \include qstringview.cpp const-pointer-to-first-ch
 
     \c{storage_type} is \c{char16_t}.
 
-    \note The character array represented by the return value is \e not null-terminated.
-
     \sa begin(), end(), data()
 */
 
@@ -473,7 +468,7 @@ QT_BEGIN_NAMESPACE
 /*!
     \fn qsizetype QStringView::size() const
 
-    Returns the size of this string view, in UTF-16 code points (that is,
+    Returns the size of this string view, in UTF-16 code units (that is,
     surrogate pairs count as two for the purposes of this function, the same
     as in QString).
 
@@ -516,15 +511,18 @@ QT_BEGIN_NAMESPACE
     \fn template <typename...Args> QString QString::arg(Args &&...args) const
     \since 5.14
 
+//![qstring-multi-arg]
     Replaces occurrences of \c{%N} in this string with the corresponding
     argument from \a args. The arguments are not positional: the first of
     the \a args replaces the \c{%N} with the lowest \c{N} (all of them), the
     second of the \a args the \c{%N} with the next-lowest \c{N} etc.
 
-    \c Args can consist of anything that implicitly converts to QString,
-    QStringView or QLatin1StringView.
+    \c Args can consist of anything that implicitly converts to QAnyStringView.
+//![qstring-multi-arg]
 
-    In addition, the following types are also supported: QChar, QLatin1Char.
+    \note In Qt versions prior to 6.9, QAnyStringView and UTF-8 strings
+    (QUtf8StringView, QByteArray, QByteArrayView, \c{const char8_t*}, etc) were
+    not supported as \a args.
 
     \sa QString::arg()
 */
@@ -536,8 +534,10 @@ QT_BEGIN_NAMESPACE
 
     This function is provided for STL compatibility.
 
+//! [calling-on-empty-is-UB]
     \warning Calling this function on an empty string view constitutes
     undefined behavior.
+//! [calling-on-empty-is-UB]
 
     \sa back(), first(), last()
 */
@@ -549,8 +549,7 @@ QT_BEGIN_NAMESPACE
 
     This function is provided for STL compatibility.
 
-    \warning Calling this function on an empty string view constitutes
-    undefined behavior.
+    \include qstringview.cpp calling-on-empty-is-UB
 
     \sa front(), first(), last()
 */
@@ -562,8 +561,7 @@ QT_BEGIN_NAMESPACE
 
     This function is provided for compatibility with other Qt containers.
 
-    \warning Calling this function on an empty string view constitutes
-    undefined behavior.
+    \include qstringview.cpp calling-on-empty-is-UB
 
     \sa front(), back(), last()
 */
@@ -575,8 +573,7 @@ QT_BEGIN_NAMESPACE
 
     This function is provided for compatibility with other Qt containers.
 
-    \warning Calling this function on an empty string view constitutes
-    undefined behavior.
+    \include qstringview.cpp calling-on-empty-is-UB
 
     \sa back(), front(), first()
 */
@@ -595,7 +592,7 @@ QT_BEGIN_NAMESPACE
     \a length is negative (default), the function returns all characters that
     are available from \a start.
 
-    \sa first(), last(), sliced(), chopped(), chop(), truncate()
+    \sa first(), last(), sliced(), chopped(), chop(), truncate(), slice()
 */
 
 /*!
@@ -609,7 +606,7 @@ QT_BEGIN_NAMESPACE
     The entire string view is returned if \a length is greater than or equal
     to size(), or less than zero.
 
-    \sa first(), last(), sliced(), startsWith(), chopped(), chop(), truncate()
+    \sa first(), last(), sliced(), startsWith(), chopped(), chop(), truncate(), slice()
 */
 
 /*!
@@ -623,7 +620,7 @@ QT_BEGIN_NAMESPACE
     The entire string view is returned if \a length is greater than or equal
     to size(), or less than zero.
 
-    \sa first(), last(), sliced(), endsWith(), chopped(), chop(), truncate()
+    \sa first(), last(), sliced(), endsWith(), chopped(), chop(), truncate(), slice()
 */
 
 /*!
@@ -635,7 +632,7 @@ QT_BEGIN_NAMESPACE
 
     \note The behavior is undefined when \a n < 0 or \a n > size().
 
-    \sa last(), sliced(), startsWith(), chopped(), chop(), truncate()
+    \sa last(), sliced(), startsWith(), chopped(), chop(), truncate(), slice()
 */
 
 /*!
@@ -647,7 +644,7 @@ QT_BEGIN_NAMESPACE
 
     \note The behavior is undefined when \a n < 0 or \a n > size().
 
-    \sa first(), sliced(), endsWith(), chopped(), chop(), truncate()
+    \sa first(), sliced(), endsWith(), chopped(), chop(), truncate(), slice()
 */
 
 /*!
@@ -657,10 +654,12 @@ QT_BEGIN_NAMESPACE
     Returns a string view that points to \a n characters of this string view,
     starting at position \a pos.
 
+//! [UB-sliced-index-length]
     \note The behavior is undefined when \a pos < 0, \a n < 0,
     or \a pos + \a n > size().
+//! [UB-sliced-index-length]
 
-    \sa first(), last(), chopped(), chop(), truncate()
+    \sa first(), last(), chopped(), chop(), truncate(), slice()
 */
 
 /*!
@@ -671,9 +670,36 @@ QT_BEGIN_NAMESPACE
     Returns a string view starting at position \a pos in this object,
     and extending to its end.
 
+//! [UB-sliced-index-only]
     \note The behavior is undefined when \a pos < 0 or \a pos > size().
+//! [UB-sliced-index-only]
 
-    \sa first(), last(), chopped(), chop(), truncate()
+    \sa first(), last(), chopped(), chop(), truncate(), slice()
+*/
+
+/*!
+    \fn QStringView &QStringView::slice(qsizetype pos, qsizetype n)
+    \since 6.8
+
+    Modifies this string view to start from position \a pos, extending
+    for \a n code points.
+
+    \include qstringview.cpp UB-sliced-index-length
+
+    \sa sliced(), first(), last(), chopped(), chop(), truncate()
+*/
+
+/*!
+    \fn QStringView &QStringView::slice(qsizetype pos)
+    \since 6.8
+    \overload
+
+    Modifies this string view to start from position \a pos, extending
+    to its end.
+
+    \include qstringview.cpp UB-sliced-index-only
+
+    \sa sliced(), first(), last(), chopped(), chop(), truncate()
 */
 
 /*!
@@ -686,7 +712,7 @@ QT_BEGIN_NAMESPACE
 
     \note The behavior is undefined when \a length < 0 or \a length > size().
 
-    \sa mid(), left(), right(), chop(), truncate()
+    \sa sliced(), left(), right(), chop(), truncate(), slice()
 */
 
 /*!
@@ -698,7 +724,7 @@ QT_BEGIN_NAMESPACE
 
     \note The behavior is undefined when \a length < 0 or \a length > size().
 
-    \sa mid(), left(), right(), chopped(), chop()
+    \sa sliced(), left(), right(), chopped(), chop()
 */
 
 /*!
@@ -710,7 +736,7 @@ QT_BEGIN_NAMESPACE
 
     \note The behavior is undefined when \a length < 0 or \a length > size().
 
-    \sa mid(), left(), right(), chopped(), truncate()
+    \sa sliced(), left(), right(), chopped(), truncate(), slice()
 */
 
 /*!
@@ -727,11 +753,24 @@ QT_BEGIN_NAMESPACE
     \fn int QStringView::compare(QStringView str, Qt::CaseSensitivity cs) const
     \since 5.12
 
-    Returns an integer that compares to zero as this string view compares to the
-    string view \a str.
+    Compares this string view with string view \a str and returns a negative integer if
+    this string view is less than \a str, a positive integer if it is greater than
+    \a str, and zero if they are equal.
 
-    If \a cs is Qt::CaseSensitive (the default), the comparison is case sensitive;
-    otherwise the comparison is case-insensitive.
+    \include qstring.qdocinc {search-comparison-case-sensitivity} {comparison}
+
+    \sa operator==(), operator<(), operator>()
+*/
+
+/*!
+    \fn int QStringView::compare(QUtf8StringView str, Qt::CaseSensitivity cs) const
+    \since 6.5
+
+    Compares this string view with QUtf8StringView \a str and returns a negative integer if
+    this string view is less than \a str, a positive integer if it is greater than
+    \a str, and zero if they are equal.
+
+    \include qstring.qdocinc {search-comparison-case-sensitivity} {comparison}
 
     \sa operator==(), operator<(), operator>()
 */
@@ -742,22 +781,22 @@ QT_BEGIN_NAMESPACE
     \fn int QStringView::compare(QChar ch, Qt::CaseSensitivity cs) const
     \since 5.15
 
-    Returns an integer that compares to zero as this string view compares to the
-    Latin-1 string \a l1, or character \a ch, respectively.
+    Compares this string view to the Latin-1 string view \a l1, or the character \a ch.
+    Returns a negative integer if this string view is less than \a l1 or \a ch,
+    a positive integer if it is greater than \a l1 or \a ch, and zero if they are equal.
 
-    If \a cs is Qt::CaseSensitive (the default), the comparison is case sensitive;
-    otherwise the comparison is case-insensitive.
+    \include qstring.qdocinc {search-comparison-case-sensitivity} {comparison}
 
     \sa operator==(), operator<(), operator>()
 */
 
 /*!
-    \fn QStringView::operator==(QStringView lhs, QStringView rhs)
-    \fn QStringView::operator!=(QStringView lhs, QStringView rhs)
-    \fn QStringView::operator< (QStringView lhs, QStringView rhs)
-    \fn QStringView::operator<=(QStringView lhs, QStringView rhs)
-    \fn QStringView::operator> (QStringView lhs, QStringView rhs)
-    \fn QStringView::operator>=(QStringView lhs, QStringView rhs)
+    \fn QStringView::operator==(const QStringView &lhs, const QStringView &rhs)
+    \fn QStringView::operator!=(const QStringView &lhs, const QStringView &rhs)
+    \fn QStringView::operator< (const QStringView &lhs, const QStringView &rhs)
+    \fn QStringView::operator<=(const QStringView &lhs, const QStringView &rhs)
+    \fn QStringView::operator> (const QStringView &lhs, const QStringView &rhs)
+    \fn QStringView::operator>=(const QStringView &lhs, const QStringView &rhs)
 
     Operators for comparing \a lhs to \a rhs.
 
@@ -779,18 +818,24 @@ QT_BEGIN_NAMESPACE
     \sa {Comparing Strings}
 */
 
+/*
+//! [utf16-or-latin1-or-ch]
+the UTF-16 string viewed by \a str, the Latin-1 string viewed by \a l1,
+or the character \a ch
+//! [utf16-or-latin1-or-ch]
+*/
+
 /*!
     \fn bool QStringView::startsWith(QStringView str, Qt::CaseSensitivity cs) const
     \fn bool QStringView::startsWith(QLatin1StringView l1, Qt::CaseSensitivity cs) const
     \fn bool QStringView::startsWith(QChar ch) const
     \fn bool QStringView::startsWith(QChar ch, Qt::CaseSensitivity cs) const
 
-    Returns \c true if this string view starts with string view \a str,
-    Latin-1 string \a l1, or character \a ch, respectively;
-    otherwise returns \c false.
+    Returns \c true if this string view starts with
+    \include qstringview.cpp utf16-or-latin1-or-ch
+    respectively; otherwise returns \c false.
 
-    If \a cs is Qt::CaseSensitive (the default), the search is case-sensitive;
-    otherwise the search is case-insensitive.
+    \include qstring.qdocinc {search-comparison-case-sensitivity} {search}
 
     \sa endsWith()
 */
@@ -801,12 +846,11 @@ QT_BEGIN_NAMESPACE
     \fn bool QStringView::endsWith(QChar ch) const
     \fn bool QStringView::endsWith(QChar ch, Qt::CaseSensitivity cs) const
 
-    Returns \c true if this string view ends with string view \a str,
-    Latin-1 string \a l1, or character \a ch, respectively;
-    otherwise returns \c false.
+    Returns \c true if this string view ends with
+    \include qstringview.cpp utf16-or-latin1-or-ch
+    respectively; otherwise returns \c false.
 
-    If \a cs is Qt::CaseSensitive (the default), the search is case-sensitive;
-    otherwise the search is case-insensitive.
+     \include qstring.qdocinc {search-comparison-case-sensitivity} {search}
 
     \sa startsWith()
 */
@@ -814,18 +858,17 @@ QT_BEGIN_NAMESPACE
 /*!
     \fn qsizetype QStringView::indexOf(QStringView str, qsizetype from = 0, Qt::CaseSensitivity cs = Qt::CaseSensitive) const
     \fn qsizetype QStringView::indexOf(QLatin1StringView l1, qsizetype from = 0, Qt::CaseSensitivity cs = Qt::CaseSensitive) const
-    \fn qsizetype QStringView::indexOf(QChar c, qsizetype from = 0, Qt::CaseSensitivity cs = Qt::CaseSensitive) const
+    \fn qsizetype QStringView::indexOf(QChar ch, qsizetype from = 0, Qt::CaseSensitivity cs = Qt::CaseSensitive) const
     \since 5.14
 
-    Returns the index position of the first occurrence of the string view \a str,
-    Latin-1 string \a l1, or character \a ch, respectively, in this string view,
-    searching forward from index position \a from. Returns -1 if \a str is not found.
+    Returns the index position of the first occurrence of
+    \include qstringview.cpp utf16-or-latin1-or-ch
+    respectively, in this string view, searching forward from index position
+    \a from. Returns -1 if \a str, \a l1 or \a ch is not found, respectively.
 
-    If \a cs is Qt::CaseSensitive (default), the search is case
-    sensitive; otherwise the search is case insensitive.
+    \include qstring.qdocinc {search-comparison-case-sensitivity} {search}
 
-    If \a from is -1, the search starts at the last character; if it is
-    -2, at the next to last character and so on.
+    \include qstring.qdocinc negative-index-start-search-from-end
 
     \sa QString::indexOf()
 */
@@ -833,14 +876,14 @@ QT_BEGIN_NAMESPACE
 /*!
     \fn bool QStringView::contains(QStringView str, Qt::CaseSensitivity cs) const
     \fn bool QStringView::contains(QLatin1StringView l1, Qt::CaseSensitivity cs) const
-    \fn bool QStringView::contains(QChar c, Qt::CaseSensitivity cs) const
+    \fn bool QStringView::contains(QChar ch, Qt::CaseSensitivity cs) const
     \since 5.14
 
-    Returns \c true if this string view contains an occurrence of the string view
-    \a str, Latin-1 string \a l1, or character \a ch; otherwise returns \c false.
+    Returns \c true if this string view contains an occurrence of
+    \include qstringview.cpp utf16-or-latin1-or-ch
+    respectively; otherwise returns \c false.
 
-    If \a cs is Qt::CaseSensitive (the default), the search is
-    case-sensitive; otherwise the search is case-insensitive.
+    \include qstring.qdocinc {search-comparison-case-sensitivity} {search}
 
     \sa indexOf()
 */
@@ -848,17 +891,19 @@ QT_BEGIN_NAMESPACE
 /*!
     \fn qsizetype QStringView::lastIndexOf(QStringView str, qsizetype from, Qt::CaseSensitivity cs) const
     \fn qsizetype QStringView::lastIndexOf(QLatin1StringView l1, qsizetype from, Qt::CaseSensitivity cs) const
-    \fn qsizetype QStringView::lastIndexOf(QChar c, qsizetype from, Qt::CaseSensitivity cs) const
+    \fn qsizetype QStringView::lastIndexOf(QChar ch, qsizetype from, Qt::CaseSensitivity cs) const
     \since 5.14
 
-    Returns the index position of the last occurrence of the string view \a str,
-    Latin-1 string \a l1, or character \a ch, respectively, in this string view,
-    searching backward from index position \a from. If \a from is -1,
-    the search starts at the last character; if \a from is -2, at the next to last
-    character and so on. Returns -1 if \a str is not found.
+    Returns the index position of the last occurrence of
+    \include qstringview.cpp utf16-or-latin1-or-ch
+    respectively, in this string view, searching backward from index
+    position \a from.
 
-    If \a cs is Qt::CaseSensitive (default), the search is case
-    sensitive; otherwise the search is case insensitive.
+    \include qstring.qdocinc negative-index-start-search-from-end
+
+    Returns -1 if \a str, \a l1 or \a ch is not found, respectively.
+
+    \include qstring.qdocinc {search-comparison-case-sensitivity} {search}
 
     \note When searching for a 0-length \a str or \a l1, the match at
     the end of the data is excluded from the search by a negative \a
@@ -877,19 +922,18 @@ QT_BEGIN_NAMESPACE
     \since 6.2
     \overload lastIndexOf()
 
-    Returns the index position of the last occurrence of the string view \a str
-    or Latin-1 string \a l1, respectively, in this string view,
-    searching backward from the last character of this string view.
-    Returns -1 if \a str is not found.
+    Returns the index position of the last occurrence of the UTF-16 string viewed
+    by \a str or the Latin-1 string viewed by \a l1 respectively, in this string
+    view searching backward from the last character of this string view. Returns
+    -1 if \a str or \a l1 is not found, respectively.
 
-    If \a cs is Qt::CaseSensitive (default), the search is case
-    sensitive; otherwise the search is case insensitive.
+    \include qstring.qdocinc {search-comparison-case-sensitivity} {search}
 
     \sa QString::lastIndexOf()
 */
 
 /*!
-    \fn QStringView::lastIndexOf(QChar c, Qt::CaseSensitivity cs) const
+    \fn QStringView::lastIndexOf(QChar ch, Qt::CaseSensitivity cs) const
     \since 6.3
     \overload lastIndexOf()
 */
@@ -918,9 +962,11 @@ QT_BEGIN_NAMESPACE
 
     Returns the index position of the last match of the regular
     expression \a re in the string view, which starts before the index
-    position \a from. If \a from is -1, the search starts at the last
-    character; if \a from is -2, at the next to last character and so
-    on. Returns -1 if \a re didn't match anywhere.
+    position \a from.
+
+    \include qstring.qdocinc negative-index-start-search-from-end
+
+    Returns -1 if \a re didn't match anywhere.
 
     If the match is successful and \a rmatch is not \nullptr, it also
     writes the results of the match into the QRegularExpressionMatch object
@@ -1075,6 +1121,32 @@ QT_BEGIN_NAMESPACE
     \sa QString::isValidUtf16()
 */
 
+/*!
+    \fn bool QStringView::isLower() const
+    \since 6.7
+    Returns \c true if this view is identical to its lowercase folding.
+
+    Note that this does \e not mean that the string view does not contain
+    uppercase letters (some uppercase letters do not have a lowercase
+    folding; they are left unchanged by toString().toLower()).
+    For more information, refer to the Unicode standard, section 3.13.
+
+    \sa QChar::toLower(), isUpper()
+*/
+
+/*!
+    \fn bool QStringView::isUpper() const
+    \since 6.7
+    Returns \c true if this view is identical to its uppercase folding.
+
+    Note that this does \e not mean that the the string view does not contain
+    lowercase letters (some lowercase letters do not have a uppercase
+    folding; they are left unchanged by toString().toUpper()).
+    For more information, refer to the Unicode standard, section 3.13.
+
+    \sa QChar::toUpper(), isLower()
+*/
+
 /*!
     \fn QStringView::toWCharArray(wchar_t *array) const
     \since 5.14
@@ -1104,8 +1176,7 @@ QT_BEGIN_NAMESPACE
     Returns the number of occurrences of the character \a ch in the
     string view.
 
-    If \a cs is Qt::CaseSensitive (default), the search is
-    case sensitive; otherwise the search is case insensitive.
+    \include qstring.qdocinc {search-comparison-case-sensitivity} {search}
 
     \sa QString::count(), contains(), indexOf()
 */
@@ -1119,8 +1190,7 @@ QT_BEGIN_NAMESPACE
     Returns the number of (potentially overlapping) occurrences of the
     string view \a str in this string view.
 
-    If \a cs is Qt::CaseSensitive (default), the search is
-    case sensitive; otherwise the search is case insensitive.
+    \include qstring.qdocinc {search-comparison-case-sensitivity} {search}
 
     \sa QString::count(), contains(), indexOf()
 */
@@ -1132,10 +1202,9 @@ QT_BEGIN_NAMESPACE
     \overload count()
 
     Returns the number of (potentially overlapping) occurrences of the
-    Latin-1 string \a l1 in this string view.
+    Latin-1 string viewed by \a l1 in this string view.
 
-    If \a cs is Qt::CaseSensitive (default), the search is
-    case sensitive; otherwise the search is case insensitive.
+    \include qstring.qdocinc {search-comparison-case-sensitivity} {search}
 
     \sa QString::count(), contains(), indexOf()
 */
@@ -1400,4 +1469,31 @@ QT_BEGIN_NAMESPACE
     \sa QStringTokenizer, qTokenize()
 */
 
+/*!
+    \fn QStringView::operator std::u16string_view() const
+    \since 6.7
+
+    Converts this QStringView object to a \c{std::u16string_view} object.
+    The returned view will have the same data pointer and length of
+    this view.
+*/
+
+/*!
+    \fn QStringView::maxSize()
+    \since 6.8
+
+    It returns the maximum number of elements that the view can
+    theoretically represent. In practice, the number can be much smaller,
+    limited by the amount of memory available to the system.
+*/
+
+/*!
+    \fn QStringView::max_size() const
+    \since 6.8
+
+    This function is provided for STL compatibility.
+
+    Returns maxSize().
+*/
+
 QT_END_NAMESPACE
