@@ -1,5 +1,6 @@
 // Copyright (C) 2020 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 // qfutureinterface.h included from qfuture.h
 #include "qfuture.h"
@@ -12,6 +13,8 @@
 #include <private/qthreadpool_p.h>
 #include <private/qobject_p.h>
 
+#include <climits> // For INT_MAX
+
 // GCC 12 gets confused about QFutureInterfaceBase::state, for some non-obvious
 // reason
 //  warning: ‘unsigned int __atomic_or_fetch_4(volatile void*, unsigned int, int)’ writing 4 bytes into a region of size 0 overflows the destination [-Wstringop-overflow=]
@@ -886,6 +889,7 @@ void QFutureInterfaceBase::setContinuation(std::function<void(const QFutureInter
     // If the state is ready, run continuation immediately,
     // otherwise save it for later.
     if (isFinished()) {
+        d->continuationExecuted = true;
         lock.unlock();
         func(*this);
         lock.relock();
@@ -895,8 +899,8 @@ void QFutureInterfaceBase::setContinuation(std::function<void(const QFutureInter
     // future's data stays alive.
     if (d->continuationState != QFutureInterfaceBasePrivate::Cleaned) {
         if (d->continuation) {
-            qWarning() << "Adding a continuation to a future which already has a continuation. "
-                          "The existing continuation is overwritten.";
+            qWarning("Adding a continuation to a future which already has a continuation. "
+                     "The existing continuation is overwritten.");
         }
         d->continuation = std::move(func);
         d->continuationData = continuationFutureData;
@@ -917,10 +921,11 @@ void QFutureInterfaceBase::cleanContinuation()
 void QFutureInterfaceBase::runContinuation() const
 {
     QMutexLocker lock(&d->continuationMutex);
-    if (d->continuation) {
+    if (d->continuation && !d->continuationExecuted) {
         // Save the continuation in a local function, to avoid calling
         // a null std::function below, in case cleanContinuation() is
         // called from some other thread right after unlock() below.
+        d->continuationExecuted = true;
         auto fn = std::move(d->continuation);
         lock.unlock();
         fn(*this);
