@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 #include "private/qbmphandler_p.h"
 
@@ -608,7 +609,7 @@ bool qt_write_dib(QDataStream &s, const QImage &image, int bpl, int bpl_bmp, int
     if (image.depth() != 32) {                // write color table
         uchar *color_table = new uchar[4*image.colorCount()];
         uchar *rgb = color_table;
-        QList<QRgb> c = image.colorTable();
+        const QList<QRgb> c = image.colorTable();
         for (int i = 0; i < image.colorCount(); i++) {
             *rgb++ = qBlue (c[i]);
             *rgb++ = qGreen(c[i]);
