@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include <qglobal.h>
 
@@ -54,8 +18,9 @@
 
 QT_BEGIN_NAMESPACE
 
-Q_LOGGING_CATEGORY(lcQpaKeyMapper, "qt.qpa.keymapper");
-Q_LOGGING_CATEGORY(lcQpaKeyMapperKeys, "qt.qpa.keymapper.keys");
+#ifdef Q_OS_MACOS
+Q_STATIC_LOGGING_CATEGORY(lcQpaKeyMapperKeys, "qt.qpa.keymapper.keys");
+#endif
 
 static Qt::KeyboardModifiers swapModifiersIfNeeded(const Qt::KeyboardModifiers modifiers)
 {
@@ -73,36 +38,6 @@ static Qt::KeyboardModifiers swapModifiersIfNeeded(const Qt::KeyboardModifiers m
     return swappedModifiers;
 }
 
-Qt::Key QAppleKeyMapper::fromNSString(Qt::KeyboardModifiers qtModifiers, NSString *characters,
-                                      NSString *charactersIgnoringModifiers, QString &text)
-{
-    if ([characters isEqualToString:@"\t"]) {
-        if (qtModifiers & Qt::ShiftModifier)
-            return Qt::Key_Backtab;
-        return Qt::Key_Tab;
-    } else if ([characters isEqualToString:@"\r"]) {
-        if (qtModifiers & Qt::KeypadModifier)
-            return Qt::Key_Enter;
-        return Qt::Key_Return;
-    }
-    if ([characters length] != 0 || [charactersIgnoringModifiers length] != 0) {
-        QChar ch;
-        if (((qtModifiers & Qt::MetaModifier) || (qtModifiers & Qt::AltModifier)) &&
-            ([charactersIgnoringModifiers length] != 0)) {
-            ch = QChar([charactersIgnoringModifiers characterAtIndex:0]);
-        } else if ([characters length] != 0) {
-            ch = QChar([characters characterAtIndex:0]);
-        }
-        if (!(qtModifiers & (Qt::ControlModifier | Qt::MetaModifier)) &&
-            (ch.unicode() < 0xf700 || ch.unicode() > 0xf8ff)) {
-            text = QString::fromNSString(characters);
-        }
-        if (!ch.isNull())
-            return Qt::Key(ch.toUpper().unicode());
-    }
-    return Qt::Key_unknown;
-}
-
 #ifdef Q_OS_MACOS
 static constexpr std::tuple<NSEventModifierFlags, Qt::KeyboardModifier> cocoaModifierMap[] = {
     { NSEventModifierFlagShift, Qt::ShiftModifier },
@@ -276,7 +211,7 @@ static int toKeyCode(const QChar &key, int virtualKey, int modifiers)
     qCDebug(lcQpaKeyMapperKeys, "Mapping key: %d (0x%04x) / vk %d (0x%04x)",
         key.unicode(), key.unicode(), virtualKey, virtualKey);
 
-    if (key == QChar(kClearCharCode) && virtualKey == 0x47)
+    if (key == char16_t(kClearCharCode) && virtualKey == 0x47)
         return Qt::Key_Clear;
 
     if (key.isDigit()) {
@@ -311,11 +246,11 @@ static int toKeyCode(const QChar &key, int virtualKey, int modifiers)
     }
 
     // Check if they belong to key codes in private unicode range
-    if (key >= QChar(NSUpArrowFunctionKey) && key <= QChar(NSModeSwitchFunctionKey)) {
+    if (key >= char16_t(NSUpArrowFunctionKey) && key <= char16_t(NSModeSwitchFunctionKey)) {
         if (auto qtKey = functionKeys.value(key.unicode())) {
             qCDebug(lcQpaKeyMapperKeys) << "Got" << qtKey;
             return qtKey;
-        } else if (key >= QChar(NSF1FunctionKey) && key <= QChar(NSF35FunctionKey)) {
+        } else if (key >= char16_t(NSF1FunctionKey) && key <= char16_t(NSF35FunctionKey)) {
             auto functionKey = Qt::Key_F1 + (key.unicode() - NSF1FunctionKey) ;
             qCDebug(lcQpaKeyMapperKeys) << "Got" << functionKey;
             return functionKey;
@@ -396,11 +331,11 @@ QChar QAppleKeyMapper::toCocoaKey(Qt::Key key)
 {
     // Prioritize overloaded keys
     if (key == Qt::Key_Return)
-        return QChar(NSNewlineCharacter);
+        return char16_t(NSCarriageReturnCharacter);
     if (key == Qt::Key_Backspace)
-        return QChar(NSBackspaceCharacter);
+        return char16_t(NSBackspaceCharacter);
 
-    static QHash<Qt::Key, char16_t> reverseCocoaKeys;
+    Q_CONSTINIT static QHash<Qt::Key, char16_t> reverseCocoaKeys;
     if (reverseCocoaKeys.isEmpty()) {
         reverseCocoaKeys.reserve(cocoaKeys.size());
         for (auto it = cocoaKeys.begin(); it != cocoaKeys.end(); ++it)
@@ -420,7 +355,7 @@ Qt::Key QAppleKeyMapper::fromCocoaKey(QChar keyCode)
 
 // ------------------------------------------------
 
-Qt::KeyboardModifiers QAppleKeyMapper::queryKeyboardModifiers()
+Qt::KeyboardModifiers QAppleKeyMapper::queryKeyboardModifiers() const
 {
     return fromCocoaModifiers(NSEvent.modifierFlags);
 }
@@ -522,23 +457,21 @@ const QAppleKeyMapper::KeyMap &QAppleKeyMapper::keyMapForKey(VirtualKeyCode virt
         if (err == noErr && actualStringLength)
             carbonUnicodeKey = QChar(unicodeString[0]);
 
-        if (@available(macOS 10.15, *)) {
-            if (canMapCocoaEvent) {
-                // Until we've verified that the Cocoa API works as expected
-                // we first run the event through the Carbon APIs and then
-                // compare the results to Cocoa.
-                auto cocoaModifiers = toCocoaModifiers(qtModifiers);
-                auto *charactersWithModifiers = [NSApp.currentEvent charactersByApplyingModifiers:cocoaModifiers];
+        if (canMapCocoaEvent) {
+            // Until we've verified that the Cocoa API works as expected
+            // we first run the event through the Carbon APIs and then
+            // compare the results to Cocoa.
+            auto cocoaModifiers = toCocoaModifiers(qtModifiers);
+            auto *charactersWithModifiers = [NSApp.currentEvent charactersByApplyingModifiers:cocoaModifiers];
 
-                QChar cocoaUnicodeKey;
-                if (charactersWithModifiers.length > 0)
-                    cocoaUnicodeKey = QChar([charactersWithModifiers characterAtIndex:0]);
+            QChar cocoaUnicodeKey;
+            if (charactersWithModifiers.length > 0)
+                cocoaUnicodeKey = QChar([charactersWithModifiers characterAtIndex:0]);
 
-                if (cocoaUnicodeKey != carbonUnicodeKey) {
-                    qCWarning(lcQpaKeyMapper) << "Mismatch between Cocoa" << cocoaUnicodeKey
-                        << "and Carbon" << carbonUnicodeKey << "for virtual key" << virtualKey
-                        << "with" << qtModifiers;
-                }
+            if (cocoaUnicodeKey != carbonUnicodeKey) {
+                qCWarning(lcQpaKeyMapper) << "Mismatch between Cocoa" << cocoaUnicodeKey
+                    << "and Carbon" << carbonUnicodeKey << "for virtual key" << virtualKey
+                    << "with" << qtModifiers;
             }
         }
 
@@ -551,7 +484,7 @@ const QAppleKeyMapper::KeyMap &QAppleKeyMapper::keyMapForKey(VirtualKeyCode virt
         qCDebug(lcQpaKeyMapper).verbosity(0) << "\t" << qtModifiers
             << "+" << qUtf8Printable(QString::asprintf("0x%02x", virtualKey))
             << "=" << qUtf8Printable(QString::asprintf("%d / 0x%02x /", qtKey, qtKey))
-                   << QString::asprintf("%c", qtKey);
+                   << QKeySequence(qtKey).toString();
     }
 
     return keyMap;
@@ -574,9 +507,9 @@ const QAppleKeyMapper::KeyMap &QAppleKeyMapper::keyMapForKey(VirtualKeyCode virt
     where each modifier-key combination has been mapped to the
     key it will produce.
 */
-QList<int> QAppleKeyMapper::possibleKeys(const QKeyEvent *event) const
+QList<QKeyCombination> QAppleKeyMapper::possibleKeyCombinations(const QKeyEvent *event) const
 {
-    QList<int> ret;
+    QList<QKeyCombination> ret;
 
     const auto nativeVirtualKey = event->nativeVirtualKey();
     if (!nativeVirtualKey)
@@ -589,16 +522,54 @@ QList<int> QAppleKeyMapper::possibleKeys(const QKeyEvent *event) const
 
     auto eventModifiers = event->modifiers();
 
-    // The complete set of event modifiers, along with the
-    // unmodified key, is always a valid key combination,
-    // and the first priority.
-    ret << int(eventModifiers) + int(unmodifiedKey);
+    int startingModifierLayer = 0;
+    if (toCocoaModifiers(eventModifiers) & NSEventModifierFlagCommand) {
+        // When the Command key is pressed AppKit seems to do key equivalent
+        // matching using a Latin/Roman interpretation of the current keyboard
+        // layout. For example, for a Greek layout, pressing Option+Command+C
+        // produces a key event with chars="ç" and unmodchars="ψ", but AppKit
+        // still treats this as a match for a key equivalent of Option+Command+C.
+        // We can't do the same by just applying the modifiers to our key map,
+        // as that too contains "ψ" for the Option+Command combination. What we
+        // can do instead is take advantage of the fact that the Command
+        // modifier layer in all/most keyboard layouts contains a Latin
+        // layer. We then combine that with the modifiers of the event
+        // to produce the resulting "Latin" key combination.
+
+        // Depending on whether Qt::AA_MacDontSwapCtrlAndMeta is set or not
+        // the index of the Command layer in modifierCombinations will differ.
+        const int commandLayer = qApp->testAttribute(Qt::AA_MacDontSwapCtrlAndMeta) ? 8 : 2;
+        // FIXME: We don't clear the key map when Qt::AA_MacDontSwapCtrlAndMeta
+        // is set, so changing Qt::AA_MacDontSwapCtrlAndMeta at runtime will
+        // fail once we've built the key map.
+        ret << QKeyCombination::fromCombined(int(eventModifiers) + int(keyMap[commandLayer]));
+
+        // If the unmodified key is outside of Latin1, we also treat
+        // that as a valid key combination, even if AppKit natively
+        // does not. For example, for a Greek layout, we still want
+        // to support Option+Command+ψ as a key combination, as it's
+        // unlikely to clash with the Latin key combination we added
+        // above.
+
+        // However, if the unmodified key is within Latin1, we skip
+        // it, to avoid these types of conflicts. For example, in
+        // the same Greek layout, pressing the key next to Tab will
+        // produce a Latin ';' symbol, but we've already treated that
+        // as 'q' above, thanks to the Command modifier, so we skip
+        // the potential Command+; key combination. This is also in
+        // line with what AppKit natively does.
+
+        // Skipping Latin1 unmodified keys also handles the case of
+        // a Latin layout, where the unmodified and modified keys
+        // are the same.
+
+        if (unmodifiedKey <= 0xff)
+            startingModifierLayer = 1;
+    }
 
     // FIXME: We only compute the first 8 combinations. Why?
-    for (int i = 1; i < 8; ++i) {
+    for (int i = startingModifierLayer; i < 15; ++i) {
         auto keyAfterApplyingModifiers = keyMap[i];
-        if (keyAfterApplyingModifiers == unmodifiedKey)
-            continue;
         if (!keyAfterApplyingModifiers)
              continue;
 
@@ -609,7 +580,39 @@ QList<int> QAppleKeyMapper::possibleKeys(const QKeyEvent *event) const
             // If the event includes more modifiers than the candidate they
             // will need to be included in the resulting key combination.
             auto additionalModifiers = eventModifiers & ~candidateModifiers;
-            ret << int(additionalModifiers) + int(keyAfterApplyingModifiers);
+
+            auto keyCombination = QKeyCombination::fromCombined(
+                int(additionalModifiers) + int(keyAfterApplyingModifiers));
+
+            // If there's an existing key combination with the same key,
+            // but a different set of modifiers, we want to choose only
+            // one of them, by priority (see below).
+            const auto existingCombination = std::find_if(
+                ret.begin(), ret.end(), [&](auto existingCombination) {
+                    return existingCombination.key() == keyAfterApplyingModifiers;
+                });
+
+            if (existingCombination != ret.end()) {
+                // We prioritize the combination with the more specific
+                // modifiers. In the case where the number of modifiers
+                // are the same, we want to prioritize Command over Option
+                // over Control over Shift. Unfortunately the order (and
+                // hence value) of the modifiers in Qt::KeyboardModifier
+                // does not match our preferred order when Control and
+                // Meta is switched, but we can work around that by
+                // explicitly swapping the modifiers and using that
+                // for the comparison. This also works when the
+                // Qt::AA_MacDontSwapCtrlAndMeta application attribute
+                // is set, as the incoming modifiers are then left
+                // as is, and we can still trust the order.
+                auto existingModifiers = swapModifiersIfNeeded(existingCombination->keyboardModifiers());
+                auto replacementModifiers = swapModifiersIfNeeded(additionalModifiers);
+                if (replacementModifiers > existingModifiers)
+                    *existingCombination = keyCombination;
+            } else {
+                // All is good, no existing combination has this key
+                ret << keyCombination;
+            }
         }
     }
 
@@ -618,36 +621,65 @@ QList<int> QAppleKeyMapper::possibleKeys(const QKeyEvent *event) const
 
 
 
-#else
-// Keyboard keys (non-modifiers)
-API_AVAILABLE(ios(13.4)) static QHash<NSString *, Qt::Key> uiKitKeys = {
-#if QT_IOS_PLATFORM_SDK_EQUAL_OR_ABOVE(__IPHONE_13_4)
-    { UIKeyInputF1, Qt::Key_F1 },
-    { UIKeyInputF2, Qt::Key_F2 },
-    { UIKeyInputF3, Qt::Key_F3 },
-    { UIKeyInputF4, Qt::Key_F4 },
-    { UIKeyInputF5, Qt::Key_F5 },
-    { UIKeyInputF6, Qt::Key_F6 },
-    { UIKeyInputF7, Qt::Key_F7 },
-    { UIKeyInputF8, Qt::Key_F8 },
-    { UIKeyInputF9, Qt::Key_F9 },
-    { UIKeyInputF10, Qt::Key_F10 },
-    { UIKeyInputF11, Qt::Key_F11 },
-    { UIKeyInputF12, Qt::Key_F12 },
-    { UIKeyInputHome, Qt::Key_Home },
-    { UIKeyInputEnd, Qt::Key_End },
-    { UIKeyInputPageUp, Qt::Key_PageUp },
-    { UIKeyInputPageDown, Qt::Key_PageDown },
-#endif
-    { UIKeyInputEscape, Qt::Key_Escape },
-    { UIKeyInputUpArrow, Qt::Key_Up },
-    { UIKeyInputDownArrow, Qt::Key_Down },
-    { UIKeyInputLeftArrow, Qt::Key_Left },
-    { UIKeyInputRightArrow, Qt::Key_Right }
-};
+#else // iOS
 
+Qt::Key QAppleKeyMapper::fromNSString(Qt::KeyboardModifiers qtModifiers, NSString *characters,
+                                      NSString *charactersIgnoringModifiers, QString &text)
+{
+    if ([characters isEqualToString:@"\t"]) {
+        if (qtModifiers & Qt::ShiftModifier)
+            return Qt::Key_Backtab;
+        return Qt::Key_Tab;
+    } else if ([characters isEqualToString:@"\r"]) {
+        if (qtModifiers & Qt::KeypadModifier)
+            return Qt::Key_Enter;
+        return Qt::Key_Return;
+    }
+    if ([characters length] != 0 || [charactersIgnoringModifiers length] != 0) {
+        QChar ch;
+        if (((qtModifiers & Qt::MetaModifier) || (qtModifiers & Qt::AltModifier)) &&
+            ([charactersIgnoringModifiers length] != 0)) {
+            ch = QChar([charactersIgnoringModifiers characterAtIndex:0]);
+        } else if ([characters length] != 0) {
+            ch = QChar([characters characterAtIndex:0]);
+        }
+        if (!(qtModifiers & (Qt::ControlModifier | Qt::MetaModifier)) &&
+            (ch.unicode() < 0xf700 || ch.unicode() > 0xf8ff)) {
+            text = QString::fromNSString(characters);
+        }
+        if (!ch.isNull())
+            return Qt::Key(ch.toUpper().unicode());
+    }
+    return Qt::Key_unknown;
+}
+
+// Keyboard keys (non-modifiers)
 API_AVAILABLE(ios(13.4)) Qt::Key QAppleKeyMapper::fromUIKitKey(NSString *keyCode)
 {
+    static QHash<NSString *, Qt::Key> uiKitKeys = {
+        { UIKeyInputF1, Qt::Key_F1 },
+        { UIKeyInputF2, Qt::Key_F2 },
+        { UIKeyInputF3, Qt::Key_F3 },
+        { UIKeyInputF4, Qt::Key_F4 },
+        { UIKeyInputF5, Qt::Key_F5 },
+        { UIKeyInputF6, Qt::Key_F6 },
+        { UIKeyInputF7, Qt::Key_F7 },
+        { UIKeyInputF8, Qt::Key_F8 },
+        { UIKeyInputF9, Qt::Key_F9 },
+        { UIKeyInputF10, Qt::Key_F10 },
+        { UIKeyInputF11, Qt::Key_F11 },
+        { UIKeyInputF12, Qt::Key_F12 },
+        { UIKeyInputHome, Qt::Key_Home },
+        { UIKeyInputEnd, Qt::Key_End },
+        { UIKeyInputPageUp, Qt::Key_PageUp },
+        { UIKeyInputPageDown, Qt::Key_PageDown },
+        { UIKeyInputEscape, Qt::Key_Escape },
+        { UIKeyInputUpArrow, Qt::Key_Up },
+        { UIKeyInputDownArrow, Qt::Key_Down },
+        { UIKeyInputLeftArrow, Qt::Key_Left },
+        { UIKeyInputRightArrow, Qt::Key_Right }
+    };
+
     if (auto key = uiKitKeys.value(keyCode))
         return key;
 
