@@ -48,7 +48,7 @@
 
 QT_BEGIN_NAMESPACE
 #ifndef QT_NO_DBUS
-Q_LOGGING_CATEGORY(lcQpaThemeDBus, "qt.qpa.theme.dbus")
+Q_STATIC_LOGGING_CATEGORY(lcQpaThemeDBus, "qt.qpa.theme.dbus")
 #endif
 
 using namespace Qt::StringLiterals;
@@ -178,17 +178,8 @@ private:
         Provider provider;
         Setting setting;
         ChangeSignal(Provider p, Setting s) : provider(p), setting(s) {}
-        ChangeSignal() {}
     };
 
-    // Json keys
-    static constexpr QLatin1StringView s_dbusLocation = QLatin1StringView("DBusLocation");
-    static constexpr QLatin1StringView s_dbusKey = QLatin1StringView("DBusKey");
-    static constexpr QLatin1StringView s_provider = QLatin1StringView("Provider");
-    static constexpr QLatin1StringView s_setting = QLatin1StringView("Setting");
-    static constexpr QLatin1StringView s_signals = QLatin1StringView("DbusSignals");
-    static constexpr QLatin1StringView s_root = QLatin1StringView("Qt.qpa.DBusSignals");
-
     QFlatMap <DBusKey, ChangeSignal> m_signalMap;
 
     void init(const QString &service, const QString &path,
@@ -200,6 +191,17 @@ private:
     void saveJson(const QString &fileName) const;
 };
 
+namespace {
+namespace JsonKeys {
+constexpr auto dbusLocation() { return "DBusLocation"_L1; }
+constexpr auto dbusKey() { return "DBusKey"_L1; }
+constexpr auto provider() { return "Provider"_L1; }
+constexpr auto setting() { return "Setting"_L1; }
+constexpr auto dbusSignals() { return "DbusSignals"_L1; }
+constexpr auto root() { return "Qt.qpa.DBusSignals"_L1; }
+} // namespace JsonKeys
+}
+
 QGenericUnixThemeDBusListener::QGenericUnixThemeDBusListener(const QString &service,
                                const QString &path, const QString &interface, const QString &signal)
 {
@@ -208,10 +210,10 @@ QGenericUnixThemeDBusListener::QGenericUnixThemeDBusListener(const QString &serv
 
 QGenericUnixThemeDBusListener::QGenericUnixThemeDBusListener()
 {
-    static constexpr QLatin1StringView service("");
-    static constexpr QLatin1StringView path("/org/freedesktop/portal/desktop");
-    static constexpr QLatin1StringView interface("org.freedesktop.portal.Settings");
-    static constexpr QLatin1StringView signal("SettingChanged");
+    const auto service = u""_s;
+    const auto path = u"/org/freedesktop/portal/desktop"_s;
+    const auto interface = u"org.freedesktop.portal.Settings"_s;
+    const auto signal = u"SettingChanged"_s;
 
     init (service, path, interface, signal);
 }
@@ -273,26 +275,26 @@ void QGenericUnixThemeDBusListener::loadJson(const QString &fileName)
     QJsonParseError error;
     QJsonDocument doc = QJsonDocument::fromJson(file.readAll(), &error);
     CHECK((error.error == QJsonParseError::NoError), error.errorString());
-    CHECK(doc.isObject(), "Parse Error: Expected root object" << s_root);
+    CHECK(doc.isObject(), "Parse Error: Expected root object" << JsonKeys::root());
 
     const QJsonObject &root = doc.object();
-    CHECK(root.contains(s_root), "Parse Error: Expected root object" << s_root);
-    CHECK(root[s_root][s_signals].isArray(), "Parse Error: Expected array" << s_signals);
+    CHECK(root.contains(JsonKeys::root()), "Parse Error: Expected root object" << JsonKeys::root());
+    CHECK(root[JsonKeys::root()][JsonKeys::dbusSignals()].isArray(), "Parse Error: Expected array" << JsonKeys::dbusSignals());
 
-    const QJsonArray &sigs = root[s_root][s_signals].toArray();
-    CHECK((sigs.count() > 0), "Parse Error: Found empty array" << s_signals);
+    const QJsonArray &sigs = root[JsonKeys::root()][JsonKeys::dbusSignals()].toArray();
+    CHECK((sigs.count() > 0), "Parse Error: Found empty array" << JsonKeys::dbusSignals());
 
     for (auto sig = sigs.constBegin(); sig != sigs.constEnd(); ++sig) {
-        CHECK(sig->isObject(), "Parse Error: Expected object array" << s_signals);
+        CHECK(sig->isObject(), "Parse Error: Expected object array" << JsonKeys::dbusSignals());
         const QJsonObject &obj = sig->toObject();
-        CHECK(obj.contains(s_dbusLocation), "Parse Error: Expected key" << s_dbusLocation);
-        CHECK(obj.contains(s_dbusKey), "Parse Error: Expected key" << s_dbusKey);
-        CHECK(obj.contains(s_provider), "Parse Error: Expected key" << s_provider);
-        CHECK(obj.contains(s_setting), "Parse Error: Expected key" << s_setting);
-        const QString &location = obj[s_dbusLocation].toString();
-        const QString &key = obj[s_dbusKey].toString();
-        const QString &providerString = obj[s_provider].toString();
-        const QString &settingString = obj[s_setting].toString();
+        CHECK(obj.contains(JsonKeys::dbusLocation()), "Parse Error: Expected key" << JsonKeys::dbusLocation());
+        CHECK(obj.contains(JsonKeys::dbusKey()), "Parse Error: Expected key" << JsonKeys::dbusKey());
+        CHECK(obj.contains(JsonKeys::provider()), "Parse Error: Expected key" << JsonKeys::provider());
+        CHECK(obj.contains(JsonKeys::setting()), "Parse Error: Expected key" << JsonKeys::setting());
+        const QString &location = obj[JsonKeys::dbusLocation()].toString();
+        const QString &key = obj[JsonKeys::dbusKey()].toString();
+        const QString &providerString = obj[JsonKeys::provider()].toString();
+        const QString &settingString = obj[JsonKeys::setting()].toString();
         PARSE(provider, Provider, providerString);
         PARSE(setting, Setting, settingString);
         const DBusKey dkey(location, key);
@@ -336,18 +338,18 @@ void QGenericUnixThemeDBusListener::saveJson(const QString &fileName) const
         const DBusKey &dkey = sig.key();
         const ChangeSignal &csig = sig.value();
         QJsonObject obj;
-        obj[s_dbusLocation] = dkey.location;
-        obj[s_dbusKey] = dkey.key;
-        obj[s_provider] = QLatin1StringView(QMetaEnum::fromType<Provider>()
+        obj[JsonKeys::dbusLocation()] = dkey.location;
+        obj[JsonKeys::dbusKey()] = dkey.key;
+        obj[JsonKeys::provider()] = QLatin1StringView(QMetaEnum::fromType<Provider>()
                                             .valueToKey(static_cast<int>(csig.provider)));
-        obj[s_setting] = QLatin1StringView(QMetaEnum::fromType<Setting>()
+        obj[JsonKeys::setting()] = QLatin1StringView(QMetaEnum::fromType<Setting>()
                                            .valueToKey(static_cast<int>(csig.setting)));
         sigs.append(obj);
     }
     QJsonObject obj;
-    obj[s_signals] = sigs;
+    obj[JsonKeys::dbusSignals()] = sigs;
     QJsonObject root;
-    root[s_root] = obj;
+    root[JsonKeys::root()] = obj;
     QJsonDocument doc(root);
     file.write(doc.toJson());
     file.close();
@@ -384,8 +386,9 @@ std::optional<QGenericUnixThemeDBusListener::ChangeSignal>
 {
     const DBusKey dkey(location, key);
     std::optional<QGenericUnixThemeDBusListener::ChangeSignal> ret;
-    if (m_signalMap.contains(dkey))
-        ret.emplace(m_signalMap.value(dkey));
+    const auto it = m_signalMap.find(dkey);
+    if (it != m_signalMap.cend())
+        ret.emplace(it.value());
 
     return ret;
 }
@@ -503,6 +506,8 @@ QVariant QGenericUnixTheme::themeHint(ThemeHint hint) const
         return QVariant(mouseCursorTheme());
     case QPlatformTheme::MouseCursorSize:
         return QVariant(mouseCursorSize());
+    case QPlatformTheme::PreferFileIconFromTheme:
+        return true;
     default:
         break;
     }
@@ -692,7 +697,12 @@ static constexpr QLatin1StringView settingsPrefix(QKdeThemePrivate::KdeSettingTy
     case QKdeThemePrivate::KdeSettingType::ToolBarStyle:
         return QLatin1StringView("Toolbar style/");
     }
-    Q_UNREACHABLE_RETURN(QLatin1StringView());
+    // GCC 8.x does not treat __builtin_unreachable() as constexpr
+#  if !defined(Q_CC_GNU_ONLY) || (Q_CC_GNU >= 900)
+    // NOLINTNEXTLINE(qt-use-unreachable-return): Triggers on Clang, breaking GCC 8
+    Q_UNREACHABLE();
+#  endif
+    return {};
 }
 
 static constexpr QKdeThemePrivate::KdeSettingType settingsType(QKdeThemePrivate::KdeSetting setting)
@@ -731,7 +741,12 @@ static constexpr QKdeThemePrivate::KdeSettingType settingsType(QKdeThemePrivate:
     CASE(TooltipBackground, Colors);
     CASE(TooltipForeground, Colors);
     };
-    Q_UNREACHABLE_RETURN(QKdeThemePrivate::KdeSettingType::Root);
+    // GCC 8.x does not treat __builtin_unreachable() as constexpr
+#  if !defined(Q_CC_GNU_ONLY) || (Q_CC_GNU >= 900)
+    // NOLINTNEXTLINE(qt-use-unreachable-return): Triggers on Clang, breaking GCC 8
+    Q_UNREACHABLE();
+#  endif
+    return QKdeThemePrivate::KdeSettingType::Root;
 }
 #undef CASE
 
@@ -797,7 +812,12 @@ static constexpr QLatin1StringView settingsKey(QKdeThemePrivate::KdeSetting sett
     case QKdeThemePrivate::KdeSetting::TooltipForeground:
         return QLatin1StringView("Tooltip/ForegroundNormal");
     };
-    Q_UNREACHABLE_RETURN(QLatin1StringView());
+    // GCC 8.x does not treat __builtin_unreachable() as constexpr
+#  if !defined(Q_CC_GNU_ONLY) || (Q_CC_GNU >= 900)
+    // NOLINTNEXTLINE(qt-use-unreachable-return): Triggers on Clang, breaking GCC 8
+    Q_UNREACHABLE();
+#  endif
+    return {};
 }
 
 void QKdeThemePrivate::refresh()
@@ -835,6 +855,10 @@ void QKdeThemePrivate::refresh()
     const QVariant singleClickValue = readKdeSetting(KdeSetting::SingleClick);
     if (singleClickValue.isValid())
         singleClick = singleClickValue.toBool();
+    else if (kdeVersion >= 6) // Plasma 6 defaults to double-click
+        singleClick = false;
+    else // earlier version to single-click
+        singleClick = true;
 
     const QVariant showIconsOnPushButtonsValue = readKdeSetting(KdeSetting::ShowIconsOnPushButtons);
     if (showIconsOnPushButtonsValue.isValid())
@@ -1108,6 +1132,8 @@ QVariant QKdeTheme::themeHint(QPlatformTheme::ThemeHint hint) const
         return QVariant(mouseCursorTheme());
     case QPlatformTheme::MouseCursorSize:
         return QVariant(mouseCursorSize());
+    case QPlatformTheme::PreferFileIconFromTheme:
+        return true;
     default:
         break;
     }
@@ -1196,11 +1222,11 @@ QPlatformTheme *QKdeTheme::createKdeTheme()
     // - fallback to /etc/kde<version>
 
     QStringList kdeDirs;
-    const QString kdeHomePathVar = QFile::decodeName(qgetenv("KDEHOME"));
+    const QString kdeHomePathVar = qEnvironmentVariable("KDEHOME");
     if (!kdeHomePathVar.isEmpty())
         kdeDirs += kdeHomePathVar;
 
-    const QString kdeDirsVar = QFile::decodeName(qgetenv("KDEDIRS"));
+    const QString kdeDirsVar = qEnvironmentVariable("KDEDIRS");
     if (!kdeDirsVar.isEmpty())
         kdeDirs += kdeDirsVar.split(u':', Qt::SkipEmptyParts);
 
@@ -1385,6 +1411,8 @@ QVariant QGnomeTheme::themeHint(QPlatformTheme::ThemeHint hint) const
         return QVariant(mouseCursorTheme());
     case QPlatformTheme::MouseCursorSize:
         return QVariant(mouseCursorSize());
+    case QPlatformTheme::PreferFileIconFromTheme:
+        return true;
     default:
         break;
     }
@@ -1490,6 +1518,7 @@ QStringList QGenericUnixTheme::themeNames()
         QList<QByteArray> gtkBasedEnvironments;
         gtkBasedEnvironments << "GNOME"
                              << "X-CINNAMON"
+                             << "PANTHEON"
                              << "UNITY"
                              << "MATE"
                              << "XFCE"
