@@ -23,6 +23,8 @@ QT_REQUIRE_CONFIG(directwrite);
 #include <QtGui/private/qfontengine_p.h>
 #include <QtCore/QSharedPointer>
 
+#include <dwrite_2.h>
+
 struct IDWriteFont;
 struct IDWriteFontFace;
 struct IDWriteFontFile;
@@ -30,10 +32,18 @@ struct IDWriteFactory;
 struct IDWriteBitmapRenderTarget;
 struct IDWriteGdiInterop;
 struct IDWriteGlyphRunAnalysis;
+struct IDWriteColorGlyphRunEnumerator;
+
+#if QT_CONFIG(directwritecolrv1)
+struct IDWriteFontFace7;
+struct IDWritePaintReader;
+struct DWRITE_PAINT_ELEMENT;
+#endif
 
 QT_BEGIN_NAMESPACE
 
 class QWindowsFontEngineData;
+class QColrPaintGraphRenderer;
 
 class Q_GUI_EXPORT QWindowsFontEngineDirectWrite : public QFontEngine
 {
@@ -52,8 +62,8 @@ public:
     QFixed emSquareSize() const override;
 
     glyph_t glyphIndex(uint ucs4) const override;
-    bool stringToCMap(const QChar *str, int len, QGlyphLayout *glyphs, int *nglyphs,
-                      ShaperFlags flags) const override;
+    int stringToCMap(const QChar *str, int len, QGlyphLayout *glyphs, int *nglyphs,
+                     ShaperFlags flags) const override;
     void recalcAdvances(QGlyphLayout *glyphs, ShaperFlags) const override;
 
     void addGlyphsToPath(glyph_t *glyphs, QFixedPoint *positions, int nglyphs,
@@ -98,15 +108,59 @@ public:
 
     void initializeHeightMetrics() const override;
 
+    Properties properties() const override;
+
+    QPainterPath unscaledGlyph(glyph_t glyph) const;
+    void getUnscaledGlyph(glyph_t glyph, QPainterPath *path, glyph_metrics_t *metrics) override;
+
+    QList<QFontVariableAxis> variableAxes() const override;
+
 private:
+#if QT_CONFIG(directwritecolrv1)
+    bool traverseColr1(IDWritePaintReader *paintReader,
+                       IDWriteFontFace7 *face7,
+                       const DWRITE_PAINT_ELEMENT *paintElement,
+                       QColrPaintGraphRenderer *graphRenderer) const;
+    bool renderColr1GlyphRun(QImage *image,
+                             const DWRITE_GLYPH_RUN *glyphRun,
+                             const DWRITE_MATRIX &transform,
+                             QColor color) const;
+#endif
+
+    QRect paintGraphBounds(glyph_t glyph, const DWRITE_MATRIX &transform) const;
+    QRect alphaTextureBounds(glyph_t glyph, const DWRITE_MATRIX &transform);
+    QRect colorBitmapBounds(glyph_t glyph, const DWRITE_MATRIX &transform);
+    bool renderColr0GlyphRun(QImage *image,
+                             const DWRITE_COLOR_GLYPH_RUN *colorGlyphRun,
+                             const DWRITE_MATRIX &transform,
+                             DWRITE_RENDERING_MODE renderMode,
+                             DWRITE_MEASURING_MODE measureMode,
+                             DWRITE_GRID_FIT_MODE gridFitMode,
+                             QColor color,
+                             QRect boundingRect) const;
+    QImage renderColorGlyph(DWRITE_GLYPH_RUN *glyphRun,
+                            const DWRITE_MATRIX &transform,
+                            DWRITE_RENDERING_MODE renderMode,
+                            DWRITE_MEASURING_MODE measureMode,
+                            DWRITE_GRID_FIT_MODE gridFitMode,
+                            QColor color,
+                            QRect boundingRect) const;
     QImage imageForGlyph(glyph_t t,
                          const QFixedPoint &subPixelPosition,
                          int margin,
                          const QTransform &xform,
                          const QColor &color = QColor());
     void collectMetrics();
-    void renderGlyphRun(QImage *destination, float r, float g, float b, float a, IDWriteGlyphRunAnalysis *glyphAnalysis, const QRect &boundingRect);
+    void renderGlyphRun(QImage *destination,
+                        float r,
+                        float g,
+                        float b,
+                        float a,
+                        IDWriteGlyphRunAnalysis *glyphAnalysis,
+                        const QRect &boundingRect,
+                        DWRITE_RENDERING_MODE renderMode) const;
     static QString filenameFromFontFile(IDWriteFontFile *fontFile);
+    DWRITE_RENDERING_MODE hintingPreferenceToRenderingMode(const QFontDef &fontDef) const;
 
     const QSharedPointer<QWindowsFontEngineData> m_fontEngineData;
 
@@ -121,6 +175,8 @@ private:
     QFixed m_maxAdvanceWidth;
     FaceId m_faceId;
     QString m_uniqueFamilyName;
+    QList<QFontVariableAxis> m_variableAxes;
+    DWRITE_PIXEL_GEOMETRY m_pixelGeometry = DWRITE_PIXEL_GEOMETRY_RGB;
 };
 
 QT_END_NAMESPACE
