@@ -1,5 +1,6 @@
 // Copyright (C) 2023 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qrestaccessmanager.h"
 #include "qrestaccessmanager_p.h"
@@ -31,8 +32,6 @@ Q_LOGGING_CATEGORY(lcQrest, "qt.network.access.rest")
     \inmodule QtNetwork
     \reentrant
 
-    \preliminary
-
     QRestAccessManager is a convenience wrapper on top of
     QNetworkAccessManager. It amends datatypes and HTTP methods
     that are useful for typical RESTful client applications.
