@@ -1,6 +1,7 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // Copyright (C) 2016 Intel Corporation.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:trusted-data
 
 #include "qbytearray.h"
 #include "qset.h"
@@ -69,16 +70,17 @@ static auto &ifreq_index(Req &req, std::enable_if_t<sizeof(std::declval<Req>().i
 uint QNetworkInterfaceManager::interfaceIndexFromName(const QString &name)
 {
 #if QT_CONFIG(ipv6ifname)
-    return ::if_nametoindex(name.toLatin1());
+    return ::if_nametoindex(name.toLatin1().constData());
 #elif defined(SIOCGIFINDEX)
     struct ifreq req;
     int socket = qt_safe_socket(AF_INET, SOCK_STREAM, 0);
     if (socket < 0)
         return 0;
 
-    QByteArray name8bit = name.toLatin1();
+    const QByteArray name8bit = name.toLatin1();
     memset(&req, 0, sizeof(ifreq));
-    memcpy(req.ifr_name, name8bit, qMin<int>(name8bit.length() + 1, sizeof(req.ifr_name) - 1));
+    if (!name8bit.isNull())
+        memcpy(req.ifr_name, name8bit.data(), qMin(size_t(name8bit.length()) + 1, sizeof(req.ifr_name) - 1));
 
     uint id = 0;
     if (qt_safe_ioctl(socket, SIOCGIFINDEX, &req) >= 0)
@@ -235,7 +237,8 @@ static QList<QNetworkInterfacePrivate *> interfaceListing()
     for ( ; it != names.constEnd(); ++it) {
         ifreq req;
         memset(&req, 0, sizeof(ifreq));
-        memcpy(req.ifr_name, *it, qMin<int>(it->length() + 1, sizeof(req.ifr_name) - 1));
+        if (!it->isNull())
+            memcpy(req.ifr_name, it->constData(), qMin(size_t(it->length()) + 1, sizeof(req.ifr_name) - 1));
 
         QNetworkInterfacePrivate *iface = findInterface(socket, interfaces, req);
 
@@ -246,7 +249,8 @@ static QList<QNetworkInterfacePrivate *> interfaceListing()
             iface->name = QString::fromLatin1(req.ifr_name);
 
             // reset the name:
-            memcpy(req.ifr_name, oldName, qMin<int>(oldName.length() + 1, sizeof(req.ifr_name) - 1));
+            if (!oldName.isNull())
+                memcpy(req.ifr_name, oldName.constData(), qMin(size_t(oldName.length()) + 1, sizeof(req.ifr_name) - 1));
         } else
 #endif
         {
@@ -307,12 +311,20 @@ QT_BEGIN_INCLUDE_NAMESPACE
 QT_END_INCLUDE_NAMESPACE
 # endif
 
+static int openSocket(int &socket)
+{
+    if (socket == -1)
+        socket = qt_safe_socket(AF_INET, SOCK_DGRAM, 0);
+    return socket;
+}
+
 # if defined(Q_OS_LINUX) &&  __GLIBC__ - 0 >= 2 && __GLIBC_MINOR__ - 0 >= 1 && !defined(QT_LINUXBASE)
 #  include <netpacket/packet.h>
 
 static QList<QNetworkInterfacePrivate *> createInterfaces(ifaddrs *rawList)
 {
     Q_UNUSED(getMtu);
+    Q_UNUSED(openSocket);
     QList<QNetworkInterfacePrivate *> interfaces;
     QDuplicateTracker<QString> seenInterfaces;
     QDuplicateTracker<int> seenIndexes;
@@ -384,13 +396,6 @@ QT_BEGIN_INCLUDE_NAMESPACE
 #endif // QT_PLATFORM_UIKIT
 QT_END_INCLUDE_NAMESPACE
 
-static int openSocket(int &socket)
-{
-    if (socket == -1)
-        socket = qt_safe_socket(AF_INET, SOCK_DGRAM, 0);
-    return socket;
-}
-
 static QNetworkInterface::InterfaceType probeIfType(int socket, int iftype, struct ifmediareq *req)
 {
     // Determine the interface type.
@@ -534,8 +539,8 @@ static void getAddressExtraInfo(QNetworkAddressEntry *entry, struct sockaddr *sa
 
 static QList<QNetworkInterfacePrivate *> createInterfaces(ifaddrs *rawList)
 {
-    Q_UNUSED(getMtu);
     QList<QNetworkInterfacePrivate *> interfaces;
+    int socket = -1;
 
     // make sure there's one entry for each interface
     for (ifaddrs *ptr = rawList; ptr; ptr = ptr->ifa_next) {
@@ -556,9 +561,18 @@ static QList<QNetworkInterfacePrivate *> createInterfaces(ifaddrs *rawList)
             iface->index = ifindex;
             iface->name = QString::fromLatin1(ptr->ifa_name);
             iface->flags = convertFlags(ptr->ifa_flags);
+
+            if ((socket = openSocket(socket)) >= 0) {
+                struct ifreq ifr;
+                qstrncpy(ifr.ifr_name, ptr->ifa_name, sizeof(ifr.ifr_name));
+                iface->mtu = getMtu(socket, &ifr);
+            }
         }
     }
 
+    if (socket != -1)
+        qt_safe_close(socket);
+
     return interfaces;
 }
 
