@@ -5,6 +5,8 @@
 #define QIOSPLATFORMACCESSIBILITY_H
 
 #include <qpa/qplatformaccessibility.h>
+#include <QtCore/private/qcore_mac_p.h>
+#include <quiaccessibilityelement.h>
 
 #if QT_CONFIG(accessibility)
 
@@ -17,6 +19,10 @@ public:
     ~QIOSPlatformAccessibility();
 
     virtual void notifyAccessibilityUpdate(QAccessibleEvent *event);
+
+private:
+    QMacNotificationObserver m_focusObserver;
+    QMacAccessibilityElement *m_focusElement;
 };
 
 QT_END_NAMESPACE
