@@ -1,46 +1,11 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qwindowsopengltester.h"
 #include "qwindowscontext.h"
 
 #include <QtCore/qvariant.h>
+#include <QtCore/qmap.h>
 #include <QtCore/qdebug.h>
 #include <QtCore/qtextstream.h>
 #include <QtCore/qcoreapplication.h>
@@ -49,13 +14,14 @@
 #include <QtCore/qstandardpaths.h>
 #include <QtCore/qlibraryinfo.h>
 #include <QtCore/qhash.h>
+#include <private/qsystemlibrary_p.h>
+#include <QtGui/qtgui-config.h>
 
 #ifndef QT_NO_OPENGL
 #include <private/qopengl_p.h>
 #endif
 
 #include <QtCore/qt_windows.h>
-#include <private/qsystemlibrary_p.h>
 #include <d3d9.h>
 
 QT_BEGIN_NAMESPACE
@@ -94,19 +60,14 @@ public:
     bool retrieveAdapterIdentifier(UINT n, D3DADAPTER_IDENTIFIER9 *adapterIdentifier) const;
 
 private:
-    QSystemLibrary m_d3d9lib;
     IDirect3D9 *m_direct3D9 = nullptr;
 };
 
-QDirect3D9Handle::QDirect3D9Handle() :
-    m_d3d9lib(QStringLiteral("d3d9"))
+QDirect3D9Handle::QDirect3D9Handle()
 {
-    using PtrDirect3DCreate9 = IDirect3D9 *(WINAPI *)(UINT);
-
-    if (m_d3d9lib.load()) {
-        if (auto direct3DCreate9 = (PtrDirect3DCreate9)m_d3d9lib.resolve("Direct3DCreate9"))
-            m_direct3D9 = direct3DCreate9(D3D_SDK_VERSION);
-    }
+#ifndef QT_NO_OPENGL
+    m_direct3D9 = Direct3DCreate9(D3D_SDK_VERSION);
+#endif
 }
 
 QDirect3D9Handle::~QDirect3D9Handle()
@@ -224,13 +185,13 @@ QVariant GpuDescription::toVariant() const
     result.insert(QStringLiteral("deviceId"), QVariant(deviceId));
     result.insert(QStringLiteral("subSysId"),QVariant(subSysId));
     result.insert(QStringLiteral("revision"), QVariant(revision));
-    result.insert(QStringLiteral("driver"), QVariant(QLatin1String(driverName)));
+    result.insert(QStringLiteral("driver"), QVariant(QLatin1StringView(driverName)));
     result.insert(QStringLiteral("driverProduct"), QVariant(driverVersion.segmentAt(0)));
     result.insert(QStringLiteral("driverVersion"), QVariant(driverVersion.segmentAt(1)));
     result.insert(QStringLiteral("driverSubVersion"), QVariant(driverVersion.segmentAt(2)));
     result.insert(QStringLiteral("driverBuild"), QVariant(driverVersion.segmentAt(3)));
     result.insert(QStringLiteral("driverVersionString"), driverVersion.toString());
-    result.insert(QStringLiteral("description"), QVariant(QLatin1String(description)));
+    result.insert(QStringLiteral("description"), QVariant(QLatin1StringView(description)));
     result.insert(QStringLiteral("printable"), QVariant(toString()));
     return result;
 }
@@ -358,7 +319,7 @@ bool QWindowsOpenGLTester::testDesktopGL()
 
     // Test #1: Load opengl32.dll and try to resolve an OpenGL 2 function.
     // This will typically fail on systems that do not have a real OpenGL driver.
-    lib = LoadLibraryA("opengl32.dll");
+    lib = QSystemLibrary::load(L"opengl32");
     if (lib) {
         CreateContext = reinterpret_cast<CreateContextType>(
             reinterpret_cast<QFunctionPointer>(::GetProcAddress(lib, "wglCreateContext")));
@@ -447,8 +408,8 @@ bool QWindowsOpenGLTester::testDesktopGL()
             qCDebug(lcQpaGl, "OpenGL 1.x entry points not found");
         }
 
-        // Check for a shader-specific function.
-        if (WGL_GetProcAddress("glCreateShader")) {
+        // Check for a shader-specific function, as well as FBO support.
+        if (WGL_GetProcAddress("glCreateShader") && WGL_GetProcAddress("glBindFramebuffer")) {
             result = true;
             qCDebug(lcQpaGl, "OpenGL 2.0 entry points available");
         } else {
