@@ -156,7 +156,7 @@ static void updateFormatFromContext(QSurfaceFormat &format)
                 format.setOption(QSurfaceFormat::ResetNotification);
         }
 
-        if (format.version() < qMakePair(3, 0)) {
+        if (format.version() < std::pair(3, 0)) {
             format.setOption(QSurfaceFormat::DeprecatedFunctions);
             return;
         }
@@ -169,7 +169,7 @@ static void updateFormatFromContext(QSurfaceFormat &format)
             format.setOption(QSurfaceFormat::DeprecatedFunctions);
         if (value & GL_CONTEXT_FLAG_DEBUG_BIT)
             format.setOption(QSurfaceFormat::DebugContext);
-        if (format.version() < qMakePair(3, 2))
+        if (format.version() < std::pair(3, 2))
             return;
 
         // Version 3.2 and newer have a profile
@@ -242,9 +242,9 @@ QGLXContext::QGLXContext(Display *display, QXcbScreen *screen, const QSurfaceFor
             // Robustness must match that of the shared context.
             if (share && share->format().testOption(QSurfaceFormat::ResetNotification))
                 m_format.setOption(QSurfaceFormat::ResetNotification);
-            Q_ASSERT(glVersions.count() > 0);
+            Q_ASSERT(glVersions.size() > 0);
 
-            for (int i = 0; !m_context && i < glVersions.count(); i++) {
+            for (int i = 0; !m_context && i < glVersions.size(); i++) {
                 const int version = glVersions[i];
                 if (version > requestedVersion)
                     continue;
@@ -395,14 +395,19 @@ QGLXContext::QGLXContext(Display *display, GLXContext context, void *visualInfo,
         int numConfigs = 0;
         static const int attribs[] = { GLX_FBCONFIG_ID, configId, None };
         configs = glXChooseFBConfig(m_display, screenNumber, attribs, &numConfigs);
-        if (!configs || numConfigs < 1) {
+        if (!configs) {
+            qWarning("QGLXContext: Failed to find config(invalid arguments for glXChooseFBConfig)");
+            return;
+        } else if (numConfigs < 1) {
             qWarning("QGLXContext: Failed to find config");
+            XFree(configs);
             return;
         }
         if (configs && numConfigs > 1) // this is suspicious so warn but let it continue
             qWarning("QGLXContext: Multiple configs for FBConfig ID %d", configId);
 
         m_config = configs[0];
+        XFree(configs);
     }
 
     Q_ASSERT(vinfo || m_config);
