@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qgtk3theme.h"
 #include "qgtk3dialoghelpers.h"
@@ -88,6 +89,7 @@ QGtk3Theme::QGtk3Theme()
     };
 
     GtkSettings *settings = gtk_settings_get_default();
+    SETTING_CONNECT("gtk-cursor-blink");
     SETTING_CONNECT("gtk-cursor-blink-time");
     SETTING_CONNECT("gtk-double-click-distance");
     SETTING_CONNECT("gtk-double-click-time");
@@ -119,7 +121,10 @@ QVariant QGtk3Theme::themeHint(QPlatformTheme::ThemeHint hint) const
 {
     switch (hint) {
     case QPlatformTheme::CursorFlashTime:
-        return QVariant(gtkSetting<gint>("gtk-cursor-blink-time"));
+        if (gtkSetting<gboolean>("gtk-cursor-blink"))
+            return QVariant(gtkSetting<gint>("gtk-cursor-blink-time"));
+        else
+            return 0;
     case QPlatformTheme::MouseDoubleClickDistance:
         return QVariant(gtkSetting<gint>("gtk-double-click-distance"));
     case QPlatformTheme::MouseDoubleClickInterval:
@@ -159,10 +164,28 @@ QString QGtk3Theme::gtkFontName() const
     return QGnomeTheme::gtkFontName();
 }
 
+void QGtk3Theme::requestColorScheme(Qt::ColorScheme scheme)
+{
+    if (m_requestedColorScheme == scheme)
+        return;
+    qCDebug(lcQGtk3Interface) << scheme << "has been requested. Theme supports color scheme:"
+                              << m_storage->colorScheme();
+    m_requestedColorScheme = scheme;
+    m_storage->handleThemeChange();
+}
+
 Qt::ColorScheme QGtk3Theme::colorScheme() const
 {
     Q_ASSERT(m_storage);
-    return m_storage->colorScheme();
+#ifdef QT_DEBUG
+    if (m_requestedColorScheme != Qt::ColorScheme::Unknown
+        && m_requestedColorScheme != m_storage->colorScheme()) {
+        qCDebug(lcQGtk3Interface) << "Requested color scheme" << m_requestedColorScheme
+                                  << "differs from theme color scheme" << m_storage->colorScheme();
+    }
+#endif
+    return m_requestedColorScheme == Qt::ColorScheme::Unknown ? m_storage->colorScheme()
+                                                              : m_requestedColorScheme;
 }
 
 bool QGtk3Theme::usePlatformNativeDialog(DialogType type) const
@@ -211,7 +234,18 @@ bool QGtk3Theme::useNativeFileDialog()
 const QPalette *QGtk3Theme::palette(Palette type) const
 {
     Q_ASSERT(m_storage);
-    return m_storage->palette(type);
+#ifdef QT_DEBUG
+    if (m_requestedColorScheme != Qt::ColorScheme::Unknown
+        && m_requestedColorScheme != m_storage->colorScheme()) {
+        qCDebug(lcQGtk3Interface) << "Current KDE theme doesn't support reuqested color scheme"
+                                  << m_requestedColorScheme << "Falling back to fusion palette.";
+        return QPlatformTheme::palette(type);
+    }
+#endif
+
+    return (m_requestedColorScheme != Qt::ColorScheme::Unknown
+            && m_requestedColorScheme != m_storage->colorScheme())
+               ? QPlatformTheme::palette(type) : m_storage->palette(type);
 }
 
 QPixmap QGtk3Theme::standardPixmap(StandardPixmap sp, const QSizeF &size) const
