@@ -1,5 +1,6 @@
 // Copyright (C) 2017 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qxdgdesktopportaltheme.h"
 #include "qxdgdesktopportalfiledialog_p.h"
@@ -20,8 +21,9 @@ QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
 
-class QXdgDesktopPortalThemePrivate : public QPlatformThemePrivate
-{
+class QXdgDesktopPortalThemePrivate : public QObject
+    {
+    Q_OBJECT
 public:
     enum XdgColorschemePref {
         None,
@@ -30,7 +32,7 @@ public:
     };
 
     QXdgDesktopPortalThemePrivate()
-        : QPlatformThemePrivate()
+        : QObject()
     { }
 
     ~QXdgDesktopPortalThemePrivate()
@@ -62,6 +64,17 @@ public:
         }
     }
 
+public Q_SLOTS:
+    void settingChanged(const QString &group, const QString &key,
+                        const QDBusVariant &value)
+    {
+        if (group == "org.freedesktop.appearance"_L1 && key == "color-scheme"_L1) {
+            colorScheme = colorSchemeFromXdgPref(static_cast<XdgColorschemePref>(value.variant().toUInt()));
+            QWindowSystemInterface::handleThemeChange();
+        }
+    }
+
+public:
     QPlatformTheme *baseTheme = nullptr;
     uint fileChooserPortalVersion = 0;
     Qt::ColorScheme colorScheme = Qt::ColorScheme::Unknown;
@@ -72,24 +85,19 @@ QXdgDesktopPortalTheme::QXdgDesktopPortalTheme()
 {
     Q_D(QXdgDesktopPortalTheme);
 
-    QStringList themeNames;
-    themeNames += QGuiApplicationPrivate::platform_integration->themeNames();
-    // 1) Look for a theme plugin.
-    for (const QString &themeName : std::as_const(themeNames)) {
+    const QStringList themeNames = QGuiApplicationPrivate::platform_integration->themeNames();
+    for (const QString &themeName : themeNames) {
+        if (QXdgDesktopPortalTheme::isXdgPlugin(themeName))
+            continue;
+        // 1) Look for a theme plugin.
         d->baseTheme = QPlatformThemeFactory::create(themeName, nullptr);
         if (d->baseTheme)
             break;
-    }
 
-    // 2) If no theme plugin was found ask the platform integration to
-    // create a theme
-    if (!d->baseTheme) {
-        for (const QString &themeName : std::as_const(themeNames)) {
-            d->baseTheme = QGuiApplicationPrivate::platform_integration->createPlatformTheme(themeName);
-            if (d->baseTheme)
-                break;
-        }
-        // No error message; not having a theme plugin is allowed.
+        // 2) If no theme plugin was found ask the platform integration to create a theme
+        d->baseTheme = QGuiApplicationPrivate::platform_integration->createPlatformTheme(themeName);
+        if (d->baseTheme)
+            break;
     }
 
     // 3) Fall back on the built-in "null" platform theme.
@@ -126,6 +134,11 @@ QXdgDesktopPortalTheme::QXdgDesktopPortalTheme()
         const QXdgDesktopPortalThemePrivate::XdgColorschemePref xdgPref = static_cast<QXdgDesktopPortalThemePrivate::XdgColorschemePref>(dbusVariant.variant().toUInt());
         d->colorScheme = QXdgDesktopPortalThemePrivate::colorSchemeFromXdgPref(xdgPref);
     }
+
+    QDBusConnection::sessionBus().connect(
+            "org.freedesktop.portal.Desktop"_L1, "/org/freedesktop/portal/desktop"_L1,
+            "org.freedesktop.portal.Settings"_L1, "SettingChanged"_L1, d_ptr.get(),
+            SLOT(settingChanged(QString,QString,QDBusVariant)));
 }
 
 QPlatformMenuItem* QXdgDesktopPortalTheme::createPlatformMenuItem() const
@@ -246,4 +259,13 @@ QString QXdgDesktopPortalTheme::standardButtonText(int button) const
     return d->baseTheme->standardButtonText(button);
 }
 
+bool QXdgDesktopPortalTheme::isXdgPlugin(const QString &key)
+{
+    return key.compare("xdgdesktopportal"_L1, Qt::CaseInsensitive) == 0 ||
+           key.compare("flatpak"_L1, Qt::CaseInsensitive) == 0 ||
+           key.compare("snap"_L1, Qt::CaseInsensitive) == 0;
+}
+
 QT_END_NAMESPACE
+
+#include "qxdgdesktopportaltheme.moc"
