@@ -1,5 +1,6 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qsslsocket_openssl_symbols_p.h"
 #include "qtlsbackend_openssl_p.h"
@@ -388,16 +389,26 @@ QList<QSslCertificate> systemCaCertificates()
     {
         const QList<QByteArray> directories = QSslSocketPrivate::unixRootCertDirectories();
         QSet<QString> certFiles = {
-            QStringLiteral("/etc/pki/tls/certs/ca-bundle.crt"), // Fedora, Mandriva
+            QStringLiteral("/etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem"), // Red Hat 2013+
+            QStringLiteral("/etc/pki/tls/certs/ca-bundle.crt"), // Red Hat older, Mandriva
             QStringLiteral("/usr/local/share/certs/ca-root-nss.crt") // FreeBSD's ca_root_nss
         };
-        static const QStringList nameFilters = {u"*.pem"_s, u"*.crt"_s};
+
+        static const size_t extLen = strlen(".pem"); // or strlen(".crt")
+        auto hasMatchingExtension = [](const QString &fileName) {
+            if (size_t(fileName.size()) < extLen + 1)
+                return false;
+            auto ext = QStringView{fileName}.last(extLen);
+            return ext == ".pem"_L1 || ext == ".crt"_L1;
+        };
+
         using F = QDirListing::IteratorFlag;
         constexpr auto flags = F::FilesOnly | F::ResolveSymlinks; // Files and symlinks to files
-        for (const auto &directory : directories) {
-            for (const auto &dirEntry : QDirListing(directory, nameFilters, flags)) {
+        for (const QByteArray &directory : directories) {
+            for (const auto &dirEntry : QDirListing(QFile::decodeName(directory), flags)) {
                 // use canonical path here to not load the same certificate twice if symlinked
-                certFiles.insert(dirEntry.canonicalFilePath());
+                if (hasMatchingExtension(dirEntry.fileName()))
+                    certFiles.insert(dirEntry.canonicalFilePath());
             }
         }
         for (const QString& file : std::as_const(certFiles))
