@@ -4,6 +4,7 @@
 
 #include <QtCore/QCoreApplication>
 #include <QtCore/QDeadlineTimer>
+#include <QtCore/qdebug.h>
 #include <QtCore/QDir>
 #include <QtCore/QHash>
 #include <QtCore/QProcess>
@@ -48,6 +49,7 @@ struct Options
     QStringList amStarttestArgs;
     QString apkPath;
     QString ndkStackPath;
+    QList<QStringList> preTestRunAdbCommands;
     bool showLogcatOutput = false;
     std::optional<QProcess> stdoutLogger;
 };
@@ -168,6 +170,12 @@ static bool parseOptions()
             g_options.helpRequested = true;
         } else if (argument.compare("--verbose"_L1, Qt::CaseInsensitive) == 0) {
             g_options.verbose = true;
+        } else if (argument.compare("--pre-test-adb-command"_L1, Qt::CaseInsensitive) == 0) {
+            if (i + 1 == arguments.size())
+                g_options.helpRequested = true;
+            else {
+                g_options.preTestRunAdbCommands += QProcess::splitCommand(arguments.at(++i));
+            }
         } else if (argument.compare("--"_L1, Qt::CaseInsensitive) == 0) {
             ++i;
             break;
@@ -175,6 +183,14 @@ static bool parseOptions()
             g_options.testArgsList << arguments.at(i);
         }
     }
+
+    if (!g_options.skipAddInstallRoot) {
+        // we need to run make INSTALL_ROOT=path install to install the application file(s) first
+        g_options.makeCommand = "%1 INSTALL_ROOT=%2 install"_L1
+            .arg(g_options.makeCommand)
+            .arg(QDir::toNativeSeparators(g_options.buildPath));
+    }
+
     for (;i < arguments.size(); ++i)
         g_options.testArgsList << arguments.at(i);
 
@@ -226,7 +242,7 @@ static void printHelp()
                     "\n"
                     "    --show-logcat: Print Logcat output to stdout. If an ANR occurs during\n"
                     "       the test run, logs from the system_server process are included.\n"
-                    "       This argument is implied if a test fails.\n"
+                    "       This argument is implied if a test crashes.\n"
                     "\n"
                     "    --ndk-stack: Path to ndk-stack tool that symbolizes crash stacktraces.\n"
                     "       By default, ANDROID_NDK_ROOT env var is used to deduce the tool path.\n"
@@ -235,6 +251,9 @@ static void printHelp()
                     "\n"
                     "    --verbose: Prints out information during processing.\n"
                     "\n"
+                    "    --pre-test-adb-command <command>: call the adb <command> after\n"
+                    "       installation and before the test run.\n"
+                    "\n"
                     "    --help: Displays this information.\n",
                     qPrintable(QCoreApplication::arguments().at(0))
             );
@@ -711,16 +730,17 @@ void analyseLogcat(const QString &timeStamp, int *exitCode)
         }
     }
 
-    // If we have a failure, attempt to print both logcat and the crash buffer which
+    // If we have a crash, attempt to print both logcat and the crash buffer which
     // includes the crash stacktrace that is not included in the default logcat.
-    if (g_options.showLogcatOutput || *exitCode != 0) {
+    const bool testCrashed = *exitCode == EXIT_ERROR && !g_testInfo.isTestRunnerInterrupted.load();
+    if (g_options.showLogcatOutput || testCrashed) {
         qDebug() << "********** logcat dump **********";
         qDebug().noquote() << testLogcat.join(u'\n').trimmed();
         qDebug() << "********** End logcat dump **********";
-    }
 
-    if (!crashLogcat.isEmpty() && *exitCode != 0)
-        printLogcatCrash(crashLogcat);
+        if (!crashLogcat.isEmpty())
+            printLogcatCrash(crashLogcat);
+    }
 }
 
 static QString getCurrentTimeString()
@@ -811,17 +831,12 @@ int main(int argc, char *argv[])
                        "to generate the apk.";
         return EXIT_ERROR;
     }
-    if (!execCommand(g_options.makeCommand, nullptr, true)) {
-        if (!g_options.skipAddInstallRoot) {
-            // we need to run make INSTALL_ROOT=path install to install the application file(s) first
-            if (!execCommand("%1 INSTALL_ROOT=%2 install"_L1.arg(g_options.makeCommand,
-                                        QDir::toNativeSeparators(g_options.buildPath)), nullptr)) {
-                return EXIT_ERROR;
-            }
-        } else {
-            if (!execCommand(g_options.makeCommand, nullptr))
-                return EXIT_ERROR;
-        }
+
+    QByteArray buildOutput;
+    if (!execCommand(g_options.makeCommand, &buildOutput, true)) {
+        qCritical("The APK build command \"%s\" failed\n\n%s",
+            qPrintable(g_options.makeCommand), buildOutput.constData());
+        return EXIT_ERROR;
     }
 
     if (!QFile::exists(g_options.apkPath)) {
@@ -846,6 +861,10 @@ int main(int argc, char *argv[])
     g_testInfo.userId = userId();
 
     QString manifest = g_options.buildPath + "/AndroidManifest.xml"_L1;
+    if (!QFile::exists(manifest)) {
+        qCritical("Unable to find '%s'.", qPrintable(manifest));
+        return EXIT_ERROR;
+    }
     g_options.package = packageNameFromAndroidManifest(manifest);
     if (g_options.activity.isEmpty())
         g_options.activity = activityFromAndroidManifest(manifest);
@@ -862,6 +881,16 @@ int main(int argc, char *argv[])
     if (!g_testInfo.isPackageInstalled)
         return EXIT_ERROR;
 
+    // Call additional adb command if set after installation and before starting the test
+    for (const auto &command : g_options.preTestRunAdbCommands) {
+        QByteArray output;
+        if (!execAdbCommand(command, &output)) {
+            qCritical("The pre test ADB command \"%s\" failed with output:\n%s",
+                  qUtf8Printable(command.join(u' ')), output.constData());
+            return EXIT_ERROR;
+        }
+    }
+
     // Pre test start
     const QString formattedStartTime = getCurrentTimeString();
 
