@@ -11,7 +11,7 @@
 #include <qfileinfo.h>
 #include <qtextstream.h>
 
-#include <stdio.h>
+#include <cstdio>
 
 QT_BEGIN_NAMESPACE
 
@@ -37,11 +37,11 @@ WriteIncludes::WriteIncludes(Uic *uic) : WriteIncludesBase(uic),
     // When possible (no namespace) use the "QtModule/QClass" convention
     // and create a re-mapping of the old header "qclass.h" to it. Do not do this
     // for the "Phonon::Someclass" classes, however.
-    const QString namespaceDelimiter = "::"_L1;
+    const QLatin1StringView namespaceDelimiter = "::"_L1;
     for (const auto &e : classInfoEntries()) {
         const QString klass = QLatin1StringView(e.klass);
         const QString module = QLatin1StringView(e.module);
-        QLatin1StringView header = QLatin1StringView(e.header);
+        QLatin1StringView header(e.header);
         if (klass.contains(namespaceDelimiter)) {
             m_classToHeader.insert(klass, moduleHeader(module, header));
         } else {
@@ -73,7 +73,7 @@ void WriteIncludes::acceptUI(DomUI *node)
 void WriteIncludes::insertIncludeForClass(const QString &className, QString header, bool global)
 {
     if (debugWriteIncludes)
-        fprintf(stderr, "%s %s '%s' %d\n", Q_FUNC_INFO, qPrintable(className), qPrintable(header), global);
+        std::fprintf(stderr, "%s %s '%s' %d\n", Q_FUNC_INFO, qPrintable(className), qPrintable(header), global);
 
     do {
         if (!header.isEmpty())
@@ -90,8 +90,8 @@ void WriteIncludes::insertIncludeForClass(const QString &className, QString head
         // Quick check by class name to detect includehints provided for custom widgets.
         // Remove namespaces
         QString lowerClassName = className.toLower();
-        static const QString namespaceSeparator = "::"_L1;
-        const int namespaceIndex = lowerClassName.lastIndexOf(namespaceSeparator);
+        static const auto namespaceSeparator = "::"_L1;
+        const auto namespaceIndex = lowerClassName.lastIndexOf(namespaceSeparator);
         if (namespaceIndex != -1)
             lowerClassName.remove(0, namespaceIndex + namespaceSeparator.size());
         if (m_includeBaseNames.contains(lowerClassName)) {
