@@ -295,6 +295,15 @@ void QGraphicsProxyWidgetPrivate::sendWidgetMouseEvent(QGraphicsSceneMouseEvent
 #endif
     }
 
+#ifndef QT_NO_CURSOR
+    // Keep cursor in sync
+    if (lastWidgetUnderMouse) {
+        QCursor widgetsCursor = lastWidgetUnderMouse->cursor();
+        if (q->cursor() != widgetsCursor)
+            q->setCursor(widgetsCursor);
+    }
+#endif
+
     event->setAccepted(mouseEvent.isAccepted());
 }
 
@@ -340,10 +349,10 @@ QWidget *QGraphicsProxyWidgetPrivate::findFocusChild(QWidget *child, bool next)
 
     // Run around the focus chain until we find a widget that can take tab focus.
     if (!child) {
-        child = next ? (QWidget *)widget : widget->d_func()->focus_prev;
+        child = next ? widget.data() : widget->previousInFocusChain();
     } else {
-        child = next ? child->d_func()->focus_next : child->d_func()->focus_prev;
-        if ((next && child == widget) || (!next && child == widget->d_func()->focus_prev)) {
+        child = next ? child->nextInFocusChain() : child->previousInFocusChain();
+        if ((next && child == widget) || (!next && child == widget->previousInFocusChain())) {
              return nullptr;
         }
     }
@@ -360,8 +369,8 @@ QWidget *QGraphicsProxyWidgetPrivate::findFocusChild(QWidget *child, bool next)
             && !(child->d_func()->extra && child->d_func()->extra->focus_proxy)) {
             return child;
         }
-        child = next ? child->d_func()->focus_next : child->d_func()->focus_prev;
-    } while (child != oldChild && !(next && child == widget) && !(!next && child == widget->d_func()->focus_prev));
+        child = next ? child->nextInFocusChain() : child->previousInFocusChain();
+    } while (child != oldChild && !(next && child == widget) && !(!next && child == widget->previousInFocusChain()));
     return nullptr;
 }
 
@@ -461,7 +470,7 @@ void QGraphicsProxyWidgetPrivate::embedSubWindow(QWidget *subWin)
 */
 void QGraphicsProxyWidgetPrivate::unembedSubWindow(QWidget *subWin)
 {
-    for (QGraphicsItem *child : qAsConst(children)) {
+    for (QGraphicsItem *child : std::as_const(children)) {
         if (child->isWidget()) {
             if (QGraphicsProxyWidget *proxy = qobject_cast<QGraphicsProxyWidget *>(static_cast<QGraphicsWidget *>(child))) {
                 if (proxy->widget() == subWin) {
@@ -522,7 +531,7 @@ QGraphicsProxyWidget::~QGraphicsProxyWidget()
 /*!
     Embeds \a widget into this proxy widget. The embedded widget must reside
     exclusively either inside or outside of Graphics View. You cannot embed a
-    widget as long as it is is visible elsewhere in the UI, at the same time.
+    widget as long as it is visible elsewhere in the UI, at the same time.
 
     \a widget must be a top-level widget whose parent is \nullptr.
 
@@ -1401,7 +1410,7 @@ void QGraphicsProxyWidget::focusOutEvent(QFocusEvent *event)
         if (QWidget *focusWidget = d->widget->focusWidget()) {
             // QTBUG-88016 proxyWidget set QTextEdit(QLineEdit etc.) when input preview text,
             // inputMethod should be reset when proxyWidget lost focus
-            if (focusWidget && focusWidget->testAttribute(Qt::WA_InputMethodEnabled))
+            if (focusWidget->testAttribute(Qt::WA_InputMethodEnabled))
                 QApplication::inputMethod()->reset();
 
             d->removeSubFocusHelper(focusWidget, event->reason());
