@@ -83,8 +83,6 @@ public:
     \ingroup model-view
     \inmodule QtWidgets
 
-    \since 4.4
-
     When displaying data from models in Qt item views, e.g., a
     QTableView, the individual items are drawn by a delegate. Also,
     when an item is edited, it provides an editor widget, which is
@@ -115,7 +113,7 @@ public:
     \row    \li \l Qt::AccessibleDescriptionRole \li QString
     \row    \li \l Qt::AccessibleTextRole \li QString
     \endomit
-    \row    \li \l Qt::BackgroundRole \li QBrush \since 4.2
+    \row    \li \l Qt::BackgroundRole \li QBrush
     \row    \li \l Qt::CheckStateRole \li Qt::CheckState
     \row    \li \l Qt::DecorationRole \li QIcon, QPixmap, QImage and QColor
     \row    \li \l Qt::DisplayRole \li QString and types with a string representation
@@ -126,7 +124,7 @@ public:
     \row    \li \l Qt::StatusTipRole \li
     \endomit
     \row    \li \l Qt::TextAlignmentRole \li Qt::Alignment
-    \row    \li \l Qt::ForegroundRole \li QBrush \since 4.2
+    \row    \li \l Qt::ForegroundRole \li QBrush
     \omit
     \row    \li \l Qt::ToolTipRole
     \row    \li \l Qt::WhatsThisRole
@@ -137,12 +135,17 @@ public:
     instance provided by QItemEditorFactory is installed on all item
     delegates. You can set a custom factory using
     setItemEditorFactory() or set a new default factory with
-    QItemEditorFactory::setDefaultFactory(). It is the data stored in
-    the item model with the \l{Qt::}{EditRole} that is edited. See the
-    QItemEditorFactory class for a more high-level introduction to
-    item editor factories. The \l{Color Editor Factory Example}{Color
-    Editor Factory} example shows how to create custom editors with a
-    factory.
+    QItemEditorFactory::setDefaultFactory().
+
+    \snippet code/src_gui_itemviews_qitemeditorfactory.cpp setDefaultFactory
+
+    After the new factory has been set, all standard item delegates
+    will use it (i.e, also delegates that were created before the new
+    default factory was set).
+
+    It is the data stored in the item model with the \l{Qt::}{EditRole}
+    that is edited. See the QItemEditorFactory class for a more
+    high-level introduction to item editor factories.
 
     \section1 Subclassing QStyledItemDelegate
 
@@ -204,7 +207,7 @@ public:
     documentation for details.
 
     \sa {Delegate Classes}, QItemDelegate, QAbstractItemDelegate, QStyle,
-        {Star Delegate Example}, {Color Editor Factory Example}
+        {Star Delegate Example}
 */
 
 
