@@ -4,24 +4,18 @@
 #include "qtoolbar.h"
 
 #include <qapplication.h>
-#if QT_CONFIG(combobox)
-#include <qcombobox.h>
+#if QT_CONFIG(draganddrop)
+#include <qdrag.h>
 #endif
 #include <qevent.h>
 #include <qlayout.h>
 #include <qmainwindow.h>
 #include <qmenu.h>
-#if QT_CONFIG(menubar)
-#include <qmenubar.h>
-#endif
-#if QT_CONFIG(rubberband)
-#include <qrubberband.h>
-#endif
+#include <qmimedata.h>
 #include <qstylepainter.h>
 #include <qstyleoption.h>
 #include <qtoolbutton.h>
 #include <qwidgetaction.h>
-#include <qtimer.h>
 #include <private/qwidgetaction_p.h>
 #include <private/qmainwindowlayout_p.h>
 #include <private/qhighdpiscaling_p.h>
@@ -40,6 +34,8 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 // qmainwindow.cpp
 extern QMainWindowLayout *qt_mainwindow_layout(const QMainWindow *window);
 
@@ -105,8 +101,12 @@ void QToolBarPrivate::updateWindowFlags(bool floating, bool unplug)
 
     flags |= Qt::FramelessWindowHint;
 
-    if (unplug)
+#if QT_CONFIG(draganddrop)
+    // If we are performing a platform drag the flag is not needed and we want to avoid recreating
+    // the platform window when it would be removed later
+    if (unplug && !QMainWindowLayout::needsPlatformDrag())
         flags |= Qt::X11BypassWindowManagerHint;
+#endif
 
     q->setWindowFlags(flags);
 }
@@ -117,8 +117,6 @@ void QToolBarPrivate::setWindowState(bool floating, bool unplug, const QRect &re
     bool visible = !q->isHidden();
     bool wasFloating = q->isFloating(); // ...is also currently using popup menus
 
-    q->hide();
-
     updateWindowFlags(floating, unplug);
 
     if (floating != wasFloating)
@@ -172,12 +170,29 @@ void QToolBarPrivate::startDrag(bool moving)
     QMainWindowLayout *layout = qt_mainwindow_layout(win);
     Q_ASSERT(layout != nullptr);
 
+#if QT_CONFIG(draganddrop)
+    const bool wasFloating = q->isFloating();
+#endif
+
     if (!moving) {
-        state->widgetItem = layout->unplug(q);
+        state->widgetItem = layout->unplug(q, QDockWidgetPrivate::DragScope::Group);
         Q_ASSERT(state->widgetItem != nullptr);
     }
     state->dragging = !moving;
     state->moving = moving;
+
+#if QT_CONFIG(draganddrop)
+    if (QMainWindowLayout::needsPlatformDrag() && state->dragging) {
+        auto result = layout->performPlatformWidgetDrag(state->widgetItem, state->pressPos);
+        if (result == Qt::IgnoreAction && !wasFloating) {
+            layout->revert(state->widgetItem);
+            delete state;
+            state = nullptr;
+        } else {
+            endDrag();
+        }
+    }
+#endif
 }
 
 void QToolBarPrivate::endDrag()
@@ -243,6 +258,13 @@ bool QToolBarPrivate::mousePressEvent(QMouseEvent *event)
 
 bool QToolBarPrivate::mouseReleaseEvent(QMouseEvent*)
 {
+#if QT_CONFIG(draganddrop)
+    // if we are peforming a platform drag ignore the release here and  end the drag when the actual
+    // drag ends.
+    if (QMainWindowLayout::needsPlatformDrag())
+        return false;
+#endif
+
     if (state != nullptr) {
         endDrag();
         return true;
@@ -293,6 +315,11 @@ bool QToolBarPrivate::mouseMoveEvent(QMouseEvent *event)
                 q->grabMouse();
     }
 
+    if (!state) {
+        q->releaseMouse();
+        return true;
+    }
+
     if (state->dragging) {
         QPoint pos = event->globalPosition().toPoint();
         // if we are right-to-left, we move so as to keep the right edge the same distance
@@ -358,6 +385,10 @@ void QToolBarPrivate::plug(const QRect &r)
     \ingroup mainwindow-classes
     \inmodule QtWidgets
 
+    A toolbar is typically created by calling
+    \l QMainWindow::addToolBar(const QString &title), but it can also
+    be added as the first widget in a QVBoxLayout, for example.
+
     Toolbar buttons are added by adding \e actions, using addAction()
     or insertAction(). Groups of buttons can be separated using
     addSeparator() or insertSeparator(). If a toolbar button is not
@@ -381,7 +412,7 @@ void QToolBarPrivate::plug(const QRect &r)
     addWidget(). Please use widget actions created by inheriting QWidgetAction
     and implementing QWidgetAction::createWidget() instead.
 
-    \sa QToolButton, QMenu, QAction, {Qt Widgets - Application Example}
+    \sa QToolButton, QMenu, QAction
 */
 
 /*!
@@ -872,8 +903,13 @@ void QToolBar::changeEvent(QEvent *event)
         break;
     case QEvent::StyleChange:
         d->layout->invalidate();
-        if (!d->explicitIconSize)
-            setIconSize(QSize());
+        if (!d->explicitIconSize) {
+            QStyleOptionToolBar opt;
+            initStyleOption(&opt);
+            const int metric = style()->pixelMetric(QStyle::PM_ToolBarIconSize, &opt, this);
+            setIconSize({metric, metric});
+            d->explicitIconSize = false;
+        }
         d->layout->updateMarginAndSpacing();
         break;
     case QEvent::LayoutDirectionChange:
@@ -955,7 +991,8 @@ static void enableMacToolBar(QToolBar *toolbar, bool enable)
         return; // Not Cocoa platform plugin.
 
     typedef void (*SetContentBorderAreaEnabledFunction)(QWindow *window, void *identifier, bool enabled);
-    (reinterpret_cast<SetContentBorderAreaEnabledFunction>(function))(toolbar->window()->windowHandle(), toolbar, enable);
+    (reinterpret_cast<SetContentBorderAreaEnabledFunction>(QFunctionPointer(function)))(
+        toolbar->window()->windowHandle(), toolbar, enable);
 }
 #endif
 
@@ -1043,6 +1080,7 @@ bool QToolBar::event(QEvent *event)
             d->layout->setExpanded(false);
             break;
         }
+        break;
     default:
         break;
     }
