@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 #include <qtconcurrentrun.h>
 #include <QFuture>
 #include <QMutex>
@@ -815,7 +815,7 @@ public:
     void run() override {
         int iter = 60;
         while (--iter && !cancel.loadRelaxed())
-            QThread::currentThread()->msleep(25);
+            QThread::currentThread()->sleep(std::chrono::milliseconds{25});
     }
 };
 
@@ -1089,12 +1089,25 @@ void report3(QPromise<int> &promise)
     promise.addResult(1);
 }
 
+static void staticReport3(QPromise<int> &promise)
+{
+    promise.addResult(0);
+    promise.addResult(2);
+    promise.addResult(1);
+}
+
 void reportN(QPromise<double> &promise, int n)
 {
     for (int i = 0; i < n; ++i)
         promise.addResult(0);
 }
 
+static void staticReportN(QPromise<double> &promise, int n)
+{
+    for (int i = 0; i < n; ++i)
+        promise.addResult(0);
+}
+
 void reportString1(QPromise<QString> &promise, const QString &s)
 {
     promise.addResult(s);
@@ -1168,11 +1181,21 @@ void tst_QtConcurrentRun::withPromise()
     QCOMPARE(run(report3).results(),
              QList<int>({0, 2, 1}));
 
-    QCOMPARE(run(reportN, 4).results(),
-             QList<double>({0, 0, 0, 0}));
+    QCOMPARE(run(&staticReport3).results(),
+             QList<int>({0, 2, 1}));
+    QCOMPARE(run(staticReport3).results(),
+             QList<int>({0, 2, 1}));
+
+    QCOMPARE(run(&reportN, 2).results(),
+             QList<double>({0, 0}));
     QCOMPARE(run(reportN, 2).results(),
              QList<double>({0, 0}));
 
+    QCOMPARE(run(&staticReportN, 2).results(),
+             QList<double>({0, 0}));
+    QCOMPARE(run(staticReportN, 2).results(),
+             QList<double>({0, 0}));
+
     QString s = QLatin1String("string");
     const QString &crs = QLatin1String("cr string");
     const QString cs = QLatin1String("c string");
@@ -1259,11 +1282,21 @@ void tst_QtConcurrentRun::withPromiseInThreadPool()
     QCOMPARE(run(pool.data(), report3).results(),
              QList<int>({0, 2, 1}));
 
-    QCOMPARE(run(pool.data(), reportN, 4).results(),
-             QList<double>({0, 0, 0, 0}));
+    QCOMPARE(run(pool.data(), &staticReport3).results(),
+             QList<int>({0, 2, 1}));
+    QCOMPARE(run(pool.data(), staticReport3).results(),
+             QList<int>({0, 2, 1}));
+
+    QCOMPARE(run(pool.data(), &reportN, 2).results(),
+             QList<double>({0, 0}));
     QCOMPARE(run(pool.data(), reportN, 2).results(),
              QList<double>({0, 0}));
 
+    QCOMPARE(run(pool.data(), &staticReportN, 2).results(),
+             QList<double>({0, 0}));
+    QCOMPARE(run(pool.data(), staticReportN, 2).results(),
+             QList<double>({0, 0}));
+
     QString s = QLatin1String("string");
     const QString &crs = QLatin1String("cr string");
     const QString cs = QLatin1String("c string");
