@@ -1,39 +1,21 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Copyright (C) 2016 by Southwest Research Institute (R)
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// Copyright (C) 2016 by Southwest Research Institute (R)
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 #include <QFloat16>
+#include <QMetaType>
+#include <QTextStream>
+#include <QtCompilerDetection>
+
+#include <private/qcomparisontesthelper_p.h>
 
 #include <math.h>
 
 //#define DO_FULL_TEST
 
+using namespace Qt::StringLiterals;
+
 static_assert(sizeof(float) == sizeof(quint32), "Float not 32-bit");
 
 class tst_qfloat16: public QObject
@@ -41,6 +23,10 @@ class tst_qfloat16: public QObject
     Q_OBJECT
 
 private slots:
+    void compareCompiles();
+    void relationalOperatorsAreConstexpr();
+    void ordering_data();
+    void ordering();
     void fuzzyCompare_data();
     void fuzzyCompare();
     void fuzzyIsNull_data();
@@ -66,8 +52,203 @@ private slots:
     void properties();
     void limits();
     void mantissaOverflow();
+    void dataStream();
+    void textStream();
+
+    // std::format support
+    void formatCompileTime();
+    void format_data();
+    void format();
+    void formatMultiArg();
 };
 
+void tst_qfloat16::compareCompiles()
+{
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16>();
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, float>();
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, double>();
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, long double>();
+#if QFLOAT16_IS_NATIVE
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, qfloat16::NativeType>();
+#endif
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, int>();
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, qint8>();
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, quint8>();
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, qint16>();
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, quint16>();
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, char16_t>();
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, long>();
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, unsigned long>();
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, wchar_t>();
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, qint32>();
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, quint32>();
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, qint64>();
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, quint64>();
+#ifdef QT_SUPPORTS_INT128
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, qint128>();
+    QTestPrivate::testAllComparisonOperatorsCompile<qfloat16, quint128>();
+#endif
+}
+
+void tst_qfloat16::relationalOperatorsAreConstexpr()
+{
+#if QFLOAT16_IS_NATIVE
+
+#define CHECK_CONSTEXPR(Type) \
+    do { \
+        constexpr qfloat16 lhs = qfloat16(0.0f); \
+        constexpr Type rhs = 1; \
+        static_assert(lhs < rhs); \
+        static_assert(rhs >= lhs); \
+    } while (false)
+
+    CHECK_CONSTEXPR(qfloat16);
+    CHECK_CONSTEXPR(float);
+    CHECK_CONSTEXPR(double);
+    CHECK_CONSTEXPR(long double);
+    CHECK_CONSTEXPR(qfloat16::NativeType);
+    CHECK_CONSTEXPR(qint8);
+    CHECK_CONSTEXPR(quint8);
+    CHECK_CONSTEXPR(qint16);
+    CHECK_CONSTEXPR(quint16);
+    CHECK_CONSTEXPR(qint32);
+    CHECK_CONSTEXPR(quint32);
+    CHECK_CONSTEXPR(long);
+    CHECK_CONSTEXPR(unsigned long);
+    CHECK_CONSTEXPR(qint64);
+    CHECK_CONSTEXPR(quint64);
+#ifdef QT_SUPPORTS_INT128
+    CHECK_CONSTEXPR(qint128);
+    CHECK_CONSTEXPR(quint128);
+#endif
+
+#undef CHECK_CONSTEXPR
+
+#else
+    QSKIP("This check is only relevant for native float16 types");
+#endif // QFLOAT16_IS_NATIVE
+}
+
+void tst_qfloat16::ordering_data()
+{
+    QTest::addColumn<float>("left");
+    QTest::addColumn<float>("right");
+
+    auto row = [](float left, float right) {
+        QTest::addRow("%f_vs_%f", left, right) << left << right;
+    };
+
+    row(0.0f, 0.0f);
+    row(0.000001f, 0.0f);
+    row(0.0f, 0.000001f);
+    row(-1.000001f, 1.000001f);
+    const float nan = std::numeric_limits<float>::quiet_NaN();
+    const float inf = std::numeric_limits<float>::infinity();
+    row(nan, nan);
+    row(nan, inf);
+    row(inf, nan);
+    row(-inf, nan);
+    row(nan, -inf);
+    row(-inf, inf);
+    row(inf, -inf);
+    row(-inf, 0.0f);
+    row(0.0f, inf);
+    row(0.0f, nan);
+    row(nan, 0.0f);
+    row(2.0f, 314.159f);
+    row(-314.159f, 2.0f);
+    row(-2.0f, 314.159f);
+    row(nan, 314.159f);
+    row(-314.159f, inf);
+    row(-inf, 314.159f);
+    row(2.0f, -inf);
+    row(-2.0f, nan);
+    row(-inf, -2.0f);
+
+    // testing with values outside qfloat16 range
+    row(0.0f, 13e5f);
+    // generateRow(inf, 13e5f); // fails qfloat16 vs qfloat16 and qfloat16 vs int (QTBUG-118193)
+    row(0.0f, -13e5f);
+    // generateRow(-inf, -13e5f); // fails qfloat16 vs qfloat16 and qfloat16 vs int (QTBUG-118193)
+}
+
+void tst_qfloat16::ordering()
+{
+    QFETCH(float, left);
+    QFETCH(float, right);
+
+    const auto expectedOrder = Qt::compareThreeWay(left, right);
+    const auto lhs = qfloat16(left);
+
+#define POSTCHECK(msg) \
+    if (QTest::currentTestFailed()) { qDebug(msg); return; }
+
+#define CHECK_FP(RHS) \
+    do { \
+        QTestPrivate::testAllComparisonOperators(lhs, static_cast<RHS>(right), expectedOrder); \
+        POSTCHECK("qfloat16 vs " #RHS " comparison failed") \
+    } while (false) \
+    /* END */
+
+    CHECK_FP(qfloat16);
+    CHECK_FP(float);
+    CHECK_FP(double);
+    // Qt built for VxWorks x86_64 fails to link due to undefined reference to
+    // __extendhfxf2 when below check is enabled.
+    // This has been acknowledged by WindRiver as a bug
+    // and is being tracked in ticket 00154117
+#if !(defined(Q_OS_VXWORKS) && defined(Q_PROCESSOR_X86_64))
+    CHECK_FP(long double);
+#endif
+
+#undef CHECK_FP
+
+    auto check_int = [=](auto rhs) {
+        // check that we're in range before converting, otherwise
+        // [conv.fpint]/1 says it would be UB
+        using RHS = decltype(rhs);
+        if (right > double(std::numeric_limits<RHS>::max()))
+            return;
+        if (auto min = std::numeric_limits<RHS>::min(); min != 0 && right < double(min))
+            return;
+        rhs = RHS(right);
+        const auto expectedRes = Qt::compareThreeWay(left, rhs);
+        QTestPrivate::testAllComparisonOperators(lhs, rhs, expectedRes);
+    };
+#define CHECK_INT(RHS) \
+    do { \
+        check_int(static_cast<RHS>(0)); \
+        POSTCHECK("qfloat16 vs " #RHS " comparison failed") \
+    } while (false) \
+    /* END */
+
+    if (qIsFinite(right)) {
+        CHECK_INT(int);
+        CHECK_INT(qint8);
+        CHECK_INT(signed char);
+        CHECK_INT(qint16);
+        CHECK_INT(qint32);
+        CHECK_INT(qint64);
+#ifdef QT_SUPPORTS_INT128
+        CHECK_INT(qint128);
+#endif
+        if (right >= 0) {
+            CHECK_INT(unsigned int);
+            CHECK_INT(quint8);
+            CHECK_INT(unsigned char);
+            CHECK_INT(quint16);
+            CHECK_INT(quint32);
+            CHECK_INT(quint64);
+    #ifdef QT_SUPPORTS_INT128
+            CHECK_INT(quint128);
+    #endif
+        }
+    }
+
+#undef CHECK_INT
+#undef POSTCHECK
+}
+
 void tst_qfloat16::fuzzyCompare_data()
 {
     QTest::addColumn<qfloat16>("val1");
@@ -213,13 +394,7 @@ void tst_qfloat16::qNaN()
     QVERIFY(qIsNaN(nan));
     QVERIFY(qIsNaN(nan + one));
     QVERIFY(qIsNaN(-nan));
-#ifdef Q_CC_INTEL
-    QEXPECT_FAIL("", "ICC optimizes zero * anything to zero", Continue);
-#endif
     QVERIFY(qIsNaN(nan * zero));
-#ifdef Q_CC_INTEL
-    QEXPECT_FAIL("", "ICC optimizes zero * anything to zero", Continue);
-#endif
     QVERIFY(qIsNaN(Bounds::infinity() * zero));
 
     QVERIFY(!nan.isNormal());
@@ -332,7 +507,7 @@ void tst_qfloat16::promotionTests()
     QCOMPARE(sizeof(double),sizeof(qfloat16(1.f)*1));
     QCOMPARE(sizeof(double),sizeof(qfloat16(1.f)/1));
 
-    QCOMPARE(QString::number(1.f),QString::number(qfloat16(1.f)));
+    QCOMPARE(QString::number(1.f),QString::number(double(qfloat16(1.f))));
 }
 
 void tst_qfloat16::arithOps_data()
@@ -550,22 +725,29 @@ void tst_qfloat16::properties()
     // and the same supposed to be for qfloat16.
 #if !defined(Q_CC_GHS)
     QVERIFY(Bounds::is_iec559);
-#else
+#endif //Q_CC_GHS
+#if QT_CONFIG(signaling_nan)
     // Technically, presence of NaN and infinities are implied from the above check, but that checkings GHS compiler complies.
     QVERIFY(Bounds::has_infinity && Bounds::has_quiet_NaN && Bounds::has_signaling_NaN);
-#endif // Q_CC_GHS
+#endif
     QVERIFY(Bounds::is_bounded);
     QVERIFY(!Bounds::is_modulo);
     QVERIFY(!Bounds::traps);
     QVERIFY(Bounds::has_infinity);
     QVERIFY(Bounds::has_quiet_NaN);
+#if QT_CONFIG(signaling_nan)
     QVERIFY(Bounds::has_signaling_NaN);
+#endif
+    QT_WARNING_PUSH
+    QT_WARNING_DISABLE_DEPRECATED // has_denorm is deprecated in C++23
 #if !defined(Q_CC_GHS)
     QCOMPARE(Bounds::has_denorm, std::denorm_present);
 #else
     // For GHS compiler the "denorm_indeterminite" is the expected return value.
     QCOMPARE(Bounds::has_denorm, std::denorm_indeterminate);
 #endif // Q_CC_GHS
+    QT_WARNING_POP
+
     QCOMPARE(Bounds::round_style, std::round_to_nearest);
     QCOMPARE(Bounds::radix, 2);
     // Untested: has_denorm_loss
@@ -652,12 +834,199 @@ void tst_qfloat16::mantissaOverflow()
     float f;
     memcpy(&f, &in, 4);
 
-    qfloat16 f16 = f;
+    qfloat16 f16 = qfloat16(f);
     qfloat16 f16s[1];
     qFloatToFloat16(f16s, &f, 1);
     QCOMPARE(f16, f16s[0]);
     QVERIFY(qIsNaN(f16));
 }
 
+void tst_qfloat16::dataStream()
+{
+    QByteArray ba;
+    QDataStream ds(&ba, QIODevice::ReadWrite);
+    ds << qfloat16(1.5) << qfloat16(-1);
+    QCOMPARE(ba.size(), 4);
+    QCOMPARE(ds.status(), QDataStream::Ok);
+    QCOMPARE(ba, QByteArray("\x3e\0\xbc\0", 4));
+
+    ds.device()->seek(0);
+    ds.resetStatus();
+    ds.setByteOrder(QDataStream::LittleEndian);
+    ds << qfloat16(0) << qfloat16(-1);
+    QCOMPARE(ds.status(), QDataStream::Ok);
+    QCOMPARE(ba, QByteArray("\0\0\0\xbc", 4));
+
+    ds.device()->seek(0);
+    ds.resetStatus();
+    qfloat16 zero = 1;
+    ds >> zero;
+    QCOMPARE(ds.status(), QDataStream::Ok);
+    QCOMPARE(zero, qfloat16(0));
+
+    ds.device()->seek(0);
+    ds.resetStatus();
+    QMetaType mt = QMetaType(QMetaType::Float16);
+    QVERIFY(mt.save(ds, &zero));
+
+    ds.device()->seek(0);
+    ds.resetStatus();
+    zero = -1;
+    QVERIFY(mt.load(ds, &zero));
+    QCOMPARE(zero, qfloat16(0));
+}
+
+void tst_qfloat16::textStream()
+{
+    QString buffer;
+    {
+        QTextStream ts(&buffer);
+        ts << qfloat16(0) << Qt::endl << qfloat16(1.5);
+        QCOMPARE(ts.status(), QTextStream::Ok);
+    }
+    QCOMPARE(buffer, "0\n1.5");
+
+    {
+        QTextStream ts(&buffer);
+        qfloat16 zero = qfloat16(-2.5), threehalves = 1234;
+        ts >> zero >> threehalves;
+        QCOMPARE(ts.status(), QTextStream::Ok);
+        QCOMPARE(zero, qfloat16(0));
+        QCOMPARE(threehalves, 1.5);
+    }
+}
+
+void tst_qfloat16::formatCompileTime()
+{
+#ifdef QT_SUPPORTS_STD_FORMAT
+    // Starting from __cpp_lib_format == 202106L,
+    // std::format requires the format string to be evaluated at compile-time,
+    // so check it here.
+
+    const qfloat16 val{1.234f};
+    std::locale loc{"C"};
+
+    // char
+    std::string buffer;
+    std::format_to(std::back_inserter(buffer), "{}", val);
+    std::format_to(std::back_inserter(buffer), "{:*>15.7f}", val);
+    std::format_to(std::back_inserter(buffer), "{:*^+#15.7g}", val);
+    std::format_to(std::back_inserter(buffer), "{:*<-#15.7A}", val);
+    std::format_to(std::back_inserter(buffer), "{:*^ 15.7e}", val);
+    std::format_to(std::back_inserter(buffer), loc, "{:*^10.3Lf}", val);
+    std::format_to(std::back_inserter(buffer), loc, "{:*< 10.7LE}", val);
+
+    // wchar_t
+    std::wstring wbuffer;
+    std::format_to(std::back_inserter(wbuffer), L"{}", val);
+    std::format_to(std::back_inserter(wbuffer), L"{:*>15.7f}", val);
+    std::format_to(std::back_inserter(wbuffer), L"{:*^+#15.7g}", val);
+    std::format_to(std::back_inserter(wbuffer), L"{:*<-#15.7A}", val);
+    std::format_to(std::back_inserter(wbuffer), L"{:*^ 15.7e}", val);
+    std::format_to(std::back_inserter(wbuffer), loc, L"{:*^10.3Lf}", val);
+    std::format_to(std::back_inserter(wbuffer), loc, L"{:*< 10.7LE}", val);
+#else
+    QSKIP("This test requires std::format support!");
+#endif // QT_SUPPORTS_STD_FORMAT
+}
+
+void tst_qfloat16::format_data()
+{
+#ifdef QT_SUPPORTS_STD_FORMAT
+    QTest::addColumn<QString>("format");
+    QTest::addColumn<qfloat16>("value");
+    QTest::addColumn<std::locale>("locale");
+    QTest::addColumn<QString>("expectedString");
+
+    auto row = [](const QString &format, qfloat16 val, const QString &expected,
+                  const std::locale &loc = std::locale::classic())
+    {
+        QTest::addRow("%s:%s", loc.name().c_str(), qPrintable(format))
+                << format << val << loc << expected;
+    };
+
+    row(u"{}"_s, qfloat16(1.f), u"1"_s);
+    row(u"{:#}"_s, qfloat16(1.f), u"1."_s);
+    row(u"{:f}"_s, qfloat16(1.f), u"1.000000"_s);
+    row(u"{:*>10.2a}"_s, qfloat16(-1.23f), u"**-1.3bp+0"_s);
+
+    try {
+        // check if this locale is a) valid and b) works as expected
+#if defined(Q_CC_MSVC)
+        std::locale loc("de-DE");
+#else
+        std::locale loc("de_DE");
+#endif
+        if (std::format(loc, "{:L}", 1.25) == "1,25") {
+            row(u"{:+Lf}"_s, qfloat16(1.f), u"+1,000000"_s, loc);
+            row(u"{:*^10.3LF}"_s, qfloat16(-0.1234f), u"**-0,123**"_s, loc);
+            row(u"{:*^#10.4Lg}"_s, qfloat16(-1.f), u"**-1,000**"_s, loc);
+            row(u"{:*<14.3LE}"_s, qfloat16(-0.1234f), u"-1,234E-01****"_s, loc);
+        }
+    } catch (const std::runtime_error &) {
+        // locale doesn't exist (std::locale constructor threw)
+    }
+#else
+    QSKIP("This test requires std::format support!");
+#endif // QT_SUPPORTS_STD_FORMAT
+}
+
+void tst_qfloat16::format()
+{
+#ifdef QT_SUPPORTS_STD_FORMAT
+    QFETCH(const QString, format);
+    QFETCH(const qfloat16, value);
+    QFETCH(const std::locale, locale);
+    QFETCH(const QString, expectedString);
+
+    // char
+    {
+        std::string buffer;
+        const auto formatStr = format.toStdString();
+        std::vformat_to(std::back_inserter(buffer), locale, formatStr,
+                        std::make_format_args(value));
+        const QString actualString = QString::fromStdString(buffer);
+        QCOMPARE_EQ(actualString, expectedString);
+    }
+
+    // wchar_t
+    {
+        std::wstring buffer;
+        const auto formatStr = format.toStdWString();
+        std::vformat_to(std::back_inserter(buffer), locale, formatStr,
+                        std::make_wformat_args(value));
+        const QString actualString = QString::fromStdWString(buffer);
+        QCOMPARE_EQ(actualString, expectedString);
+    }
+#else
+    QSKIP("This test requires std::format support!");
+#endif // QT_SUPPORTS_STD_FORMAT
+}
+
+void tst_qfloat16::formatMultiArg()
+{
+#ifdef QT_SUPPORTS_STD_FORMAT
+    const qfloat16 v1{-0.1234f};
+    const qfloat16 v2{5.67f};
+
+    const QString expectedString = u"**+5.67**_*****-1.234E-01"_s;
+    // char
+    {
+        std::string buffer;
+        std::format_to(std::back_inserter(buffer), "{1:*^+9.2f}_{0:*>15.3E}", v1, v2);
+        QCOMPARE_EQ(QString::fromStdString(buffer), expectedString);
+    }
+
+    // wchar_t
+    {
+        std::wstring buffer;
+        std::format_to(std::back_inserter(buffer), L"{1:*^+9.2f}_{0:*>15.3E}", v1, v2);
+        QCOMPARE_EQ(QString::fromStdWString(buffer), expectedString);
+    }
+#else
+    QSKIP("This test requires std::format support!");
+#endif // QT_SUPPORTS_STD_FORMAT
+}
+
 QTEST_APPLESS_MAIN(tst_qfloat16)
 #include "tst_qfloat16.moc"
