@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
@@ -9,6 +9,7 @@
 #include <QProcess>
 #endif
 #include <QtCore/QDebug>
+#include <QtCore/QElapsedTimer>
 #include <QtCore/QFileInfo>
 #include <QtCore/QDir>
 #include <QtCore/QVariant>
@@ -21,10 +22,13 @@
 
 #ifdef Q_OS_WIN
 #  include <QtGui/private/qguiapplication_p.h>
-#  include <QtGui/private/qwindowsmime_p.h>
+#  include <QtGui/qwindowsmimeconverter.h>
 #  include <QtGui/qpa/qplatformintegration.h>
+#  include <QtCore/qt_windows.h>
 #endif
 
+using namespace Qt::StringLiterals;
+
 class tst_QClipboard : public QObject
 {
     Q_OBJECT
@@ -41,6 +45,7 @@ private slots:
     void testSignals();
     void setMimeData();
     void clearBeforeSetText();
+    void getTextFromHTMLMimeType();
 #  ifdef Q_OS_WIN
     void testWindowsMimeRegisterType();
     void testWindowsMime_data();
@@ -56,12 +61,16 @@ void tst_QClipboard::initTestCase()
 #endif
     if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
         QSKIP("Wayland: Manipulating the clipboard requires real input events. Can't auto test.");
+    if (qgetenv("XDG_CURRENT_DESKTOP").toLower().contains("ubuntu:gnome")
+        && QSysInfo::productVersion() == "24.04"_L1
+        && QSysInfo::prettyProductName() == "Ubuntu 24.04 LTS"_L1)
+        QSKIP("This hangs on Ubuntu 24.04(.0) GNOME/X11, see also QTBUG-129567.");
 }
 
 #if QT_CONFIG(clipboard)
 void tst_QClipboard::init()
 {
-#if QT_CONFIG(process)
+#if QT_CONFIG(process) && !defined(Q_OS_ANDROID)
     const QString testdataDir = QFileInfo(QFINDTESTDATA("copier")).absolutePath();
     QVERIFY2(QDir::setCurrent(testdataDir), qPrintable("Could not chdir to " + testdataDir));
 #endif
@@ -424,12 +433,30 @@ void tst_QClipboard::clearBeforeSetText()
     QCOMPARE(QGuiApplication::clipboard()->text(), text);
 }
 
+void tst_QClipboard::getTextFromHTMLMimeType()
+{
+    QClipboard * clipboard = QGuiApplication::clipboard();
+    QMimeData * mimeData = new QMimeData();
+    const QString testString("TEST");
+    const QString htmlString(QLatin1String("<html><body>") + testString + QLatin1String("</body></html>"));
+
+    mimeData->setText(testString);
+    mimeData->setHtml(htmlString);
+    clipboard->setMimeData(mimeData);
+
+    QCOMPARE(clipboard->text(), testString);
+    QVERIFY(clipboard->mimeData()->hasText());
+    QVERIFY(clipboard->mimeData()->hasHtml());
+    QCOMPARE(clipboard->mimeData()->text(), testString);
+    QCOMPARE(clipboard->mimeData()->html(), htmlString);
+}
+
 #  ifdef Q_OS_WIN
 
-using QWindowsMime = QNativeInterface::Private::QWindowsMime;
+using QWindowsMimeConverter = QWindowsMimeConverter;
 using QWindowsApplication = QNativeInterface::Private::QWindowsApplication;
 
-class TestMime : public QWindowsMime
+class TestMime : public QWindowsMimeConverter
 {
 public:
     bool canConvertFromMime(const FORMATETC &, const QMimeData *) const override
