@@ -1,35 +1,11 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QtTest/QTest>
 #include <QtTest/QTestEventLoop>
 
+#include <QtCore/QElapsedTimer>
 #include <QtCore/QQueue>
 #include <QtCore/QString>
 #include <QtCore/QCoreApplication>
@@ -48,6 +24,8 @@
 
 #include "../../../network-settings.h"
 
+using namespace std::chrono_literals;
+
 class tst_QSocks5SocketEngine : public QObject, public QAbstractSocketEngineReceiver
 {
     Q_OBJECT
@@ -73,14 +51,14 @@ private slots:
     void incomplete();
 
 protected slots:
-    void proxyAuthenticationRequired(const QNetworkProxy &proxy, QAuthenticator *auth);
+    void proxyAuthenticationRequired(const QNetworkProxy &proxy, QAuthenticator *auth) override;
 
 private:
-    void readNotification() { }
-    void writeNotification() { }
-    void closeNotification() { }
-    void exceptionNotification() { }
-    void connectionNotification() { }
+    void readNotification() override { }
+    void writeNotification() override { }
+    void closeNotification() override { }
+    void exceptionNotification() override { }
+    void connectionNotification() override { }
 };
 
 class MiniSocks5ResponseHandler : public QObject
@@ -137,12 +115,17 @@ private slots:
 
 void tst_QSocks5SocketEngine::initTestCase()
 {
+    if (qEnvironmentVariable("QT_QPA_PLATFORM").contains("offscreen")
+          && !qEnvironmentVariableIsEmpty("QEMU_LD_PREFIX"))
+        QSKIP("Not support yet for B2Qt");
+
 #ifdef QT_TEST_SERVER
      QVERIFY(QtNetworkSettings::verifyConnection(QtNetworkSettings::socksProxyServerName(), 1080));
      QVERIFY(QtNetworkSettings::verifyConnection(QtNetworkSettings::httpServerName(), 80));
      QVERIFY(QtNetworkSettings::verifyConnection(QtNetworkSettings::imapServerName(), 143));
 #else
-    QVERIFY(QtNetworkSettings::verifyTestNetworkSettings());
+    if (!QtNetworkSettings::verifyTestNetworkSettings())
+        QSKIP("No network test server available");
 #endif
 }
 
@@ -282,7 +265,7 @@ void tst_QSocks5SocketEngine::errorTest()
     socket.setProxy(QNetworkProxy(QNetworkProxy::Socks5Proxy, hostname, port, username, username));
     socket.connectToHost("0.1.2.3", 12345);
 
-    connect(&socket, SIGNAL(error(QAbstractSocket::SocketError)),
+    connect(&socket, SIGNAL(errorOccurred(QAbstractSocket::SocketError)),
             &QTestEventLoop::instance(), SLOT(exitLoop()));
     QTestEventLoop::instance().enterLoop(10);
     QVERIFY(!QTestEventLoop::instance().timeout());
@@ -307,12 +290,15 @@ void tst_QSocks5SocketEngine::simpleConnectToIMAP()
     QCOMPARE(socketDevice.state(), QAbstractSocket::ConnectedState);
     QCOMPARE(socketDevice.peerAddress(), QtNetworkSettings::imapServerIp());
 
-    // Wait for the greeting
-    QVERIFY(socketDevice.waitForRead());
+    // Wait for the greeting, if it hasn't arrived yet
+    qint64 available = socketDevice.bytesAvailable();
+    if (available == 0) {
+        QVERIFY2(socketDevice.waitForRead(), qPrintable("Socket error:" + socketDevice.errorString()));
+        available = socketDevice.bytesAvailable();
+    }
+    QVERIFY(available > 0);
 
     // Read the greeting
-    qint64 available = socketDevice.bytesAvailable();
-    QVERIFY(available > 0);
     QByteArray array;
     array.resize(available);
     QVERIFY(socketDevice.read(array.data(), array.size()) == available);
@@ -358,7 +344,7 @@ void tst_QSocks5SocketEngine::simpleErrorsAndStates()
         QCOMPARE(socketDevice.state(), QAbstractSocket::UnconnectedState);
         QVERIFY(!socketDevice.connectToHost(QHostInfo::fromName(QtNetworkSettings::socksProxyServerName()).addresses().first(), 8088));
         QCOMPARE(socketDevice.state(), QAbstractSocket::ConnectingState);
-        if (socketDevice.waitForWrite(15000)) {
+        if (socketDevice.waitForWrite(15s)) {
             QVERIFY(socketDevice.state() == QAbstractSocket::UnconnectedState ||
                     socketDevice.state() == QAbstractSocket::ConnectedState);
         } else {
@@ -448,7 +434,7 @@ void tst_QSocks5SocketEngine::serverTest()
     QCOMPARE(server.state(), QAbstractSocket::BoundState);
 
     // Listen for incoming connections
-    QVERIFY(server.listen());
+    QVERIFY(server.listen(50));
     QCOMPARE(server.state(), QAbstractSocket::ListeningState);
 
     // Initialize a Tcp socket
@@ -752,7 +738,7 @@ void tst_QSocks5SocketEngine::downloadBigFile()
                     QTestEventLoop::instance().exitLoop();
             });
 
-    connect(&socket, QOverload<QAbstractSocket::SocketError>::of(&QAbstractSocket::error),
+    connect(&socket, &QAbstractSocket::errorOccurred,
             [&socket, &stopWatch] (QAbstractSocket::SocketError errorCode)
             {
                 qWarning().noquote().nospace() << QTest::currentTestFunction()
@@ -932,13 +918,13 @@ void tst_QSocks5SocketEngine::fragmentation_data()
     responses << authMethodBasic << authSuccess.left(1) << authSuccess.mid(1) << connectResponseIPv4;
     QTest::newRow("auth-response") << responses;
 
-    for (int i = 1; i < connectResponseIPv4.length() - 1; i++) {
+    for (int i = 1; i < connectResponseIPv4.size() - 1; i++) {
         responses.clear();
         responses << authMethodNone << connectResponseIPv4.left(i) << connectResponseIPv4.mid(i);
         QTest::newRow(qPrintable(QString("connect-response-ipv4-") + QString::number(i))) << responses;
     }
 
-    for (int i = 1; i < connectResponseIPv6.length() - 1; i++) {
+    for (int i = 1; i < connectResponseIPv6.size() - 1; i++) {
         responses.clear();
         responses << authMethodNone << connectResponseIPv6.left(i) << connectResponseIPv6.mid(i);
         QTest::newRow(qPrintable(QString("connect-response-ipv6-") + QString::number(i))) << responses;
@@ -981,13 +967,13 @@ void tst_QSocks5SocketEngine::incomplete_data()
     responses << authMethodBasic << authSuccess.left(1);
     QTest::newRow("auth-response") << responses;
 
-    for (int i = 1; i < connectResponseIPv4.length() - 1; i++) {
+    for (int i = 1; i < connectResponseIPv4.size() - 1; i++) {
         responses.clear();
         responses << authMethodNone << connectResponseIPv4.left(i);
         QTest::newRow(qPrintable(QString("connect-response-ipv4-") + QString::number(i))) << responses;
     }
 
-    for (int i = 1; i < connectResponseIPv6.length() - 1; i++) {
+    for (int i = 1; i < connectResponseIPv6.size() - 1; i++) {
         responses.clear();
         responses << authMethodNone << connectResponseIPv6.left(i);
         QTest::newRow(qPrintable(QString("connect-response-ipv6-") + QString::number(i))) << responses;
@@ -1005,7 +991,7 @@ void tst_QSocks5SocketEngine::incomplete()
 
     connect(&socket, SIGNAL(connected()),
             &QTestEventLoop::instance(), SLOT(exitLoop()));
-    connect(&socket, SIGNAL(error(QAbstractSocket::SocketError)),
+    connect(&socket, SIGNAL(errorOccurred(QAbstractSocket::SocketError)),
             &QTestEventLoop::instance(), SLOT(exitLoop()));
     QTestEventLoop::instance().enterLoop(70);
     QVERIFY(!QTestEventLoop::instance().timeout());
