@@ -1,9 +1,22 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
-#include <QtGui>
-#include <QtWidgets>
-#include <QTest>
+#include <QtTest/qtest.h>
+
+#include <QtWidgets/qapplication.h>
+#include <QtWidgets/qscroller.h>
+#include <QtWidgets/qwidget.h>
+
+#include "private/qscroller_p.h"
+
+#include <QtGui/qevent.h>
+#include <QtGui/qpointingdevice.h>
+#include <QtGui/qstylehints.h>
+
+#include <QtCore/qeasingcurve.h>
+#include <QtCore/qpoint.h>
+#include <QtCore/qrect.h>
+
 #include <QtGui/private/qevent_p.h>
 #include <QtGui/private/qeventpoint_p.h>
 #include <qpa/qwindowsysteminterface.h>
@@ -108,7 +121,7 @@ private slots:
     void mouseEventTimestamp();
 
 private:
-    QPointingDevice *m_touchScreen = QTest::createTouchDevice();
+    const std::unique_ptr<QPointingDevice> m_touchScreen{QTest::createTouchDevice()};
 };
 
 /*! \internal
@@ -133,7 +146,7 @@ void tst_QScroller::kineticScroll(tst_QScrollerWidget *sw, QPointF from, QPoint
     QMutableEventPoint::setGlobalPosition(touchPoint, touchStart);
 
     QTouchEvent touchEvent1(QEvent::TouchBegin,
-                            m_touchScreen,
+                            m_touchScreen.get(),
                             Qt::NoModifier,
                             (QList<QTouchEvent::TouchPoint>() << touchPoint));
 
@@ -148,7 +161,7 @@ void tst_QScroller::kineticScroll(tst_QScrollerWidget *sw, QPointF from, QPoint
     QMutableEventPoint::setGlobalPosition(touchPoint, touchUpdate);
     QMutableEventPoint::setState(touchPoint, QEventPoint::State::Updated);
     QTouchEvent touchEvent2(QEvent::TouchUpdate,
-                            m_touchScreen,
+                            m_touchScreen.get(),
                             Qt::NoModifier,
                             (QList<QEventPoint>() << touchPoint));
     QApplication::sendEvent(sw, &touchEvent2);
@@ -172,7 +185,7 @@ void tst_QScroller::kineticScroll(tst_QScrollerWidget *sw, QPointF from, QPoint
     QMutableEventPoint::setGlobalPosition(touchPoint, touchEnd);
     QMutableEventPoint::setState(touchPoint, QEventPoint::State::Released);
     QTouchEvent touchEvent5(QEvent::TouchEnd,
-                            m_touchScreen,
+                            m_touchScreen.get(),
                             Qt::NoModifier,
                             (QList<QEventPoint>() << touchPoint));
     QApplication::sendEvent(sw, &touchEvent5);
@@ -200,7 +213,7 @@ void tst_QScroller::kineticScrollNoTest(tst_QScrollerWidget *sw, QPointF from, Q
     QMutableEventPoint::setScenePosition(touchPoint, touchStart);
     QMutableEventPoint::setGlobalPosition(touchPoint, touchStart);
     QTouchEvent touchEvent1(QEvent::TouchBegin,
-                            m_touchScreen,
+                            m_touchScreen.get(),
                             Qt::NoModifier,
                             (QList<QEventPoint>() << touchPoint));
     QApplication::sendEvent(sw, &touchEvent1);
@@ -212,7 +225,7 @@ void tst_QScroller::kineticScrollNoTest(tst_QScrollerWidget *sw, QPointF from, Q
     QMutableEventPoint::setScenePosition(touchPoint, touchUpdate);
     QMutableEventPoint::setGlobalPosition(touchPoint, touchUpdate);
     QTouchEvent touchEvent2(QEvent::TouchUpdate,
-                            m_touchScreen,
+                            m_touchScreen.get(),
                             Qt::NoModifier,
                             (QList<QEventPoint>() << touchPoint));
     QApplication::sendEvent(sw, &touchEvent2);
@@ -225,7 +238,7 @@ void tst_QScroller::kineticScrollNoTest(tst_QScrollerWidget *sw, QPointF from, Q
     QMutableEventPoint::setScenePosition(touchPoint, touchEnd);
     QMutableEventPoint::setGlobalPosition(touchPoint, touchEnd);
     QTouchEvent touchEvent5(QEvent::TouchEnd,
-                            m_touchScreen,
+                            m_touchScreen.get(),
                             Qt::NoModifier,
                             (QList<QEventPoint>() << touchPoint));
     QApplication::sendEvent(sw, &touchEvent5);
@@ -324,7 +337,6 @@ void tst_QScroller::scrollTo()
 {
     QScopedPointer<tst_QScrollerWidget> sw(new tst_QScrollerWidget);
     sw->show();
-    QApplicationPrivate::setActiveWindow(sw.data());
     if (!QTest::qWaitForWindowExposed(sw.data()) || !QTest::qWaitForWindowActive(sw.data()))
         QSKIP("Failed to show and activate window");
 
@@ -356,7 +368,6 @@ void tst_QScroller::scroll()
     QScroller::grabGesture(sw.data(), QScroller::TouchGesture);
     sw->setGeometry(100, 100, 400, 300);
     sw->show();
-    QApplicationPrivate::setActiveWindow(sw.data());
     if (!QTest::qWaitForWindowExposed(sw.data()) || !QTest::qWaitForWindowActive(sw.data()))
         QSKIP("Failed to show and activate window");
 
@@ -397,7 +408,6 @@ void tst_QScroller::overshoot()
     QScroller::grabGesture(sw.data(), QScroller::TouchGesture);
     sw->setGeometry(100, 100, 400, 300);
     sw->show();
-    QApplicationPrivate::setActiveWindow(sw.data());
     if (!QTest::qWaitForWindowExposed(sw.data()) || !QTest::qWaitForWindowActive(sw.data()))
         QSKIP("Failed to show and activate window");
 
@@ -466,6 +476,26 @@ void tst_QScroller::overshoot()
     QVERIFY(qFuzzyCompare(sw->currentPos.y(), 500));
     QCOMPARE(sw->receivedOvershoot, false);
 
+    // -- try to scroll with overshoot (always off, slow)
+    sw->reset();
+    sw->scrollArea = QRectF(0, 0, 1000, 1000);
+
+    sp1.setScrollMetric(QScrollerProperties::HorizontalOvershootPolicy, QVariant::fromValue(QScrollerProperties::OvershootAlwaysOff));
+    s1->setScrollerProperties(sp1);
+    kineticScrollNoTest(sw.data(), QPointF(500, 500), QPoint(0, 0), QPoint(200, 0), QPoint(215, 0));
+
+    // Check that segment parameters are consistent
+    QScrollerPrivate* priv = s1->d_func();
+    QVERIFY(priv->xSegments.size() == 1);
+    auto& segment = priv->xSegments.head();
+    QCOMPARE_LT(segment.startPos + segment.deltaPos, segment.stopPos);
+
+    QTRY_COMPARE(s1->state(), QScroller::Inactive);
+
+    QVERIFY(qFuzzyCompare(sw->currentPos.x(), 0));
+    QVERIFY(qFuzzyCompare(sw->currentPos.y(), 500));
+    QCOMPARE(sw->receivedOvershoot, false);
+
     // -- try to scroll with overshoot (always on but max overshoot = 0)
     sp1.setScrollMetric(QScrollerProperties::OvershootDragDistanceFactor, 0.0);
     sp1.setScrollMetric(QScrollerProperties::OvershootScrollDistanceFactor, 0.0);
@@ -537,7 +567,6 @@ void tst_QScroller::mouseEventTimestamp()
     QScroller::grabGesture(sw.data(), QScroller::LeftMouseButtonGesture);
     sw->setGeometry(100, 100, 400, 300);
     sw->show();
-    QApplicationPrivate::setActiveWindow(sw.data());
     if (!QTest::qWaitForWindowExposed(sw.data()) || !QTest::qWaitForWindowActive(sw.data()))
         QSKIP("Failed to show and activate window");
 
