@@ -1,41 +1,15 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
-
-#include <QtTest/QtTest>
 #include <qdebug.h>
 #include <qapplication.h>
 #include <limits.h>
-
 #include <qspinbox.h>
 #include <qlocale.h>
 #include <qlineedit.h>
 #include <qlayout.h>
+
+#include <QTest>
 #include <QSpinBox>
 #include <QWidget>
 #include <QString>
@@ -47,38 +21,45 @@
 #include <QLocale>
 #include <QDoubleSpinBox>
 #include <QVBoxLayout>
-#include <QKeySequence>
+#include <QSignalSpy>
 #include <QStackedWidget>
 #include <QDebug>
 #include <QStyleOptionSpinBox>
 #include <QStyle>
 #include <QProxyStyle>
+#include <QScreen>
+
+#include <QtWidgets/private/qapplication_p.h>
+
+#if QT_CONFIG(shortcut)
+#  include <QKeySequence>
+#endif
 
 class SpinBox : public QSpinBox
 {
 public:
-    SpinBox(QWidget *parent = 0)
+    SpinBox(QWidget *parent = nullptr)
         : QSpinBox(parent)
     {}
-    QString textFromValue(int v) const
+    QString textFromValue(int v) const override
     {
         return QSpinBox::textFromValue(v);
     }
-    QValidator::State validate(QString &text, int &pos) const
+    QValidator::State validate(QString &text, int &pos) const override
     {
         return QSpinBox::validate(text, pos);
     }
-    int valueFromText(const QString &text) const
+    int valueFromText(const QString &text) const override
     {
         return QSpinBox::valueFromText(text);
     }
 #if QT_CONFIG(wheelevent)
-    void wheelEvent(QWheelEvent *event)
+    void wheelEvent(QWheelEvent *event) override
     {
         QSpinBox::wheelEvent(event);
     }
 #endif
-    void initStyleOption(QStyleOptionSpinBox *option) const
+    void initStyleOption(QStyleOptionSpinBox *option) const override
     {
         QSpinBox::initStyleOption(option);
     }
@@ -126,6 +107,28 @@ public:
     Qt::KeyboardModifier stepModifier = Qt::ControlModifier;
 };
 
+class SelectAllOnStepStyle : public QProxyStyle
+{
+public:
+    SelectAllOnStepStyle(bool selectAll)
+    : selectAll(selectAll)
+    {}
+
+    int styleHint(QStyle::StyleHint hint, const QStyleOption *option,
+                  const QWidget *widget, QStyleHintReturn *returnData = nullptr) const override
+    {
+        switch (hint) {
+        case QStyle::SH_SpinBox_SelectOnStep:
+            return selectAll;
+        default:
+            return QProxyStyle::styleHint(hint, option, widget, returnData);
+        }
+    }
+
+private:
+    const bool selectAll;
+};
+
 class tst_QSpinBox : public QObject
 {
     Q_OBJECT
@@ -171,7 +174,10 @@ private slots:
 
     void removeAll();
     void startWithDash();
+
+#if QT_CONFIG(shortcut)
     void undoRedo();
+#endif
 
     void specialValue();
     void textFromValue();
@@ -203,8 +209,12 @@ private slots:
 
     void stepModifierPressAndHold_data();
     void stepModifierPressAndHold();
+
+    void stepSelectAll_data();
+    void stepSelectAll();
+
 public slots:
-    void valueChangedHelper(const QString &);
+    void textChangedHelper(const QString &);
     void valueChangedHelper(int);
 private:
     QStringList actualTexts;
@@ -220,19 +230,19 @@ static QLatin1String modifierToName(Qt::KeyboardModifier modifier)
 {
     switch (modifier) {
     case Qt::NoModifier:
-        return QLatin1Literal("No");
+        return QLatin1String("No");
         break;
     case Qt::ControlModifier:
-        return QLatin1Literal("Ctrl");
+        return QLatin1String("Ctrl");
         break;
     case Qt::ShiftModifier:
-        return QLatin1Literal("Shift");
+        return QLatin1String("Shift");
         break;
     case Qt::AltModifier:
-        return QLatin1Literal("Alt");
+        return QLatin1String("Alt");
         break;
     case Qt::MetaModifier:
-        return QLatin1Literal("Meta");
+        return QLatin1String("Meta");
         break;
     default:
         qFatal("Unexpected keyboard modifier");
@@ -338,11 +348,18 @@ void tst_QSpinBox::getSetCheck()
 
 tst_QSpinBox::tst_QSpinBox()
 {
+    QLocale::setDefault(QLocale::c());
 }
 
 void tst_QSpinBox::init()
 {
-    QLocale::setDefault(QLocale(QLocale::C));
+#if QT_CONFIG(cursor)
+    // Ensure mouse cursor was not left by previous tests where widgets
+    // will appear, as it could cause events and interfere with the tests.
+    const QScreen *screen = QGuiApplication::primaryScreen();
+    const QRect availableGeometry = screen->availableGeometry();
+    QCursor::setPos(availableGeometry.topLeft());
+#endif
 }
 
 void tst_QSpinBox::setValue_data()
@@ -451,20 +468,36 @@ void tst_QSpinBox::setPrefixSuffix()
     QFETCH(QString, expectedCleanText);
     QFETCH(bool, show);
 
-    QSpinBox spin(0);
-    spin.setPrefix(prefix);
-    spin.setSuffix(suffix);
-    spin.setValue(value);
-    if (show)
+    QSpinBox spin;
+    if (show) {
         spin.show();
+        spin.setPrefix(QString());  // trigger a recalc of sizeHint
+    }
+    const QSize size1 = spin.sizeHint();
+    spin.setPrefix(prefix);
+    const QSize size2 = spin.sizeHint();
+    spin.setSuffix(suffix);
+    const QSize size3 = spin.sizeHint();
+    spin.setValue(value);
 
     QCOMPARE(spin.prefix(), prefix);
     QCOMPARE(spin.suffix(), suffix);
     QCOMPARE(spin.text(), expectedText);
     QCOMPARE(spin.cleanText(), expectedCleanText);
+
+    if (!suffix.isEmpty()) {
+        QVERIFY(size2.width() < size3.width());
+        spin.setSuffix(QString());
+        QCOMPARE(spin.sizeHint(), size2);
+    }
+    if (!prefix.isEmpty()) {
+        QVERIFY(size1.width() < size2.width());
+        spin.setPrefix(QString());
+        QCOMPARE(spin.sizeHint(), size1);
+    }
 }
 
-void tst_QSpinBox::valueChangedHelper(const QString &text)
+void tst_QSpinBox::textChangedHelper(const QString &text)
 {
     actualTexts << text;
 }
@@ -474,35 +507,33 @@ void tst_QSpinBox::valueChangedHelper(int value)
     actualValues << value;
 }
 
-class MySpinBox: public QSpinBox
+class ReadOnlyChangeTracker: public QSpinBox
 {
 public:
-    MySpinBox(QWidget *parent = 0) : QSpinBox(parent) {}
+    ReadOnlyChangeTracker(QWidget *parent = nullptr) : QSpinBox(parent) {}
 
-    void changeEvent(QEvent *ev) {
-        eventsReceived.append(ev->type());
+    void changeEvent(QEvent *ev) override
+    {
+        if (ev->type() == QEvent::ReadOnlyChange)
+            ++readOnlyChangeEventCount;
     }
-    QList<QEvent::Type> eventsReceived;
+    int readOnlyChangeEventCount = 0;
 };
 
 void tst_QSpinBox::setReadOnly()
 {
-    MySpinBox spin(0);
+    ReadOnlyChangeTracker spin(0);
     spin.show();
     QTest::keyClick(&spin, Qt::Key_Up);
     QCOMPARE(spin.value(), 1);
     spin.setReadOnly(true);
-#ifndef Q_OS_WINRT // QTBUG-68297
-    QCOMPARE(spin.eventsReceived, QList<QEvent::Type>() << QEvent::ReadOnlyChange);
-#endif
+    QCOMPARE(spin.readOnlyChangeEventCount, 1);
     QTest::keyClick(&spin, Qt::Key_Up);
     QCOMPARE(spin.value(), 1);
     spin.stepBy(1);
     QCOMPARE(spin.value(), 2);
     spin.setReadOnly(false);
-#ifndef Q_OS_WINRT // QTBUG-68297
-    QCOMPARE(spin.eventsReceived, QList<QEvent::Type>() << QEvent::ReadOnlyChange << QEvent::ReadOnlyChange);
-#endif
+    QCOMPARE(spin.readOnlyChangeEventCount, 2);
     QTest::keyClick(&spin, Qt::Key_Up);
     QCOMPARE(spin.value(), 3);
 }
@@ -542,7 +573,7 @@ void tst_QSpinBox::setTracking()
     QSpinBox spin(0);
     spin.setKeyboardTracking(tracking);
     spin.show();
-    connect(&spin, SIGNAL(valueChanged(QString)), this, SLOT(valueChangedHelper(QString)));
+    connect(&spin, &QSpinBox::textChanged, this, &tst_QSpinBox::textChangedHelper);
 
     keys.simulate(&spin);
     QCOMPARE(actualTexts, texts);
@@ -850,7 +881,7 @@ void tst_QSpinBox::locale_data()
     QTest::addColumn<QString>("text");
     QTest::addColumn<int>("valFromText");
 
-    QTest::newRow("data0") << QLocale(QLocale::Norwegian, QLocale::Norway) << 1234 << QString("1234") << QString("2345") << 2345;
+    QTest::newRow("data0") << QLocale(QLocale::NorwegianBokmal, QLocale::Norway) << 1234 << QString("1234") << QString("2345") << 2345;
     QTest::newRow("data1") << QLocale(QLocale::German, QLocale::Germany) << 1234 << QString("1234") << QString("2345") << 2345;
 }
 
@@ -862,8 +893,9 @@ void tst_QSpinBox::locale()
     QFETCH(QString, text);
     QFETCH(int, valFromText);
 
-    QLocale old;
-
+    const auto restoreDefault = qScopeGuard([prior = QLocale()]() {
+        QLocale::setDefault(prior);
+    });
     QLocale::setDefault(loc);
     SpinBox box;
     box.setMaximum(100000);
@@ -880,6 +912,9 @@ void tst_QSpinBox::locale()
 
 void tst_QSpinBox::editingFinished()
 {
+    if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
+        QSKIP("Wayland: This fails. Figure out why.");
+
     QWidget testFocusWidget;
     testFocusWidget.setObjectName(QLatin1String("tst_qspinbox"));
     testFocusWidget.setWindowTitle(objectName());
@@ -892,7 +927,6 @@ void tst_QSpinBox::editingFinished()
     layout->addWidget(box2);
 
     testFocusWidget.show();
-    QApplication::setActiveWindow(&testFocusWidget);
     QVERIFY(QTest::qWaitForWindowActive(&testFocusWidget));
     box->activateWindow();
     box->setFocus();
@@ -906,45 +940,45 @@ void tst_QSpinBox::editingFinished()
     QTest::keyClick(box, Qt::Key_Up);
     QTest::keyClick(box, Qt::Key_Up);
 
-    QCOMPARE(editingFinishedSpy1.count(), 0);
-    QCOMPARE(editingFinishedSpy2.count(), 0);
+    QCOMPARE(editingFinishedSpy1.size(), 0);
+    QCOMPARE(editingFinishedSpy2.size(), 0);
 
     QTest::keyClick(box2, Qt::Key_Up);
     QTest::keyClick(box2, Qt::Key_Up);
     box2->setFocus();
-    QCOMPARE(editingFinishedSpy1.count(), 1);
+    QCOMPARE(editingFinishedSpy1.size(), 1);
     box->setFocus();
-    QCOMPARE(editingFinishedSpy1.count(), 1);
-    QCOMPARE(editingFinishedSpy2.count(), 1);
+    QCOMPARE(editingFinishedSpy1.size(), 1);
+    QCOMPARE(editingFinishedSpy2.size(), 1);
     QTest::keyClick(box, Qt::Key_Up);
-    QCOMPARE(editingFinishedSpy1.count(), 1);
-    QCOMPARE(editingFinishedSpy2.count(), 1);
+    QCOMPARE(editingFinishedSpy1.size(), 1);
+    QCOMPARE(editingFinishedSpy2.size(), 1);
     QTest::keyClick(box, Qt::Key_Enter);
-    QCOMPARE(editingFinishedSpy1.count(), 2);
-    QCOMPARE(editingFinishedSpy2.count(), 1);
+    QCOMPARE(editingFinishedSpy1.size(), 2);
+    QCOMPARE(editingFinishedSpy2.size(), 1);
     QTest::keyClick(box, Qt::Key_Return);
-    QCOMPARE(editingFinishedSpy1.count(), 3);
-    QCOMPARE(editingFinishedSpy2.count(), 1);
+    QCOMPARE(editingFinishedSpy1.size(), 3);
+    QCOMPARE(editingFinishedSpy2.size(), 1);
     box2->setFocus();
-    QCOMPARE(editingFinishedSpy1.count(), 4);
-    QCOMPARE(editingFinishedSpy2.count(), 1);
+    QCOMPARE(editingFinishedSpy1.size(), 4);
+    QCOMPARE(editingFinishedSpy2.size(), 1);
     QTest::keyClick(box2, Qt::Key_Enter);
-    QCOMPARE(editingFinishedSpy1.count(), 4);
-    QCOMPARE(editingFinishedSpy2.count(), 2);
+    QCOMPARE(editingFinishedSpy1.size(), 4);
+    QCOMPARE(editingFinishedSpy2.size(), 2);
     QTest::keyClick(box2, Qt::Key_Return);
-    QCOMPARE(editingFinishedSpy1.count(), 4);
-    QCOMPARE(editingFinishedSpy2.count(), 3);
+    QCOMPARE(editingFinishedSpy1.size(), 4);
+    QCOMPARE(editingFinishedSpy2.size(), 3);
 
     testFocusWidget.hide();
-    QCOMPARE(editingFinishedSpy1.count(), 4);
-    QCOMPARE(editingFinishedSpy2.count(), 4);
+    QCOMPARE(editingFinishedSpy1.size(), 4);
+    QCOMPARE(editingFinishedSpy2.size(), 4);
 
     //task203285
     editingFinishedSpy1.clear();
     testFocusWidget.show();
     QVERIFY(QTest::qWaitForWindowActive(&testFocusWidget));
     box->setKeyboardTracking(false);
-    qApp->setActiveWindow(&testFocusWidget);
+    QApplicationPrivate::setActiveWindow(&testFocusWidget);
     testFocusWidget.activateWindow();
     box->setFocus();
     QTRY_VERIFY(box->hasFocus());
@@ -954,7 +988,7 @@ void tst_QSpinBox::editingFinished()
     box2->setFocus();
     QTRY_VERIFY(qApp->focusWidget() != box);
     QCOMPARE(box->text(), QLatin1String("20"));
-    QCOMPARE(editingFinishedSpy1.count(), 1);
+    QCOMPARE(editingFinishedSpy1.size(), 1);
 }
 
 void tst_QSpinBox::removeAll()
@@ -995,6 +1029,8 @@ void tst_QSpinBox::startWithDash()
     QCOMPARE(spin.text(), QString("0"));
 }
 
+#if QT_CONFIG(shortcut)
+
 void tst_QSpinBox::undoRedo()
 {
     //test undo/redo feature (in conjunction with the "undoRedoEnabled" property)
@@ -1013,7 +1049,7 @@ void tst_QSpinBox::undoRedo()
     QVERIFY(spin.lineEdit()->isUndoAvailable());
 
     //testing CTRL+Z (undo)
-    int val = QKeySequence(QKeySequence::Undo)[0];
+    int val = QKeySequence(QKeySequence::Undo)[0].toCombined();
     Qt::KeyboardModifiers mods = (Qt::KeyboardModifiers)(val & Qt::KeyboardModifierMask);
     QTest::keyClick(&spin, val & ~mods, mods);
 
@@ -1022,7 +1058,7 @@ void tst_QSpinBox::undoRedo()
     QVERIFY(spin.lineEdit()->isRedoAvailable());
 
     //testing CTRL+Y (redo)
-    val = QKeySequence(QKeySequence::Redo)[0];
+    val = QKeySequence(QKeySequence::Redo)[0].toCombined();
     mods = (Qt::KeyboardModifiers)(val & Qt::KeyboardModifierMask);
     QTest::keyClick(&spin, val & ~mods, mods);
     QCOMPARE(spin.value(), 1);
@@ -1047,8 +1083,13 @@ void tst_QSpinBox::undoRedo()
     QVERIFY(!spin.lineEdit()->isRedoAvailable());
 }
 
+#endif // QT_CONFIG(shortcut)
+
 void tst_QSpinBox::specialValue()
 {
+    if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
+        QSKIP("Wayland: This fails. Figure out why.");
+
     QString specialText="foo";
 
     QWidget topWidget;
@@ -1064,7 +1105,6 @@ void tst_QSpinBox::specialValue()
     spin.setValue(50);
     topWidget.show();
     //make sure we have the focus (even if editingFinished fails)
-    qApp->setActiveWindow(&topWidget);
     topWidget.activateWindow();
     QVERIFY(QTest::qWaitForWindowActive(&topWidget));
     spin.setFocus();
@@ -1100,7 +1140,7 @@ void tst_QSpinBox::textFromValue()
 class sizeHint_SpinBox : public QSpinBox
 {
 public:
-    QSize sizeHint() const
+    QSize sizeHint() const override
     {
         ++sizeHintRequests;
         return QSpinBox::sizeHint();
@@ -1110,37 +1150,39 @@ public:
 
 void tst_QSpinBox::sizeHint()
 {
-    QWidget *widget = new QWidget;
-    QHBoxLayout *layout = new QHBoxLayout(widget);
+    QWidget widget;
+    QHBoxLayout *layout = new QHBoxLayout(&widget);
+
     sizeHint_SpinBox *spinBox = new sizeHint_SpinBox;
     layout->addWidget(spinBox);
-    widget->show();
-    QVERIFY(QTest::qWaitForWindowExposed(widget));
+    // Make sure all layout requests posted by the QHBoxLayout constructor and addWidget
+    // are processed before the widget is shown
+    QCoreApplication::sendPostedEvents(&widget, QEvent::LayoutRequest);
+    widget.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&widget));
 
     // Prefix
     spinBox->sizeHintRequests = 0;
     spinBox->setPrefix(QLatin1String("abcdefghij"));
-    qApp->processEvents();
     QTRY_VERIFY(spinBox->sizeHintRequests > 0);
 
     // Suffix
     spinBox->sizeHintRequests = 0;
     spinBox->setSuffix(QLatin1String("abcdefghij"));
-    qApp->processEvents();
     QTRY_VERIFY(spinBox->sizeHintRequests > 0);
 
     // Range
     spinBox->sizeHintRequests = 0;
     spinBox->setRange(0, 1234567890);
     spinBox->setValue(spinBox->maximum());
-    qApp->processEvents();
     QTRY_VERIFY(spinBox->sizeHintRequests > 0);
-
-    delete widget;
 }
 
 void tst_QSpinBox::taskQTBUG_5008_textFromValueAndValidate()
 {
+    if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
+        QSKIP("Wayland: This fails. Figure out why.");
+
     class DecoratedSpinBox : public QSpinBox
     {
     public:
@@ -1157,7 +1199,7 @@ void tst_QSpinBox::taskQTBUG_5008_textFromValueAndValidate()
         }
 
         //we use the French delimiters here
-        QString textFromValue (int value) const
+        QString textFromValue (int value) const override
         {
             return locale().toString(value);
         }
@@ -1166,7 +1208,6 @@ void tst_QSpinBox::taskQTBUG_5008_textFromValueAndValidate()
     spinbox.show();
     spinbox.activateWindow();
     spinbox.setFocus();
-    QApplication::setActiveWindow(&spinbox);
     QVERIFY(QTest::qWaitForWindowActive(&spinbox));
     QVERIFY(spinbox.hasFocus());
     QTRY_COMPARE(static_cast<QWidget *>(&spinbox), QApplication::activeWindow());
@@ -1214,11 +1255,14 @@ void tst_QSpinBox::lineEditReturnPressed()
     QSignalSpy spyCurrentChanged(spinBox.lineEdit(), SIGNAL(returnPressed()));
     spinBox.show();
     QTest::keyClick(&spinBox, Qt::Key_Return);
-    QCOMPARE(spyCurrentChanged.count(), 1);
+    QCOMPARE(spyCurrentChanged.size(), 1);
 }
 
 void tst_QSpinBox::positiveSign()
 {
+    if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
+        QSKIP("Wayland: This fails. Figure out why.");
+
     QSpinBox spinBox;
     spinBox.setRange(-20, 20);
     spinBox.setValue(-20);
@@ -1234,6 +1278,9 @@ void tst_QSpinBox::positiveSign()
 
 void tst_QSpinBox::interpretOnLosingFocus()
 {
+    if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
+        QSKIP("Wayland: This fails. Figure out why.");
+
     // QTBUG-55249: When typing an invalid value after QSpinBox::clear(),
     // it should be fixed up on losing focus.
 
@@ -1262,7 +1309,7 @@ void tst_QSpinBox::interpretOnLosingFocus()
 void tst_QSpinBox::setGroupSeparatorShown_data()
 {
     QTest::addColumn<QLocale::Language>("lang");
-    QTest::addColumn<QLocale::Country>("country");
+    QTest::addColumn<QLocale::Territory>("country");
 
     QTest::newRow("data0") << QLocale::English << QLocale::UnitedStates;
     QTest::newRow("data1") << QLocale::Swedish << QLocale::Sweden;
@@ -1274,8 +1321,11 @@ void tst_QSpinBox::setGroupSeparatorShown_data()
 void tst_QSpinBox::setGroupSeparatorShown()
 {
     QFETCH(QLocale::Language, lang);
-    QFETCH(QLocale::Country, country);
+    QFETCH(QLocale::Territory, country);
 
+    const auto restoreDefault = qScopeGuard([prior = QLocale()]() {
+        QLocale::setDefault(prior);
+    });
     QLocale loc(lang, country);
     QLocale::setDefault(loc);
     SpinBox spinBox;
@@ -1305,7 +1355,6 @@ void tst_QSpinBox::wheelEvents_data()
 {
 #if QT_CONFIG(wheelevent)
     QTest::addColumn<QPoint>("angleDelta");
-    QTest::addColumn<int>("qt4Delta");
     QTest::addColumn<int>("stepModifier");
     QTest::addColumn<Qt::KeyboardModifiers>("modifier");
     QTest::addColumn<Qt::MouseEventSource>("source");
@@ -1376,16 +1425,16 @@ void tst_QSpinBox::wheelEvents_data()
                         QLatin1String sourceName;
                         switch (source) {
                         case Qt::MouseEventNotSynthesized:
-                            sourceName = QLatin1Literal("NotSynthesized");
+                            sourceName = QLatin1String("NotSynthesized");
                             break;
                         case Qt::MouseEventSynthesizedBySystem:
-                            sourceName = QLatin1Literal("SynthesizedBySystem");
+                            sourceName = QLatin1String("SynthesizedBySystem");
                             break;
                         case Qt::MouseEventSynthesizedByQt:
-                            sourceName = QLatin1Literal("SynthesizedByQt");
+                            sourceName = QLatin1String("SynthesizedByQt");
                             break;
                         case Qt::MouseEventSynthesizedByApplication:
-                            sourceName = QLatin1Literal("SynthesizedByApplication");
+                            sourceName = QLatin1String("SynthesizedByApplication");
                             break;
                         default:
                             qFatal("Unexpected wheel event source");
@@ -1404,7 +1453,6 @@ void tst_QSpinBox::wheelEvents_data()
                                       modifierName.latin1(),
                                       sourceName.latin1())
                                 << angleDelta
-                                << units
                                 << static_cast<int>(stepModifier)
                                 << modifiers
                                 << source
@@ -1424,7 +1472,6 @@ void tst_QSpinBox::wheelEvents()
 {
 #if QT_CONFIG(wheelevent)
     QFETCH(QPoint, angleDelta);
-    QFETCH(int, qt4Delta);
     QFETCH(int, stepModifier);
     QFETCH(Qt::KeyboardModifiers, modifier);
     QFETCH(Qt::MouseEventSource, source);
@@ -1440,9 +1487,8 @@ void tst_QSpinBox::wheelEvents()
     style->stepModifier = static_cast<Qt::KeyboardModifier>(stepModifier);
     spinBox.setStyle(style.data());
 
-    QWheelEvent event(QPointF(), QPointF(), QPoint(), angleDelta, qt4Delta,
-                      Qt::Vertical, Qt::NoButton, modifier, Qt::NoScrollPhase,
-                      source);
+    QWheelEvent event(QPointF(), QPointF(), QPoint(), angleDelta,
+                      Qt::NoButton, modifier, Qt::NoScrollPhase, false, source);
     for (int expected : expectedValues) {
         qApp->sendEvent(&spinBox, &event);
         QCOMPARE(spinBox.value(), expected);
@@ -1592,6 +1638,9 @@ void tst_QSpinBox::stepModifierKeys_data()
 
 void tst_QSpinBox::stepModifierKeys()
 {
+    if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
+        QSKIP("Wayland: This fails. Figure out why.");
+
     QFETCH(int, startValue);
     QFETCH(int, stepModifier);
     QFETCH(QTestEventList, keys);
@@ -1675,6 +1724,9 @@ void tst_QSpinBox::stepModifierButtons_data()
 
 void tst_QSpinBox::stepModifierButtons()
 {
+    if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
+        QSKIP("Wayland: This fails. Figure out why.");
+
     QFETCH(QStyle::SubControl, subControl);
     QFETCH(int, stepModifier);
     QFETCH(Qt::KeyboardModifiers, modifiers);
@@ -1760,6 +1812,9 @@ void tst_QSpinBox::stepModifierPressAndHold_data()
 
 void tst_QSpinBox::stepModifierPressAndHold()
 {
+    if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
+        QSKIP("Wayland: This fails. Figure out why.");
+
     QFETCH(QStyle::SubControl, subControl);
     QFETCH(int, stepModifier);
     QFETCH(Qt::KeyboardModifiers, modifiers);
@@ -1774,11 +1829,7 @@ void tst_QSpinBox::stepModifierPressAndHold()
     stepModifierStyle->stepModifier = static_cast<Qt::KeyboardModifier>(stepModifier);
     spin.setStyle(stepModifierStyle.data());
 
-    QSignalSpy spy(&spin, QOverload<int>::of(&SpinBox::valueChanged));
-    // TODO: remove debug output when QTBUG-69492 is fixed
-    connect(&spin, QOverload<int>::of(&SpinBox::valueChanged), [=]() {
-        qDebug() << QTime::currentTime() << "valueChanged emitted";
-    });
+    QSignalSpy spy(&spin, &SpinBox::valueChanged);
 
     spin.show();
     QVERIFY(QTest::qWaitForWindowActive(&spin));
@@ -1793,13 +1844,50 @@ void tst_QSpinBox::stepModifierPressAndHold()
     qDebug() << "QGuiApplication::focusWindow():" << QGuiApplication::focusWindow();
     qDebug() << "QGuiApplication::topLevelWindows():" << QGuiApplication::topLevelWindows();
     QTest::mousePress(&spin, Qt::LeftButton, modifiers, buttonRect.center());
-    QTRY_VERIFY2(spy.length() >= 3, qPrintable(QString::fromLatin1(
-        "Expected valueChanged() to be emitted 3 or more times, but it was only emitted %1 times").arg(spy.length())));
+    QTRY_VERIFY2(spy.size() >= 3, qPrintable(QString::fromLatin1(
+        "Expected valueChanged() to be emitted 3 or more times, but it was only emitted %1 times").arg(spy.size())));
     QTest::mouseRelease(&spin, Qt::LeftButton, modifiers, buttonRect.center());
 
     const auto value = spy.last().at(0);
-    QVERIFY(value.type() == QVariant::Int);
-    QCOMPARE(value.toInt(), spy.length() * expectedStepModifier);
+    QVERIFY(value.metaType().id() == QMetaType::Int);
+    QCOMPARE(value.toInt(), spy.size() * expectedStepModifier);
+}
+
+void tst_QSpinBox::stepSelectAll_data()
+{
+    QTest::addColumn<bool>("stepShouldSelectAll");
+    QTest::addColumn<QStringList>("selectedText");
+
+    QTest::addRow("select all") << true << QStringList{"1", "0", "5", "4", "9"};
+    QTest::addRow("don't select all") << false << QStringList{{}, {}, {}, {}, "94"};
+}
+
+void tst_QSpinBox::stepSelectAll()
+{
+    QFETCH(bool, stepShouldSelectAll);
+    QFETCH(QStringList, selectedText);
+    SelectAllOnStepStyle style(stepShouldSelectAll);
+
+    SpinBox spinBox;
+    spinBox.setStyle(&style);
+
+    QCOMPARE(spinBox.lineEdit()->selectedText(), QString());
+
+    auto it = selectedText.cbegin();
+    spinBox.stepUp();
+    QCOMPARE(spinBox.lineEdit()->selectedText(), *(it++));
+    spinBox.lineEdit()->deselect();
+    spinBox.stepDown();
+    QCOMPARE(spinBox.lineEdit()->selectedText(), *(it++));
+    spinBox.lineEdit()->deselect();
+    spinBox.stepBy(5);
+    QCOMPARE(spinBox.lineEdit()->selectedText(), *(it++));
+    spinBox.lineEdit()->deselect();
+    QTest::keyClick(&spinBox, Qt::Key_Down);
+    QCOMPARE(spinBox.lineEdit()->selectedText(), *(it++));
+    QTest::keyClicks(&spinBox, "9");
+    QCOMPARE(spinBox.lineEdit()->selectedText(), QString());
+    QCOMPARE(spinBox.lineEdit()->text(), *(it++));
 }
 
 QTEST_MAIN(tst_QSpinBox)
